/** @file msxe370x.c
*
*
* @author S.Nahas
* @date Feb 3, 2009
*
* @version  $LastChangedRevision: 3021 $
* @version $LastChangedDate: 2008-06-11 10:16:10 +0200 (Wed, 11 Jun 2008) $
*
*
*/

#ifndef _THREAD_SAFE
#define _THREAD_SAFE
#endif

#include <stdio.h>
#include <time.h>
#include <string.h>

#include <msxe370x.h>
#include <addidata_console.h>
#include <addidata_dataclient.h>


#ifdef WIN32
#include <stdint_compat.h>
#else
#include <stdint.h>
#endif

//---------------------------------------------------------------------------------
/* returns the actual number of channel the ChannelMask defines */
static __INLINE__ unsigned int msxe370x_autorefresh_compute_channel_number(uint32_t ChannelMask, uint32_t number_of_channel)
{
	unsigned int i;
	unsigned int fields = 0;

	for (i = 0; i < number_of_channel; i++)
	{
		if (ChannelMask & (1<<i))
			fields++;
	}
	return fields;
}
//---------------------------------------------------------------------------------
/* set 1 in ChannelList if the corresponding channel is set in ChannelMask, up to number_of_channel */
static __INLINE__ unsigned int msxe370x_autorefresh_compute_channel_list(uint32_t ChannelMask,uint32_t ChannelList[16], uint32_t number_of_channel)
{
	unsigned int i;
	unsigned int c=0; /* counter for ChannelList */

	memset(ChannelList, 0, sizeof(ChannelList));
	for (i = 0; i < number_of_channel; i++)
	{
		if (ChannelMask & (1<<i))
		{
			ChannelList[c] = i;
			c++;
		}
	}
	return 0;
}
//---------------------------------------------------------------------------------
unsigned int msxe370x_autorefresh_compute_fields(struct msxe370x_autorefresh_parameters* params)
{
	return ( params->NbrOfChannel + 1 /* counter */ + (msxe370x_autorefresh_format_has_timestamp(params)*2));
}
//---------------------------------------------------------------------------------
static unsigned int compute_channelmask(uint32_t number_of_channel)
{
	unsigned int i;
	unsigned int ulMask=0;
	for (i=0; i<number_of_channel;i++)
	{
		ulMask = ulMask | (1 << i);
	}
	return ulMask;
}
//---------------------------------------------------------------------------------
void msxe370x_ask_autorefresh_format(struct msxe370x_autorefresh_parameters* params)
{
	uint32_t dw_Choice = 0;
	// Test if data server used
	{
		dw_Choice = addidata_console_get_yn("\nDo you want to receive a time stamp?");
		params->DataFormat = dw_Choice;
	}
	{
		dw_Choice = addidata_console_get_unsigned_integer("\nDo you want to receive the value 0: as digital value 1: as analog value (mm)?", 0,1);
		params->DataFormat = params->DataFormat | (dw_Choice << 1);
	}
	{
		dw_Choice = addidata_console_get_yn("\nDo you want to invert the value sign?");
		params->DataFormat = params->DataFormat | (dw_Choice << 2);
	}
}
//---------------------------------------------------------------------------------
void msxe370x_ask_autorefresh_parameters(struct msxe370x_autorefresh_parameters* params, uint32_t number_of_channel)
{
	uint32_t dw_Choice = 0;

	params->NbrOfChannel				= 0;
	params->ChannelMask					= 0;
	params->AverageMode					= 0;
	params->AverageValue				= 0;
	params->DivisionFactor				= 0;
	params->TriggerAction				= 0;
	params->HardwareTriggerCount		= 0;
	params->HardwareTriggerFilterTime	= 0;
	params->ByTriggerNbrOfSeqToAcquire	= 0;
	params->DataFormat					= 0;

	{
		params->ChannelMask = addidata_console_get_unsigned_hexa("\nEnter the channel mask (in Hex)\ne.g: 1=CH0, 2=CH1, 3=CH0 and CH1, 4=CH2, 7=CH0 and CH1 and CH2: ", 0, compute_channelmask(number_of_channel));
		params->NbrOfChannel = msxe370x_autorefresh_compute_channel_number(params->ChannelMask, number_of_channel);
		msxe370x_autorefresh_compute_channel_list(params->ChannelMask,params->ChannelList, number_of_channel);
	}

	dw_Choice = addidata_console_get_yn("\nDo you want to activate the average mode?");

	if (dw_Choice == 1)
	{

		{
			printf("\n1. Average mode per sequence\n");
			printf("2. Average mode per channel\n");
			printf("Select the Average mode do you want to use: ");
			params->AverageMode = addidata_console_get_unsigned_integer(NULL, 1,2);
		}

		params->AverageValue=addidata_console_get_unsigned_integer("\nEnter the average value (2 to 255): ", 2,255);

	}

	params->DivisionFactor=addidata_console_get_unsigned_integer("\nEnter the division factor (5 to 255): ", 5,255);

	// trigger start
	dw_Choice = addidata_console_get_yn("\nDo you want to use the hardware trigger to start the acquisition?");

	if (dw_Choice == 1)
	{
		{
			printf("\n1. One shot trigger\n");
			printf("2. Sequence trigger\n");
			printf("Select the trigger mode do you want to use: ");
			params->TriggerAction=addidata_console_get_unsigned_integer(NULL, 1,2);
		}

		{
			printf("\n1. rising edge\n");
			printf("2. falling edge\n");
			printf("3. both edges\n");
			printf("Select the active edge: ");
			dw_Choice = addidata_console_get_unsigned_integer(NULL, 1,3);
		}

		params->TriggerAction = params->TriggerAction | (dw_Choice << 6);

		params->HardwareTriggerCount = addidata_console_get_unsigned_integer("\nEnter the number of edges before the trigger action occure: ", 1,0xFFFF);

		params->HardwareTriggerFilterTime=addidata_console_get_unsigned_integer("\nEnter the filter time (multiplier from 250 ns step) for the input trigger (0 = disable, max : 65535 -> 16,383750 ms): ", 0,0xFFFF);
	}
	else
	{
		dw_Choice = addidata_console_get_yn("\nDo you want to use the synchronisation trigger to start the acquisition?");

		if (dw_Choice == 1)
		{
			{
				printf("\n1. One shot trigger\n");
				printf("2. Sequence trigger\n");
				printf("Select the trigger mode do you want to use: ");
				dw_Choice = addidata_console_get_unsigned_integer(NULL, 1,2);
			}

			params->TriggerAction = dw_Choice << 8;
		}
	}

	if (((params->TriggerAction & 2) == 0x2) || ((params->TriggerAction & 0x200) == 0x200))
	{
		params->ByTriggerNbrOfSeqToAcquire = addidata_console_get_unsigned_integer("\nEnter the number of sequence to acquire by each trigger: ", 0,0xFFFFFFFF);
	}

	//	Trigger Stop
	if ((params->TriggerAction == 0) || ((params->TriggerAction & 0x101) != 0))
	{
		dw_Choice = addidata_console_get_yn("\nDo you want to use the hardware trigger to stop the acquisition?");

		if (dw_Choice == 1)
		{
			params->TriggerAction = params->TriggerAction | (1 << 16);

			{
				printf("\n1. rising edge\n");
				printf("2. falling edge\n");
				printf("3. both edges\n");
				printf("Select the active edge: ");
				dw_Choice = addidata_console_get_unsigned_integer(NULL, 1,3);
			}

			params->TriggerAction = params->TriggerAction | (dw_Choice << 17);

			dw_Choice = addidata_console_get_yn("\nDo you want to use the number of edge before the trigger occur?");
			if (dw_Choice == 1)
			{
				params->TriggerAction = params->TriggerAction | (1 << 19);

				if (params->HardwareTriggerCount == 0)
				{
					params->HardwareTriggerCount = addidata_console_get_unsigned_integer("\nEnter the number of edges before the trigger action occur (1 to 65535): ", 1,0xFFFF);
				}
			}

		}
	}
}
//---------------------------------------------------------------------------------
#define RETIFNEG(__arg__) { ret = __arg__ ; if (ret<0) return -1; }
#define NEXT(_step) { count = count - _step; where += _step; }
#define RETIFCOUNT0() { if (count==0) return total; }
#define RETIFCOUNTINF2() { if (count<0) return total; }
//---------------------------------------------------------------------------------
static int fprintf_onedata_integer(FILE* stream, const char* fmt, uint32_t data, const char* datasepfmt)
{
	int total = 0; /* total number of bytes printed */
	int ret;
	RETIFNEG( fprintf(stream,fmt,data) );
	total += ret;
	if(datasepfmt)
	{
		RETIFNEG( fprintf(stream,datasepfmt) );
	}
	return (total + ret);
}
//---------------------------------------------------------------------------------
static int fprintf_onedata_float(FILE* stream, const char* fmt, uint32_t data, const char* datasepfmt)
{
	int total = 0; /* total number of bytes printed */
	int ret;

	/* the double cast below force the compiler to not do conversion */
	RETIFNEG( fprintf(stream,fmt,*((float*) (void*) &data)) );
	total += ret;
	if(datasepfmt)
	{
		RETIFNEG( fprintf(stream,datasepfmt) );
	}
	return (total + ret);
}
//---------------------------------------------------------------------------------
/** @brief format and output an array of data
 *
 * @param stream FILE* to write output in
 * @param NbrOfChannel how much channels are to be output
 * @param dataheaderfmt if not null, is prepended before data itself; printf-like format, takes the channel number as parameter
 * @param fmt printf-like format to use for output
 * @param ChannelList array of index of acquired channels
 * @param data data to be converted and printed
 * @param count size of "data" array
 * @param datasepfmt a printf-like format to be used between data (takes no parameter)
 * @param dataformatfcn function to be used to perform actual output
 *
 */

static int fprintf_data(FILE* stream, uint32_t NbrOfChannel, const char* fmt, uint32_t ChannelList[16], uint32_t data[], unsigned int count, struct msxe370x_format *format, int (*dataformatfcn) (FILE* stream, const char* fmt, uint32_t data, const char* datasepfmt))
{
	int total = 0; /* total number of bytes printed */
	unsigned int i;
	int ret;

	for(i=0; i<(NbrOfChannel-1); i++)
	{
		if (i>=count) /* stop parsing if end of array */
			return total;
		/* data header */
		{
			RETIFNEG( fprintf(stream, format->dataheader,ChannelList[i]));
			total += ret;
		}
		RETIFNEG( dataformatfcn(stream, fmt, data[i], format->datasep) );
		total += ret;
	}
	/* data header */
	{
		RETIFNEG( fprintf(stream, format->dataheader,ChannelList[NbrOfChannel-1]));
		total += ret;
	}
	RETIFNEG( dataformatfcn(stream, fmt, data[NbrOfChannel-1], NULL) );

	return (total+ret);
}
//---------------------------------------------------------------------------------
int msxe370x_autorefresh_fprintf(FILE* stream, struct msxe370x_autorefresh_parameters* params, uint32_t data[], unsigned int count, struct msxe370x_format *format)
{
	long int total = 0; /* total number of bytes printed */
	unsigned int where = 0; /* position of next data */
	const char* datafmt = NULL;
	int (*fcn) (FILE* stream, const char* fmt, uint32_t data, const char* datasepfmt);

	if (msxe370x_autorefresh_format_is_analog(params))
	{
		datafmt = format->analog;
		fcn = fprintf_onedata_float;
	}
	else
	{
		datafmt = format->digital;
		fcn = fprintf_onedata_integer;
	}

	RETIFCOUNT0();
	/* timestamp? */
	if (msxe370x_autorefresh_format_has_timestamp(params))
	{
		int ret;
		RETIFCOUNTINF2();
		RETIFNEG( addidata_fprintf_timestamp(stream, format->datasep, data[0], data[1]) );
		total += ret;
		NEXT(2);
	}
	/* counter */
	{
		int ret;
		uint32_t tmpval;
		RETIFCOUNT0();
		tmpval = data[2];
		/* counter */
		RETIFNEG( fprintf(stream,format->counter,data[where]) );
		total += ret;
		NEXT(1);
	}
	/* data */
	{
		int ret;
		RETIFCOUNT0();
		/* data */
		RETIFNEG( fprintf_data(stream, params->NbrOfChannel, datafmt, params->ChannelList, data+where, count, format, fcn) );
		NEXT(params->NbrOfChannel);
		total += ret;
	}
	/* record terminator */
	{
		int ret;
		RETIFNEG( fprintf(stream,format->recordterm) );
		total += ret;
	}
	return total;
}
//---------------------------------------------------------------------------------
int msxe370x_getautorefreshvalues_fprintf(FILE* stream, struct msxe370x_autorefresh_parameters* params, uint32_t data[], unsigned int count, struct msxe370x_format *format)
{
	long int total = 0; /* total number of bytes printed */
	unsigned int where = 0; /* position of next data */
	const char* datafmt = NULL;
	int (*fcn) (FILE* stream, const char* fmt, uint32_t data, const char* datasepfmt);

	if (msxe370x_autorefresh_format_is_analog(params))
	{
		datafmt = format->analog;
		fcn = fprintf_onedata_float;
	}
	else
	{
		datafmt = format->digital;
		fcn = fprintf_onedata_integer;
	}

	RETIFCOUNT0();
	/* counter */
	{
		int ret;
		RETIFCOUNT0();
		/* counter */
		RETIFNEG( fprintf(stream,format->counter,data[0]) );
		total += ret;
		NEXT(1);
	}
	/* data */
	{
		int ret;
		RETIFCOUNT0();
		/* data */
		RETIFNEG( fprintf_data(stream, params->NbrOfChannel, datafmt, params->ChannelList, data+where, count, format, fcn) );
		NEXT(params->NbrOfChannel);
		total += ret;
	}
	/* record terminator */
	{
		int ret;
		RETIFNEG( fprintf(stream,format->recordterm) );
		total += ret;
	}
	return total;
}
//---------------------------------------------------------------------------------
int msxe370x_autorefresh_printf(struct msxe370x_autorefresh_parameters* params, uint32_t data[], unsigned int count, struct msxe370x_format *format)
{
	return msxe370x_autorefresh_fprintf(stdout, params, data, count, format);
}
//---------------------------------------------------------------------------------
int msxe370x_getautorefreshvalues_printf(struct msxe370x_autorefresh_parameters* params, uint32_t data[], unsigned int count, struct msxe370x_format *format)
{
	return msxe370x_getautorefreshvalues_fprintf(stdout, params, data, count, format);
}
//---------------------------------------------------------------------------------
void msxe370x_ask_sequence_format(struct msxe370x_sequence_parameters* params)
{
	unsigned int dw_Choice = 0;
	// Data format
	params->DataFormat = 0;

	{
		params->DataFormat = addidata_console_get_yn("\nDo you want to receive an absolute time stamp?");
	}

	if (params->DataFormat == 0)
	{
		dw_Choice = addidata_console_get_yn("\nDo you want to receive a relative time stamp?");
		params->DataFormat = dw_Choice << 4;
	}
	{
		dw_Choice = addidata_console_get_yn("\nDo you want to receive a Sequence counter?");
		params->DataFormat = params->DataFormat | (dw_Choice << 1);
	}
	{
		dw_Choice = addidata_console_get_unsigned_integer("\nDo you want to receive the value 0: as digital value 1: as analog value (mm)?", 0,1);
		params->DataFormat = params->DataFormat | (dw_Choice << 2);
	}
	{
		dw_Choice = addidata_console_get_yn("\nDo you want to invert the value sign?");
		params->DataFormat = params->DataFormat | (dw_Choice << 3);
	}
	{
		dw_Choice = addidata_console_get_yn("\nDo you want to receive the hardware trigger input information?");
		params->DataFormat = params->DataFormat | (dw_Choice << 5);
	}
}
//---------------------------------------------------------------------------------
void msxe370x_ask_sequence_parameters(struct msxe370x_sequence_parameters* params, uint32_t number_of_channel)
{
	unsigned int i;
	unsigned int dw_Choice = 0;

	params->TriggerAction              = 0;
	params->HardwareTriggerCount       = 0;
	params->HardwareTriggerFilterTime  = 0;
	params->ByTriggerNbrOfSeqToAcquire = 0;
	params->DelayTimeUnit              = 0;
	params->DelayValue                  = 0;
	params->DataFormat                 = 0;


	params->NbrOfChannel = addidata_console_get_unsigned_integer("\nEnter the number of channels in sequence: ", 1,number_of_channel);

	for (i = 0; i < params->NbrOfChannel; i++)
	{
		{
			printf("\nEnter channel of sequence index %d: ", i);
			params->ChannelList[i] = addidata_console_get_unsigned_integer(NULL, 0,number_of_channel-1);
		}
	}

	params->DivisionFactor = addidata_console_get_unsigned_integer("\nEnter the division factor (5 to 255): ", 5,255);
	params->NbrOfSequence = addidata_console_get_unsigned_integer("\nEnter the number of sequences to acquire (0 = continuous): ", 0,0xFFFF);
	params->NbrMaxSequenceToTransfer = addidata_console_get_unsigned_integer("\nEnter the maximum number of sequence to acquire before a data transfer: ", 1,0xFFFF);

	//	Delay
	{
		printf("0. Delay not used\n");
		printf("1. Delay mode 1 used\n");
		printf("2. Delay mode 2 used\n");
		printf("Select the delay mode you want to use: ");
		params->DelayMode = addidata_console_get_unsigned_integer(NULL, 0,2);
	}

	if (params->DelayMode != 0)
	{
		params->DelayTimeUnit = addidata_console_get_unsigned_integer("Select the delay timebase you want to use (0 = Miliseconds, 1 = Seconds): ", 0,1);
		params->DelayValue = addidata_console_get_unsigned_integer("\nEnter the delay value: ", 0,0xFFFF);
	}

	//	Trigger Start
	dw_Choice = addidata_console_get_yn("\nDo you want to use the hardware trigger to start the acquisition?");

	if (dw_Choice == 1)
	{
		{
			printf("\n1. One shot trigger\n");
			printf("2. Sequence trigger\n");
			printf("Select the trigger mode do you want to use: ");
			params->TriggerAction = addidata_console_get_unsigned_integer(NULL, 1,2);
		}

		{
			printf("\n1. rising edge\n");
			printf("2. falling edge\n");
			printf("3. both edges\n");
			printf("Select the active edge: ");
			dw_Choice = addidata_console_get_unsigned_integer(NULL, 1,3);
		}

		params->TriggerAction = params->TriggerAction | (dw_Choice << 6);

		params->HardwareTriggerCount = addidata_console_get_unsigned_integer("\nEnter the number of edges before the trigger action occur: ", 0,0xFFFF);
		params->HardwareTriggerFilterTime = addidata_console_get_unsigned_integer("\nEnter the filter time (multiplier from 250 ns step) for the input trigger (0 = disable, max : 65535 -> 16,383750 ms): ", 0,0xFFFF);
	}
	else
	{
		dw_Choice = addidata_console_get_yn("\nDo you want to use the synchronisation trigger to start the acquisition? ");

		if (dw_Choice == 1)
		{
			{
				printf("\n1. One shot trigger\n");
				printf("2. Sequence trigger\n");
				printf("Select the trigger mode do you want to use: ");
				dw_Choice = addidata_console_get_unsigned_integer(NULL, 1,2);
			}

			params->TriggerAction = dw_Choice << 8;
		}
	}


	if (((params->TriggerAction & 2) == 0x2) || ((params->TriggerAction & 0x200) == 0x200))
	{
		params->ByTriggerNbrOfSeqToAcquire=addidata_console_get_unsigned_integer("\nEnter the number of sequence to acquire by each trigger: ", 1,0xFFFF);
	}

	//	Trigger Stop
	if ((params->TriggerAction == 0) || ((params->TriggerAction & 0x101) != 0))
	{
		dw_Choice = addidata_console_get_yn("\nDo you want to use the hardware trigger to stop the acquisition?");

		if (dw_Choice == 1)
		{
			params->TriggerAction = params->TriggerAction | (1 << 16);

			{
				printf("\n1. rising edge\n");
				printf("2. falling edge\n");
				printf("3. both edges\n");
				printf("Select the active edge: ");
				dw_Choice = addidata_console_get_unsigned_integer(NULL, 1,3);
			}

			params->TriggerAction = params->TriggerAction | (dw_Choice << 17);

			dw_Choice = addidata_console_get_yn("\nDo you want to use the number of edge before the trigger occur");

			if (dw_Choice == 1)
			{
				params->TriggerAction = params->TriggerAction | (1 << 19);

				if (params->HardwareTriggerCount == 0)
				{
					params->HardwareTriggerCount = addidata_console_get_unsigned_integer("\nEnter the number of edges before the trigger action occur (1 to 65535): ", 1,0xFFFF);
				}
			}

		}
	}
}
//---------------------------------------------------------------------------------
unsigned int msxe370x_sequence_compute_fields(struct msxe370x_sequence_parameters* params)
{
	return ( params->NbrOfChannel +
			(msxe370x_sequence_format_has_absolute_timestamp(params) * 2) +
			(msxe370x_sequence_format_has_relative_timestamp(params) * 2) +
			(msxe370x_sequence_format_has_sequence_counter(params)) +
			(msxe370x_sequence_format_has_hwtriggerinputinformation(params)));
}
//---------------------------------------------------------------------------------
int msxe370x_sequence_fprintf(FILE* stream, struct msxe370x_sequence_parameters* params, uint32_t data[], unsigned int count, struct msxe370x_format *format)
{
	long int total = 0; /* total number of bytes printed */
	unsigned int where = 0; /* position of next data */
	const char* datafmt = NULL;
	int (*fcn) (FILE* stream, const char* fmt, uint32_t data, const char* datasepfmt);

	if (msxe370x_sequence_format_is_analog(params))
	{
		datafmt = format->analog;
		fcn = fprintf_onedata_float;
	}
	else
	{
		datafmt = format->digital;
		fcn = fprintf_onedata_integer;
	}

	RETIFCOUNT0();
	/* timestamp? */
	if (msxe370x_sequence_format_has_absolute_timestamp(params) || msxe370x_sequence_format_has_relative_timestamp(params) )
	{
		int ret;
		RETIFCOUNTINF2();
		RETIFNEG( addidata_fprintf_timestamp(stream, format->datasep, data[0], data[1]) );
		total += ret;
		NEXT(2);
	}
	/* sequence counter? */
	if (msxe370x_sequence_format_has_sequence_counter(params))
	{
		int ret;
		RETIFCOUNT0();
		/* counter */
		RETIFNEG( fprintf(stream,format->counter,data[where]) );
		total += ret;
		NEXT(1);
	}
	/* HW trigger input information? */
	if (msxe370x_sequence_format_has_hwtriggerinputinformation(params))
	{
		int ret;
		RETIFCOUNT0();
		/* HW trigger */
		RETIFNEG( fprintf(stream,format->hwtrigger,data[where]) );
		total += ret;
		NEXT(1);
	}
	/* data */
	{
		int ret;
		RETIFCOUNT0();
		/* data */
		RETIFNEG( fprintf_data(stream, params->NbrOfChannel, (msxe370x_sequence_format_is_analog(params)?format->analog:format->digital), params->ChannelList, data+where, count, format, fcn) );
		NEXT(params->NbrOfChannel);
		total += ret;
	}
	/* record terminal */
	{
		int ret;
		RETIFNEG( fprintf(stream,format->recordterm) );
		total += ret;
	}
	return total;
}
//---------------------------------------------------------------------------------
int msxe370x_sequence_printf(struct msxe370x_sequence_parameters* params, uint32_t data[], unsigned int count, struct msxe370x_format *format)
{
	return msxe370x_sequence_fprintf(stdout, params, data, count, format);
}
