/** @file Socket.c
* Socket Functions
*
*
* @author C. Guinot
* @date 30.01.2006
*
* @version  $LastChangedRevision$
* @version $LastChangedDate$
*
*
* @par LICENCE
* @verbatim
   Copyright (C) 2004,2005  ADDI-DATA GmbH for the source code of this module.

        ADDI-DATA GmbH
        Dieselstrasse 3
        D-77833 Ottersweier
        Tel: +19(0)7223/9493-0
        Fax: +49(0)7223/9493-92
        http://www.addi-data-com
        info@addi-data.com


   @endverbatim
*
*
*/
#ifndef _WIN32
   #pragma warn -par
#endif

#include <stdio.h>
#include <math.h>
#include <conio.h>
#include <STDARG.H>

#include "winsock2.h"
#include "AddiDefine.h"
#include "AddiSocket.h"



/** \internal 
* Open a client socket
*
* @param[in] pc_ServerIPAddress : IP Address of the server
* @param[in] w_ServerPort : Port number of the server
* @param[out] ps_ClientSocket : socket descriptor
*
* @return ADDIDATA_SOCKET_WSASTARTUP_ERROR <br>
*         ADDIDATA_SOCKET_GETHOSTBYNAME_ERROR <br>
*         ADDIDATA_CREATE_SOCKET_ERROR <br>
*         ADDIDATA_SOCKET_CONNECT_ERROR <br>
*/
int i_Socket_ClientOpen (char * pc_ServerIPAddress,WORD w_ServerPort,SOCKET  *ps_ClientSocket)
   {

   struct hostent *hp;
   struct sockaddr_in sai_Server;
   WSADATA wsaData;

   /*********************************************/
   /* Initiates use of Ws2_32.dll by a process. */
   /*********************************************/
   if (WSAStartup (0x202,&wsaData) == SOCKET_ERROR)
      {
      WSACleanup();
      return(ADDIDATA_SOCKET_WSASTARTUP_ERROR);
      }

   /*********************************/
   /* Open a file to save the value */
   /*********************************/
   hp = gethostbyname(pc_ServerIPAddress);

   if (hp == NULL )
      {
      WSACleanup();
      return(ADDIDATA_SOCKET_GETHOSTBYNAME_ERROR);
      }

   /****************************************************************/
   /* Copy the resolved information into the sockaddr_in structure */
   /****************************************************************/
   memset(&sai_Server,0,sizeof(sai_Server));
   memcpy(&(sai_Server.sin_addr),hp->h_addr,hp->h_length);
   sai_Server.sin_family = AF_INET;
   sai_Server.sin_port = htons(w_ServerPort);

   /*********************/
   /* Create the socket */
   /*********************/
   /* Open a socket       */
   *ps_ClientSocket = socket(AF_INET,SOCK_STREAM,0);

   if (*ps_ClientSocket <0)
      {
      WSACleanup();
      return(ADDIDATA_CREATE_SOCKET_ERROR);

      }

   /************************************/
   /* Connect the client on the server */
   /************************************/
   if (connect(*ps_ClientSocket,(struct sockaddr*)&sai_Server,sizeof(struct sockaddr_in))== SOCKET_ERROR)
      {
      WSACleanup();
      return(ADDIDATA_SOCKET_CONNECT_ERROR);

      }

   return(0);
   }

/** \internal
* Open a client socket
*
* @param[in] pc_ServerIPAddress : IP Address of the server
* @param[in] w_ServerPort : Port number of the server
* @param[in] i_ReceiveBufferSize : Receive buffer size (byte)
* @param[out] ps_ClientSocket : socket descriptor
*
* @return ADDIDATA_SOCKET_WSASTARTUP_ERROR <br>
*         ADDIDATA_SOCKET_GETHOSTBYNAME_ERROR <br>
*         ADDIDATA_CREATE_SOCKET_ERROR <br>
*         ADDIDATA_SOCKET_RCVBUF_ERROR <br>
*         ADDIDATA_SOCKET_CONNECT_ERROR <br>
*/
int i_Socket_ClientOpenEx (char * pc_ServerIPAddress,WORD w_ServerPort,int i_ReceiveBufferSize, SOCKET  *ps_ClientSocket)
   {

   struct hostent *hp;
   struct sockaddr_in sai_Server;
   WSADATA wsaData;

   /*********************************************/
   /* Initiates use of Ws2_32.dll by a process. */
   /*********************************************/
   if (WSAStartup (0x202,&wsaData) == SOCKET_ERROR)
      {
      WSACleanup();
      return(ADDIDATA_SOCKET_WSASTARTUP_ERROR);
      }

   /*********************************/
   /* Open a file to save the value */
   /*********************************/
   hp = gethostbyname(pc_ServerIPAddress);

   if (hp == NULL )
      {
      WSACleanup();
      return(ADDIDATA_SOCKET_GETHOSTBYNAME_ERROR);
      }

   /****************************************************************/
   /* Copy the resolved information into the sockaddr_in structure */
   /****************************************************************/
   memset(&sai_Server,0,sizeof(sai_Server));
   memcpy(&(sai_Server.sin_addr),hp->h_addr,hp->h_length);
   sai_Server.sin_family = AF_INET;
   sai_Server.sin_port = htons(w_ServerPort);

   /*********************/
   /* Create the socket */
   /*********************/
   /* Open a socket       */
   *ps_ClientSocket = socket(AF_INET,SOCK_STREAM,0);

   if (*ps_ClientSocket <0)
      {
      WSACleanup();
      return(ADDIDATA_CREATE_SOCKET_ERROR);

      }

   /****************************************/
   /* Set the socket reception buffer size */
   /****************************************/


   if (setsockopt (*ps_ClientSocket, SOL_SOCKET, SO_RCVBUF, (char *) &i_ReceiveBufferSize, sizeof (i_ReceiveBufferSize)) == SOCKET_ERROR)
      {
      WSACleanup();
      return(ADDIDATA_SOCKET_RCVBUF_ERROR);
      }


   /************************************/
   /* Connect the client on the server */
   /************************************/
   if (connect(*ps_ClientSocket,(struct sockaddr*)&sai_Server,sizeof(struct sockaddr_in))== SOCKET_ERROR)
      {
      WSACleanup();
      return(ADDIDATA_SOCKET_CONNECT_ERROR);
      }

   return(0);
   }


/** \internal 
* Close a client socket
*
* @param[in] s_ClientSocket : socket descriptor
*
* @return -
*/
int i_Socket_ClientClose (SOCKET  s_ClientSocket)
   {
   INT i_ReturnValue = 0;
   closesocket (s_ClientSocket);

   WSACleanup();
   return(i_ReturnValue);
   }

