using System;
using System.Collections.Generic;
using System.Text;




namespace ExtTemperatureRead
{
    public class Msxe371x : Msxe
    {
        /// <summary>
        /// System web reference
        /// </summary>
        private MSXE371xWeb.MSXE371x sysWeb;

        /// <summary>
        /// Constructor
        /// </summary>
        /// <param name="ip"></param>
        public Msxe371x(string ip)
            : base(ip)
        {
            // Nothing specific
        }

        /// <summary>
        /// Override the connect method
        /// </summary>
        /// <param name="error"></param>
        /// <returns>
        /// 0:  success
        /// -1: wrong IP address
        /// -2: system error. Check the error variable for more informations
        /// -3: wrong system type
        /// -4: system not reachable
        /// -5: other error. Check the error variable for more informations
        /// </returns>
        public override int Connect(out string error)
        {
            ulong tmp;

            // Call Connect of the base class
            // If successful, then the connectionState variable is set to 1
            int retval = base.Connect(out error);
            if (retval != 0)
                return retval;

            // Create the web-reference object
            this.sysWeb = new MSXE371xWeb.MSXE371x();
            try
            {
                this.sysWeb.Url = "http://" + this.IPAddress + ":5555";
                this.sysWeb.Timeout = 2000;
            }
            catch (Exception) { return -1; }

            // Check if transducers are available
            retval = this.TransducerGetNbrOfType(out tmp, out error);
            if (retval != 0)
            {
                this.connectionState = 0;
                return retval;
            }

            return 0;
        }

        #region "SOAP API"

        /// <summary>
        /// Returns the number of transducer types currently defined in the database. 
        /// </summary>
        /// <param name="transducerTypeNumber">number of transducers type</param>
        /// <param name="error">Error message set if the return value is -2 or -5</param>
        /// <returns>
        /// 0:  success
        /// -1: you must call the Connect function before
        /// -2: system error. Check the error variable for more informations
        /// -3: the method is not implemented on the system. Please check the type of your system
        /// -4: the system is not reachable (not present on the network, or no SOAP server on it)
        /// -5: other error. Check the error variable for more informations
        /// </returns>
        public int TransducerGetNbrOfType(out ulong transducerTypeNumber, out string error)
        {
            // Default
            transducerTypeNumber = 0;
            error = string.Empty;

            // Check the connection state: Connect method should be called first
            if (this.connectionState != 1)
            {
                return -1;
            }

            // Call the SOAP function
            try
            {
                MSXE371xWeb.DefaultResponse response = this.sysWeb.TransducerGetNbrOfType(out transducerTypeNumber);
                if (response.iReturnValue != 0)
                {
                    error = "The system returned an error when calling the MSXE371x__TransducerGetNbrOfType function." + Environment.NewLine;
                    error += "Please check the documentation of this function for more information." + Environment.NewLine;
                    error += "(iReturnValue: " + response.iReturnValue.ToString() + ", syserrno: " + response.syserrno.ToString() + ")";
                    return -2;
                }
            }
            catch (System.Web.Services.Protocols.SoapHeaderException)
            {
                return -3;
            }
            catch (System.Net.WebException)
            {
                return -4;
            }
            catch (Exception ex)
            {
                error = "Unhandled exception occured: " + ex.Message;
                return -5;
            }
            return 0;
        }

        /// <summary>
        /// Initialise the external temperature
        /// </summary>
        /// <param name="ulConnectedTempSensor">Connected sensor type</param>
        /// <param name="ulConvertMode">Converting mode</param>
        /// <param name="ulGainSelection">Gain selection (0 for auto mode)</param>
        /// <param name="ulFrequencySelection">Acquisition frequency selection (50, 240)</param>
        /// <param name="ulPowerSaveMode">Power save mode selection</param>
        /// <param name="error">Error message set if the return value is -2 or -5</param>
        /// <returns>
        /// 0:  success
        /// -1: you must call the Connect function before
        /// -2: system error. Check the error variable for more informations
        /// -3: the method is not implemented on the system. Please check the type of your system
        /// -4: the system is not reachable (not present on the network, or no SOAP server on it)
        /// -5: other error. Check the error variable for more informations
        /// </returns>
        public int ExternalTemperatureInit(ulong connectedSensor, ulong convertMode, ulong gainSelection, ulong frequencySelection, ulong powerSaveMode, out string error)
        {
            // Default
            error = string.Empty;

            // Check the connection state: Connect method should be called first
            if (this.connectionState != 1)
            {
                return -1;
            }

            // Call the SOAP function
            try
            {
                int syserrno = 0;
                
                int retval = this.sysWeb.ExternalTemperatureInit(connectedSensor, convertMode, gainSelection, frequencySelection, powerSaveMode, 0, 0, 0, 0, out syserrno);
                if (retval != 0)
                {
                    error = "The system returned an error when calling the MSXE371x__ExternalTemperatureInit function." + Environment.NewLine;
                    error += "Please check the documentation of this function for more information." + Environment.NewLine;
                    error += "(iReturnValue: " + retval.ToString() + ", syserrno: " + syserrno.ToString() + ")";
                    return -2;
                }
            }
            catch (System.Web.Services.Protocols.SoapHeaderException)
            {
                return -3;
            }
            catch (System.Net.WebException)
            {
                return -4;
            }
            catch (Exception ex)
            {
                error = "Unhandled exception occured: " + ex.Message;
                return -5;
            }
            return 0;
        }


        /// <summary>
        /// Release the external temperature
        /// </summary>
        /// <param name="error"></param>
        /// <returns>
        /// 0: success
        /// -1: means an system error occurred 
        ///</returns>
        public int ExternalTemperatureRelease(out string error)
        {
            // Default
            error = string.Empty;

            // Check the connection state: Connect method should be called first
            if (this.connectionState != 1)
            {
                return -1;
            }

            // Call the SOAP function
            try
            {
                int syserrno = 0;

                int retval = this.sysWeb.ExternalTemperatureRelease(out syserrno);
                if (retval != 0)
                {
                    error = "The system returned an error when calling the MSXE371x__ExternalTemperatureRelease function." + Environment.NewLine;
                    error += "Please check the documentation of this function for more information." + Environment.NewLine;
                    error += "(iReturnValue: " + retval.ToString() + ", syserrno: " + syserrno.ToString() + ")";
                    return -2;
                }
            }
            catch (System.Web.Services.Protocols.SoapHeaderException)
            {
                return -3;
            }
            catch (System.Net.WebException)
            {
                return -4;
            }
            catch (Exception ex)
            {
                error = "Unhandled exception occured: " + ex.Message;
                return -5;
            }
            return 0;
        }


        /// <summary>
        /// Read the 32 bits counter value
        /// </summary>
        /// <param name="error"></param>
        /// <returns>
        /// 0: success
        ///
        /// </returns>
        public int ExternalTemperatureRead(out ulong value, out ulong timeStampHigh, out ulong timeStampLow, out string error)
        {

            // Default
            error = string.Empty;

            value = 0;
            timeStampHigh = 0;
            timeStampLow = 0;

            // Check the connection state: Connect method should be called first
            if (this.connectionState != 1)
            {
                return -1;
            }

            // Call the SOAP function
            try
            {
                MSXE371xWeb.DefaultResponse response = this.sysWeb.ExternalTemperatureRead(out value, out timeStampHigh, out timeStampLow);
                if (response.iReturnValue != 0)
                {
                    error = "The system returned an error when calling the MSXE371x__ExternalTemperatureRead function." + Environment.NewLine;
                    error += "Please check the documentation of this function for more information." + Environment.NewLine;
                    error += "(iReturnValue : " + response.iReturnValue.ToString() + ", syserrno: " + response.syserrno.ToString() + ")";
                    return -2;
                }

                //int retval = this.sysWeb.ExternalTemperatureRead(out syserrno);
                //if (retval != 0)
                //{
                //    error = "The system returned an error when calling the MSXE371x__ExternalTemperatureRead function." + Environment.NewLine;
                //    error += "Please check the documentation of this function for more information." + Environment.NewLine;
                //    error += "(iReturnValue 2: " + retval.ToString() + ", syserrno: " + syserrno.ToString() + ")";
                //    return -2;
                //}


            }
            catch (System.Web.Services.Protocols.SoapHeaderException)
            {
                return -3;
            }
            catch (System.Net.WebException)
            {
                return -4;
            }
            catch (Exception ex)
            {
                error = "Unhandled exception occured: " + ex.Message;
                return -5;
            }
            return 0;
        }


        /// <summary>
        /// Temperature gain calibration
        /// </summary>
        /// <param name="ulGainSelection"></param>
        /// <param name="dRefValue"></param>
        /// <param name="error"></param>
        /// <returns>
        /// 0: success
        /// -1 : means an system error occurred
        /// -2 : Gain selection error
        /// -3 : Frequency selection error
        /// -4 : Ref value to high for the selected gain
        /// -5 : Calibration file access error
        /// -100 : IOCTL system call error
        /// </returns>
        public int ExternalTemperatureGainCalibration(ulong gainSelection, double refValue, out string error)
        {
            // Default
            error = string.Empty;

            // Check the connection state: Connect method should be called first
            if (this.connectionState != 1)
            {
                return -1;
            }

            // Call the SOAP function
            try
            {
                int syserrno = 0;

                int retval = this.sysWeb.ExternalTemperatureGainCalibration(gainSelection, refValue, out syserrno);
                if (retval != 0)
                {
                    error = "The system returned an error when calling the MSXE371x__ExternalTemperatureGainCalibration function." + Environment.NewLine;
                    error += "Please check the documentation of this function for more information." + Environment.NewLine;
                    error += "(iReturnValue: " + retval.ToString() + ", syserrno: " + syserrno.ToString() + ")";
                    return -2;
                }
            }
            catch (System.Web.Services.Protocols.SoapHeaderException)
            {
                return -3;
            }
            catch (System.Net.WebException)
            {
                return -4;
            }
            catch (Exception ex)
            {
                error = "Unhandled exception occured: " + ex.Message;
                return -5;
            }
            return 0;
        }


        #endregion
    }
}
