using System;
using System.Threading;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using System.Collections;
using System.Net;
using System.Net.Sockets;
using AddiData.ADDIControls;
using AddiData.ADDISocket;
using AddiData;

namespace TransducerAutoRefresh
{
    public partial class TransducerAutoRefresh : Form
    {
        // Global Variable
        private MXCommon_WebReference.MXCommon MX_Common;
        private MSXE371x_WebReference.MSXE371x MSXE371x;
        private MXCommon_WebReference.DefaultResponse MXResponse;
        private MSXE371x_WebReference.DefaultResponse Response;
        private ArrayList CounterOptions0 = new ArrayList();
        private ArrayList CounterOptions1 = new ArrayList();
        private int RetValue = 0;
        private int syserno = 0;
        private TranducerInfo[] TranducerInformations;
        private CheckBox[] AutoRefreshChannels = new CheckBox[8];
        private CheckBox[] DataFrame = new CheckBox[5];
        private bool boExternalTempInit = false;
        private bool boCounterInit = false;
        private bool boCounterCompareInit = false;
        private bool boAutoRefreshInit = false;
        private bool boSaveGB_Thermocouple = false;
        private bool boSaveGB_Counter = false;
        private bool boSaveGB_CounterCompare = false;
        private bool tcVersion = false;
        private int iDataStreamSize = 0;
        private ulong ulTFormat = 0;
        ulong ulChannelMask = 0;
        ArrayList alAcquisitionChannels;
        Socket SocketClient = null;
        String DisplayText = "";
        AsyncClientStream Client = null;


        public static String ConvertFromTime_T(byte[] bBuffer, int iIndex)
        {
            ulong ulTimeStampHigh = BitConverter.ToUInt32(bBuffer, iIndex);
            ulong ulTimeStampLow = BitConverter.ToUInt32(bBuffer, iIndex + 4);

            return (ConvertFromTime_T(ulTimeStampHigh, ulTimeStampLow));
        }

        public static DateTime ConvertFromTime_T(long time_t)
        {
            DateTime dt70 = new DateTime(1970, 1, 1, 0, 0, 0, 0);
            long tickTotal = dt70.Ticks + (time_t * 10000000L);
            DateTime netTimeT = new DateTime(tickTotal);
            return netTimeT;
        }

        public static String ConvertFromTime_T(ulong ulTimeStampHigh, ulong ulTimeStampLow)
        {
            ulong ulTimeStampLowms = (ulong)(ulTimeStampLow / 1000);
            ulong ulTimeStampLowmus = (ulong)(ulTimeStampLow % 1000);

            DateTime dt70 = new DateTime(1970, 1, 1, 0, 0, 0, 0);
            long tickTotal = dt70.Ticks + (long)(ulTimeStampHigh * 10000000L);
            DateTime netTimeT = new DateTime(tickTotal);
            return (netTimeT.ToString() + "." + ulTimeStampLowms.ToString("D3") + "." + ulTimeStampLowmus.ToString("D3"));
        }

        private UInt64 ulGetNbrOfTransducerChannel(String sModuleType)
        {
            if (sModuleType.IndexOf("-", 0) == -1)
                return 0;
            sModuleType = sModuleType.Substring(sModuleType.IndexOf("-", 0) + 1);

            if (sModuleType.IndexOf("-", 0) == -1)
                return 0;
            sModuleType = sModuleType.Substring(sModuleType.IndexOf("-", 0) + 1);

            if (sModuleType.IndexOf("-", 0) == -1)
                return 0;
            sModuleType = sModuleType.Remove(sModuleType.IndexOf("-", 0));

            return (Convert.ToUInt64(sModuleType));
        }

        public class TranducerInfo
        {
            private ulong ulSelectionIndex;
            private String sName;
            private ulong ulCalibrationStatus;
            private ulong ulCalibratedChannels;
            private ulong ulType;
            private ulong ulFrequency;
            private ulong ulImpedance;
            private double dVeff;
            private double dSensibility;
            private double dRange;

            public TranducerInfo(ulong SelectionIndex,
                                 String Name,
                                 ulong CalibrationStatus,
                                 ulong CalibratedChannels,
                                 ulong Type,
                                 ulong Frequency,
                                 ulong Impedance,
                                 double Veff,
                                 double Sensibility,
                                 double Range)
            {
                ulSelectionIndex = SelectionIndex;
                sName = Name.Remove(Name.IndexOf((char)0));
                ulCalibrationStatus = CalibrationStatus;
                ulCalibratedChannels = CalibratedChannels;
                ulType = Type;
                ulFrequency = Frequency;
                ulImpedance = Impedance;
                dVeff = Veff;
                dSensibility = Sensibility;
                dRange = Range;
            }

            public ulong SelectionIndex { get { return (ulSelectionIndex); } set { ulSelectionIndex = value; } }
            public ulong CalibrationStatus { get { return (ulCalibrationStatus); } set { ulCalibrationStatus = value; } }
            public ulong CalibratedChannels { get { return (ulCalibratedChannels); } set { ulCalibratedChannels = value; } }
            public ulong Type { get { return (ulType); } set { ulType = value; } }
            public ulong Frequency { get { return (ulFrequency); } set { ulFrequency = value; } }
            public ulong Impedance { get { return (ulImpedance); } set { ulImpedance = value; } }
            public double Veff { get { return (dVeff); } set { dVeff = value; } }
            public double Sensibility { get { return (dSensibility); } set { dSensibility = value; } }
            public double Range { get { return (dRange); } set { dRange = value; } }
            public String Name { get { return (sName); } set { sName = value; } }

            public override String ToString()
            {
                String sRet;

                sRet = "Transducer Selection Index : " + ulSelectionIndex.ToString();
                sRet = sRet + Environment.NewLine + "Name                                   : " + sName;
                sRet = sRet + Environment.NewLine + "Calibration status                  : " + ulCalibrationStatus.ToString();
                sRet = sRet + Environment.NewLine + "Type Value                          : " + ulType.ToString();
                sRet = sRet + Environment.NewLine + "Frequency                           : " + ulFrequency.ToString() + " Hz";
                sRet = sRet + Environment.NewLine + "Impedance                          : " + ulImpedance.ToString() + " Ohm";
                sRet = sRet + Environment.NewLine + "Nominal Voltage                  : " + dVeff.ToString() + " V";
                sRet = sRet + Environment.NewLine + "Sensibility                            : " + dSensibility.ToString() + " mV/V/mm";
                sRet = sRet + Environment.NewLine + "Range                                 : +/-" + dRange.ToString() + " mm";

                return (sRet);
            }
        }

        public class ListData
        {
            private string myItem;
            private long myValue;

            public ListData(string strItem, long lValue)
            {

                this.myItem = strItem;
                this.myValue = lValue;
            }

            public string Item
            {
                get { return myItem; }
            }

            public long Value
            {

                get { return myValue; }
            }
        }

        public TransducerAutoRefresh()
        {
            InitializeComponent();
        }

        private void TransducerAutoRefresh_Load(object sender, EventArgs e)
        {
            MX_Common = new MXCommon_WebReference.MXCommon();
            MSXE371x = new MSXE371x_WebReference.MSXE371x();
            MX_Common.Timeout = 5000;
            MSXE371x.Timeout = 5000;

            // Gain list box initialisation
            ArrayList Gains = new ArrayList();
            Gains.Add(new ListData("Auto mode", 0));

            LB_T_Gain.DisplayMember = "Item";
            LB_T_Gain.ValueMember = "Value";
            LB_T_Gain.DataSource = Gains;

            // Frequency list box initialisation
            ArrayList Frequencys = new ArrayList();
            Frequencys.Add(new ListData("10 Hz", 10));
            Frequencys.Add(new ListData("50 Hz", 50)); //
            Frequencys.Add(new ListData("240 Hz", 240));

            LB_T_Frequency.DisplayMember = "Item";
            LB_T_Frequency.ValueMember = "Value";
            LB_T_Frequency.DataSource = Frequencys;

            // Display format list box initialisation
            ArrayList Formats = new ArrayList();
            Formats.Add(new ListData("mOhm", 0));
            Formats.Add(new ListData("C", 1));
            Formats.Add(new ListData("F", 2));

            LB_T_Format.DisplayMember = "Item";
            LB_T_Format.ValueMember = "Value";
            LB_T_Format.DataSource = Formats;

            // Counter mode list box initialisation
            ArrayList CounterModes = new ArrayList();
            CounterModes.Add(new ListData("Simple mode", 1));
            CounterModes.Add(new ListData("Double mode", 2));
            CounterModes.Add(new ListData("Quadruple mode", 4));
            CounterModes.Add(new ListData("Direct", 0));

            LB_IC_Mode.DisplayMember = "Item";
            LB_IC_Mode.ValueMember = "Value";
            LB_IC_Mode.DataSource = CounterModes;

            // Counter option list box initialisation
            CounterOptions0.Add(new ListData("Hysteresus on", 1));
            CounterOptions0.Add(new ListData("Hysteresus off", 0));
            CounterOptions1.Add(new ListData("Increment", 0));
            CounterOptions1.Add(new ListData("Decrement", 1));

            LB_IC_Option.DisplayMember = "Item";
            LB_IC_Option.ValueMember = "Value";
            LB_IC_Option.DataSource = CounterOptions0;

            // Counter compare mode list box initialisation
            ArrayList CompareModes = new ArrayList();
            CompareModes.Add(new ListData("Disabled", -1));
            CompareModes.Add(new ListData("Equals compare value", 0));
            CompareModes.Add(new ListData("Equals a multiple of the compare value", 1));

            LB_IC_CompareMode.DisplayMember = "Item";
            LB_IC_CompareMode.ValueMember = "Value";
            LB_IC_CompareMode.DataSource = CompareModes;

            // Counter compare options list box initialisation
            ArrayList CompareOptions = new ArrayList();
            CompareOptions.Add(new ListData("No synchro trigger used", 0));
            CompareOptions.Add(new ListData("Generates a synchro trigger", 1));

            LB_IC_SynchroTrigger.DisplayMember = "Item";
            LB_IC_SynchroTrigger.ValueMember = "Value";
            LB_IC_SynchroTrigger.DataSource = CompareOptions;

            // Average mode list box initialisation
            ArrayList AverageOptions = new ArrayList();
            AverageOptions.Add(new ListData("Disabled", 0));
            AverageOptions.Add(new ListData("Enabled", 1));

            LB_AverageMode.DisplayMember = "Item";
            LB_AverageMode.ValueMember = "Value";
            LB_AverageMode.DataSource = AverageOptions;

            // Trigger source list box initialisation
            ArrayList TriggerSources = new ArrayList();
            TriggerSources.Add(new ListData("Disabled", 0));
            TriggerSources.Add(new ListData("Hardware trigger", 1));
            TriggerSources.Add(new ListData("Synchro trigger", 2));
            TriggerSources.Add(new ListData("Counter compare trigger", 4));

            LB_TriggerSource.DisplayMember = "Item";
            LB_TriggerSource.ValueMember = "Value";
            LB_TriggerSource.DataSource = TriggerSources;


            // Trigger mode list box initialisation
            ArrayList TriggerModes = new ArrayList();
            TriggerModes.Add(new ListData("One shot trigger", 1));
            TriggerModes.Add(new ListData("Sequence trigger", 2));

            LB_TriggerMode.DisplayMember = "Item";
            LB_TriggerMode.ValueMember = "Value";
            LB_TriggerMode.DataSource = TriggerModes;


            // HW Trigger edge list box initialisation
            ArrayList HWTriggerEdges = new ArrayList();
            HWTriggerEdges.Add(new ListData("Rising edge", 1));
            HWTriggerEdges.Add(new ListData("Falling edge", 2));
            HWTriggerEdges.Add(new ListData("Both front", 3));

            LB_HWEdge.DisplayMember = "Item";
            LB_HWEdge.ValueMember = "Value";
            LB_HWEdge.DataSource = HWTriggerEdges;

            // Save the auto refresh channels check box
            AutoRefreshChannels[0] = CB_Channel0;
            AutoRefreshChannels[1] = CB_Channel1;
            AutoRefreshChannels[2] = CB_Channel2;
            AutoRefreshChannels[3] = CB_Channel3;
            AutoRefreshChannels[4] = CB_Channel4;
            AutoRefreshChannels[5] = CB_Channel5;
            AutoRefreshChannels[6] = CB_Channel6;
            AutoRefreshChannels[7] = CB_Channel7;

            // Save the data frame check box
            DataFrame[0] = CB_DataFrameTimeStamp;
            DataFrame[1] = CB_DataFrameMetric;
            DataFrame[2] = CB_DataFrameInvert;
            DataFrame[3] = CB_DataFrameTemperature;
            DataFrame[4] = CB_DataFrameIncCounter;
        }

        private void LB_IC_Mode_SelectedIndexChanged(object sender, EventArgs e)
        {
            if (Convert.ToUInt32(LB_IC_Mode.SelectedValue) == 0)
            {
                LB_IC_Option.DataSource = CounterOptions1;
            }
            else
            {
                LB_IC_Option.DataSource = CounterOptions0;
            }
        }

        private bool bo_ThermocoupleInitialisation()
        {
            bool bo_Ret = true;

            if (GB_Thermocouple.Enabled == true)
            {
                try
                {
                    // External temperature initialisation

                    RetValue = MSXE371x.ExternalTemperatureInit(Convert.ToUInt32(LB_T_Thermocouple.SelectedValue),
                                                                Convert.ToUInt32(LB_T_Format.SelectedValue),
                                                                Convert.ToUInt32(LB_T_Gain.SelectedValue),
                                                                Convert.ToUInt32(LB_T_Frequency.SelectedValue),
                                                                0, 0, 0, 0, 0, out syserno);
                    if (RetValue != 0)
                    {
                        MessageBox.Show("ExternalTemperatureInit Error : " + RetValue.ToString(), "Function error", MessageBoxButtons.OK, MessageBoxIcon.Exclamation);
                        bo_Ret = false;
                    }
                    else
                    {
                        boExternalTempInit = true;
                    }
                }

                catch (Exception ex)
                {
                    MessageBox.Show("ExternalTemperatureInit Exception occur : " + ex.Message, "Function error", MessageBoxButtons.OK, MessageBoxIcon.Exclamation);
                    bo_Ret = false;
                }
            }

            return (bo_Ret);
        }

        private bool bo_ThermocoupleRelease()
        {
            bool bo_Ret = true;

            if (boExternalTempInit == true)
            {
                try
                {
                    // Release external temperature

                    RetValue = MSXE371x.ExternalTemperatureRelease(out syserno);

                    if (RetValue != 0)
                    {
                        MessageBox.Show("ExternalTemperatureRelease Error : " + RetValue.ToString(), "Function error", MessageBoxButtons.OK, MessageBoxIcon.Exclamation);
                        bo_Ret = false;
                    }
                }

                catch (Exception ex)
                {
                    MessageBox.Show("ExternalTemperatureRelease Exception occur : " + ex.Message, "Function error", MessageBoxButtons.OK, MessageBoxIcon.Exclamation);
                    bo_Ret = false;
                }

                boExternalTempInit = false;
            }
            return (bo_Ret);
        }

        private bool bo_CounterInitialisation()
        {
            bool bo_Ret = true;

            if (GB_Counter.Enabled == true)
            {
                try
                {
                    // Counter initialisation

                    RetValue = MSXE371x.IncCounterInit(Convert.ToUInt32(LB_IC_Mode.SelectedValue),
                                                       Convert.ToUInt32(LB_IC_Option.SelectedValue),
                                                       0,
                                                       0,
                                                       0,
                                                       0,
                                                       out syserno);
                    if (RetValue != 0)
                    {
                        MessageBox.Show("IncCounterInit Error : " + RetValue.ToString(), "Function error", MessageBoxButtons.OK, MessageBoxIcon.Exclamation);
                        bo_Ret = false;
                    }
                    else
                    {
                        boCounterInit = true;
                    }
                }

                catch (Exception ex)
                {
                    MessageBox.Show("IncCounterInit Exception occur : " + ex.Message, "Function error", MessageBoxButtons.OK, MessageBoxIcon.Exclamation);
                    bo_Ret = false;
                }

                if ((bo_Ret == true) && (GB_CounterCompare.Enabled == true) && (Convert.ToInt32(LB_IC_CompareMode.SelectedValue) != -1))
                {
                    try
                    {
                        // Counter compare initialisation

                        RetValue = MSXE371x.IncCounterInitAndEnableCompareLogic(Convert.ToUInt64(TB_IC_CompareValue.Text),
                                                                                Convert.ToUInt32(LB_IC_CompareMode.SelectedValue),
                                                                                Convert.ToUInt32(LB_IC_SynchroTrigger.SelectedValue),
                                                                                0,
                                                                                0,
                                                                                out syserno);
                        if (RetValue != 0)
                        {
                            MessageBox.Show("IncCounterInitAndEnableCompareLogic Error : " + RetValue.ToString(), "Function error", MessageBoxButtons.OK, MessageBoxIcon.Exclamation);
                            bo_Ret = false;
                        }
                        else
                        {
                            boCounterCompareInit = true;
                        }
                    }

                    catch (Exception ex)
                    {
                        MessageBox.Show("IncCounterInitAndEnableCompareLogic Exception occur : " + ex.Message, "Function error", MessageBoxButtons.OK, MessageBoxIcon.Exclamation);
                        bo_Ret = false;
                    }
                }
            }
            return (bo_Ret);
        }

        private bool bo_CounterRelease()
        {
            bool bo_Ret = true;

            if (boCounterInit == true)
            {
                if (boCounterCompareInit == true)
                {
                    try
                    {
                        // Release the counter compare logic

                        RetValue = MSXE371x.IncCounterDisableAndReleaseCompareLogic(out syserno);
                        if (RetValue != 0)
                        {
                            MessageBox.Show("IncCounterDisableAndReleaseCompareLogic Error : " + RetValue.ToString(), "Function error", MessageBoxButtons.OK, MessageBoxIcon.Exclamation);
                            bo_Ret = false;
                        }
                    }

                    catch (Exception ex)
                    {
                        MessageBox.Show("IncCounterDisableAndReleaseCompareLogic Exception occur : " + ex.Message, "Function error", MessageBoxButtons.OK, MessageBoxIcon.Exclamation);
                        bo_Ret = false;
                    }

                    boCounterCompareInit = false;
                }

                try
                {
                    // Counter compare release

                    RetValue = MSXE371x.IncCounterRelease(out syserno);
                    if (RetValue != 0)
                    {
                        MessageBox.Show("IncCounterRelease Error : " + RetValue.ToString(), "Function error", MessageBoxButtons.OK, MessageBoxIcon.Exclamation);
                        bo_Ret = false;
                    }
                }

                catch (Exception ex)
                {
                    MessageBox.Show("IncCounterRelease Exception occur : " + ex.Message, "Function error", MessageBoxButtons.OK, MessageBoxIcon.Exclamation);
                    bo_Ret = false;
                }

                boCounterInit = false;
            }
            return (bo_Ret);
        }

        private void LB_TransducerList_SelectedIndexChanged(object sender, EventArgs e)
        {
            L_TransducerInfo.Text = TranducerInformations[LB_TransducerList.SelectedIndex].ToString();

        }

        private void LB_AverageMode_SelectedIndexChanged(object sender, EventArgs e)
        {
            TB_AverageValue.Enabled = Convert.ToBoolean(LB_AverageMode.SelectedValue);
        }

        private void CB_DataFrame3_CheckedChanged(object sender, EventArgs e)
        {
            GB_Thermocouple.Enabled = CB_DataFrameTemperature.Checked;
        }

        private void TriggerManagement()
        {
            LB_TriggerMode.Enabled = Convert.ToBoolean(LB_TriggerSource.SelectedValue);
            TB_SequenceTrigger.Enabled = Convert.ToBoolean(LB_TriggerSource.SelectedValue) && Convert.ToBoolean(Convert.ToUInt32(LB_TriggerMode.SelectedValue) == 2);
            LB_HWEdge.Enabled = Convert.ToBoolean(Convert.ToUInt32(LB_TriggerSource.SelectedValue) == 1);
            TB_HTriggerCount.Enabled = Convert.ToBoolean(Convert.ToUInt32(LB_TriggerSource.SelectedValue) == 1);

            CounterManagement();
        }

        private void LB_TriggerSource_SelectedIndexChanged(object sender, EventArgs e)
        {
            TriggerManagement();
        }

        private void LB_TriggerMode_SelectedIndexChanged(object sender, EventArgs e)
        {
            TriggerManagement();
        }

        private void CounterManagement()
        {
            GB_Counter.Enabled = CB_DataFrameIncCounter.Checked || Convert.ToBoolean(Convert.ToUInt32(LB_TriggerSource.SelectedValue) == 4) || Convert.ToBoolean(Convert.ToUInt32(LB_TriggerSource.SelectedValue) == 2);
            GB_CounterCompare.Enabled = GB_Counter.Enabled;
        }

        private void CB_DataFrame4_CheckedChanged(object sender, EventArgs e)
        {
            CounterManagement();
        }

        private void LB_IC_CompareMode_SelectedIndexChanged(object sender, EventArgs e)
        {
            TB_IC_CompareValue.Enabled = Convert.ToBoolean(Convert.ToInt32(LB_IC_CompareMode.SelectedValue) != -1);
            LB_IC_SynchroTrigger.Enabled = TB_IC_CompareValue.Enabled;
        }

        private void B_Connect_Click(object sender, EventArgs e)
        {
            System.Text.ASCIIEncoding enc = new System.Text.ASCIIEncoding();
            byte[] pcName = new byte[255];
            ulong ulNbrOfTransducers;
            ulong ulNbrOfChannels;
            ulong ulSelectionIndex;
            ulong ulCalibrationStatus;
            ulong ulCalibratedChannels;
            ulong ulType;
            ulong ulFrequency;
            ulong ulImpedance;
            double dVeff;
            double dSensibility;
            double dRange;

            if ((TB_IPAddress_0.Text.ToString() == "") || (TB_IPAddress_1.Text.ToString() == "") || (TB_IPAddress_2.Text.ToString() == "") || (TB_IPAddress_3.Text.ToString() == ""))
            {
                MessageBox.Show("The IP address you enter is not valid, please try again.", "IP address error", MessageBoxButtons.OK, MessageBoxIcon.Exclamation);
            }
            else
            {
                if ((Convert.ToUInt64(TB_IPAddress_0.Text.ToString(), 10) >= 0) && (Convert.ToUInt64(TB_IPAddress_0.Text.ToString(), 10) < 256))
                {
                    if ((Convert.ToUInt64(TB_IPAddress_1.Text.ToString(), 10) >= 0) && (Convert.ToUInt64(TB_IPAddress_1.Text.ToString(), 10) < 256))
                    {
                        if ((Convert.ToUInt64(TB_IPAddress_2.Text.ToString(), 10) >= 0) && (Convert.ToUInt64(TB_IPAddress_2.Text.ToString(), 10) < 256))
                        {
                            if ((Convert.ToUInt64(TB_IPAddress_3.Text.ToString(), 10) >= 0) && (Convert.ToUInt64(TB_IPAddress_3.Text.ToString(), 10) < 256))
                            {
                                B_StartSample.Enabled = false;
                                MX_Common.Url = "http://" + TB_IPAddress_0.Text + "." + TB_IPAddress_1.Text + "." + TB_IPAddress_2.Text + "." + TB_IPAddress_3.Text + ":5555";
                                MSXE371x.Url = MX_Common.Url;

                                try
                                {
                                    // Get the number of transducers type

                                    Response = MSXE371x.TransducerGetNbrOfType(out ulNbrOfTransducers);
                                    if (Response.iReturnValue != 0)
                                    {
                                        MessageBox.Show("TransducerGetNbrOfType Error : " + Response.iReturnValue.ToString(), "Function error", MessageBoxButtons.OK, MessageBoxIcon.Exclamation);
                                        return;
                                    }
                                }

                                catch (Exception ex)
                                {
                                    MessageBox.Show("TransducerGetNbrOfType Exception occur : " + ex.Message, "Function error", MessageBoxButtons.OK, MessageBoxIcon.Exclamation);
                                    return;
                                }

                                TranducerInformations = new TranducerInfo[ulNbrOfTransducers];
                                ArrayList Transducers = new ArrayList();
                                L_NbrOfTransducers.Text = ulNbrOfTransducers.ToString();

                                for (ulong ulCpt = 0; ulCpt < ulNbrOfTransducers; ulCpt++)
                                {
                                    // Get the transducer informations
                                    MSXE371x.TransducerGetTypeInformation(ulCpt,
                                                                          out ulSelectionIndex,
                                                                          out pcName,
                                                                          out ulCalibrationStatus,
                                                                          out ulCalibratedChannels,
                                                                          out ulType,
                                                                          out ulFrequency,
                                                                          out ulImpedance,
                                                                          out dVeff,
                                                                          out dSensibility,
                                                                          out dRange);

                                    // Save the transducer informations
                                    TranducerInformations[ulCpt] = new TranducerInfo(ulSelectionIndex,
                                                                                     enc.GetString(pcName),
                                                                                     ulCalibrationStatus,
                                                                                     ulCalibratedChannels,
                                                                                     ulType,
                                                                                     ulFrequency,
                                                                                     ulImpedance,
                                                                                     dVeff,
                                                                                     dSensibility,
                                                                                     dRange);

                                    Transducers.Add(new ListData(enc.GetString(pcName), (long)ulSelectionIndex));
                                }

                                LB_TransducerList.DataSource = Transducers;
                                LB_TransducerList.DisplayMember = "Item";
                                LB_TransducerList.ValueMember = "Value";

                                // Get the module type
                                MXResponse = MX_Common.GetModuleType(out pcName);
                                ulNbrOfChannels = ulGetNbrOfTransducerChannel(enc.GetString(pcName));
                                if (enc.GetString(pcName).Contains("{2000}"))
                                    tcVersion = true;

                                // Thermocouple list box initialisation
                                ArrayList SensorType = new ArrayList();
                                if (tcVersion)
                                {
                                    SensorType.Add(new ListData("TC type B", 0)); //
                                    SensorType.Add(new ListData("TC type E", 1)); //
                                    SensorType.Add(new ListData("TC type J", 2)); //
                                    SensorType.Add(new ListData("TC type K", 3)); //
                                    SensorType.Add(new ListData("TC type N", 4)); //
                                    SensorType.Add(new ListData("TC type R", 5)); //
                                    SensorType.Add(new ListData("TC type S", 6)); //
                                    SensorType.Add(new ListData("TC type T", 7)); //
                                }
                                else
                                {
                                    SensorType.Add(new ListData("PT100", 100));
                                    SensorType.Add(new ListData("PT200", 200));
                                    SensorType.Add(new ListData("PT500", 500));
                                    SensorType.Add(new ListData("PT1000", 1000));
                                }

                                LB_T_Thermocouple.DataSource = SensorType;
                                LB_T_Thermocouple.SelectedIndex = 0;
                                LB_T_Thermocouple.DisplayMember = "Item";
                                LB_T_Thermocouple.ValueMember = "Value";

                                for (ulong i = 0; i < ulNbrOfChannels; i++)
                                {
                                    AutoRefreshChannels[i].Enabled = true;
                                    AutoRefreshChannels[i].Checked = true;
                                }

                                for (ulong i = ulNbrOfChannels; i < 8; i++)
                                {
                                    AutoRefreshChannels[i].Enabled = false;
                                    AutoRefreshChannels[i].Checked = false;
                                }

                                GB_Transducers.Enabled = true;
                                B_StartSample.Enabled = true;
                                GB_AutoRefresh.Enabled = true;
                                GB_DataFormat.Enabled = true;
                                GB_TriggerConfiguration.Enabled = true;
                                B_StartSample.Enabled = true;
                                GB_AcquisitionMode.Enabled = true;

                            }
                            else
                            {
                                MessageBox.Show("The IP address (part 4) you enter is not valid, please try again.", "IP address error", MessageBoxButtons.OK, MessageBoxIcon.Exclamation);
                            }
                        }
                        else
                        {
                            MessageBox.Show("The IP address (part 3) you enter is not valid, please try again.", "IP address error", MessageBoxButtons.OK, MessageBoxIcon.Exclamation);
                        }
                    }
                    else
                    {
                        MessageBox.Show("The IP address (part 2) you enter is not valid, please try again.", "IP address error", MessageBoxButtons.OK, MessageBoxIcon.Exclamation);
                    }
                }
                else
                {
                    MessageBox.Show("The IP address (part 1) you enter is not valid, please try again.", "IP address error", MessageBoxButtons.OK, MessageBoxIcon.Exclamation);
                }
            }
        }

        private bool boCheckConfiguration(out ulong ulChannelMask, out ulong ulDataFormat, out int iDataStreamSize)
        {
            ulong ulChannels = 0;

            ulChannelMask = 0;
            ulDataFormat = 0;
            iDataStreamSize = 0;

            // Check the channel mask
            alAcquisitionChannels = new ArrayList();
            for (int i = 0; i < 8; i++)
            {
                ulChannelMask = ulChannelMask | (Convert.ToUInt32(AutoRefreshChannels[i].Checked) << i);
                ulChannels = ulChannels + Convert.ToUInt32(AutoRefreshChannels[i].Checked);

                if (AutoRefreshChannels[i].Checked)
                {
                    alAcquisitionChannels.Add("Channel " + i.ToString() + ":");
                }
            }

            if (ulChannelMask == 0)
            {
                MessageBox.Show("No channel selected", "Configuration error", MessageBoxButtons.OK, MessageBoxIcon.Exclamation);
                return (false);
            }

            // Check the average value
            if ((TB_AverageValue.Enabled == true) && ((Convert.ToInt32(TB_AverageValue.Text) < 1) || (Convert.ToInt32(TB_AverageValue.Text) > 255)))
            {
                MessageBox.Show("Average value selection error (0 < value < 256)", "Configuration error", MessageBoxButtons.OK, MessageBoxIcon.Exclamation);
                return (false);
            }

            // Check the hardware trigger sequence number
            if ((TB_SequenceTrigger.Enabled == true) && ((Convert.ToInt64(TB_SequenceTrigger.Text) < 1) || (Convert.ToInt64(TB_SequenceTrigger.Text) > 0xFFFFFFFF)))
            {
                MessageBox.Show("Selection of sequence number to acquire for each trigger is wrong (0 < value <= 0xFFFFFFFF)", "Configuration error", MessageBoxButtons.OK, MessageBoxIcon.Exclamation);
                return (false);
            }

            // Check the counter compare configuration
            if ((Convert.ToUInt32(LB_TriggerSource.SelectedValue) == 4) && (Convert.ToInt32(LB_IC_CompareMode.SelectedValue) == -1))
            {
                MessageBox.Show("Counter compare logic used for trigger but counter compare not configurated", "Configuration error", MessageBoxButtons.OK, MessageBoxIcon.Exclamation);
                return (false);
            }

            if ((TB_IC_CompareValue.Enabled == true) && ((Convert.ToInt64(TB_IC_CompareValue.Text) < 0) || (Convert.ToInt64(TB_IC_CompareValue.Text) > 0xFFFFFFFF)))
            {
                MessageBox.Show("Counter compare value selection error (0 <= value <= 0xFFFFFFFF)", "Configuration error", MessageBoxButtons.OK, MessageBoxIcon.Exclamation);
                return (false);
            }

            // Get the data frame format
            for (int i = 0; i < DataFrame.Length; i++)
            {
                ulDataFormat = ulDataFormat | (Convert.ToUInt32(DataFrame[i].Checked) << i);
            }

            // Get the data strem size
            iDataStreamSize = (int)(4 + (ulChannels * 4) +
                                     (Convert.ToUInt32(CB_DataFrameTimeStamp.Checked) * 8) +
                                     (Convert.ToUInt32(CB_DataFrameTemperature.Checked) * 4) +
                                     (Convert.ToUInt32(CB_DataFrameIncCounter.Checked) * 4));

            return (true);
        }

        private void B_StartSample_Click(object sender, EventArgs e)
        {
            ulong ulDataFormat;
            Thread ClientThread;

            // Check the user configuration
            if (boCheckConfiguration(out ulChannelMask, out ulDataFormat, out iDataStreamSize) == true)
            {
                try
                {
                    SocketClient = null;
                    ClientThread = null;

                    // Test if receive data via the data server
                    if (RB_DataServer.Checked == true)
                    {
                        /* Create the client stream */
                        Client = new AddiData.ADDISocket.AsyncClientStream();

                        // DATA FORMAT CONFIGURATION
                        Client.SingleFramesSize = iDataStreamSize;
                        Client.NbrOfFrames = 1;


                        // Open the socket connection to receive the data
                        SocketClient = new Socket(AddressFamily.InterNetwork, SocketType.Stream, ProtocolType.Tcp);
                        SocketClient.Connect(TB_IPAddress_0.Text + "." + TB_IPAddress_1.Text + "." + TB_IPAddress_2.Text + "." + TB_IPAddress_3.Text, 8989);
                        SocketClient.ReceiveBufferSize = 1048576;

                        // Start the first read socket 
                        // Start the read socket and the user thread
                        Client.Start(SocketClient);
                        ClientThread = new Thread(ClientThreadFct);
                    }

                    DisplayText = "";


                    ulTFormat = Convert.ToUInt32(LB_T_Format.SelectedValue);

                    // Extern temperature initialisation
                    if (bo_ThermocoupleInitialisation() == true)
                    {
                        // Counter configuration
                        if (bo_CounterInitialisation() == true)
                        {
                            try
                            {
                                //Init and start the auto refresh acquisition
                                RetValue = MSXE371x.TransducerInitAndStartAutoRefresh(Convert.ToUInt64(LB_TransducerList.SelectedValue),
                                                                                      ulChannelMask,
                                                                                      Convert.ToUInt64(LB_AverageMode.SelectedValue),
                                                                                      Convert.ToUInt64(TB_AverageValue.Text),
                                                                                      Convert.ToUInt64(LB_TriggerSource.SelectedValue),
                                                                                      Convert.ToUInt64(LB_TriggerMode.SelectedValue),
                                                                                      Convert.ToUInt64(LB_HWEdge.SelectedValue),
                                                                                      Convert.ToUInt64(TB_HTriggerCount.Text),
                                                                                      Convert.ToUInt64(TB_SequenceTrigger.Text),
                                                                                      ulDataFormat,
                                                                                      0, 0, 0, 0, out syserno);


                                if (RetValue != 0)
                                {
                                    MessageBox.Show("TransducerInitAndStartAutoRefresh Error : " + RetValue.ToString(), "Function error", MessageBoxButtons.OK, MessageBoxIcon.Exclamation);
                                }
                                else
                                {
                                    // Test if receive data via the data server
                                    if (RB_DataServer.Checked == true)
                                    {
                                        ClientThread.Priority = ThreadPriority.Lowest;
                                        ClientThread.Start();
                                    }
                                    else
                                    {
                                        ReadTimer.Enabled = true;
                                    }

                                    DisplayTimer.Enabled = true;
                                    boAutoRefreshInit = true;
                                    boSaveGB_Thermocouple = GB_Thermocouple.Enabled;
                                    boSaveGB_Counter = GB_Counter.Enabled;
                                    boSaveGB_CounterCompare = GB_CounterCompare.Enabled;
                                    B_Connect.Enabled = false;
                                    GB_Transducers.Enabled = false;
                                    B_StartSample.Enabled = false;
                                    GB_AutoRefresh.Enabled = false;
                                    GB_DataFormat.Enabled = false;
                                    GB_TriggerConfiguration.Enabled = false;
                                    GB_Thermocouple.Enabled = false;
                                    GB_Counter.Enabled = false;
                                    GB_CounterCompare.Enabled = false;
                                    B_StopSample.Enabled = true;
                                    GB_AcquisitionMode.Enabled = false;
                                }
                            }

                            catch (Exception ex)
                            {
                                MessageBox.Show("TransducerInitAndStartAutoRefresh Exception occur : " + ex.Message, "Function error", MessageBoxButtons.OK, MessageBoxIcon.Exclamation);
                            }
                        }
                    }
                }
                catch (Exception ex)
                {
                    MessageBox.Show("Socket connection exception occur : " + ex.Message, "Function error", MessageBoxButtons.OK, MessageBoxIcon.Exclamation);
                }

                // Test if error occur
                if (boAutoRefreshInit == false)
                {
                    B_StopSample_Click(sender, e);
                }
            }
        }

        private void B_StopSample_Click(object sender, EventArgs e)
        {

            DisplayTimer.Enabled = false;
            ReadTimer.Enabled = false;

            B_StartSample.Enabled = !B_StopSample.Enabled;
            B_StopSample.Enabled = false;

            // Test if socket open
            if (SocketClient != null)
            {
                if (Client != null)
                    Client.Stoop();

                try
                {
                    // Close the socket connection
                    SocketClient.Disconnect(false);
                    SocketClient.Close();
                    SocketClient = null;
                    System.GC.Collect();
                }
                catch (Exception ex)
                {
                    MessageBox.Show("Socket close connection exception occur : " + ex.Message, "Function error", MessageBoxButtons.OK, MessageBoxIcon.Exclamation);
                }
            }

            // Test if auto refresh started
            if (boAutoRefreshInit == true)
            {
                boAutoRefreshInit = false;
                B_Connect.Enabled = true;
                GB_Transducers.Enabled = true;
                B_StartSample.Enabled = true;
                GB_AutoRefresh.Enabled = true;
                GB_DataFormat.Enabled = true;
                GB_TriggerConfiguration.Enabled = true;
                GB_AcquisitionMode.Enabled = true;
                GB_Thermocouple.Enabled = boSaveGB_Thermocouple;
                GB_Counter.Enabled = boSaveGB_Counter;
                GB_CounterCompare.Enabled = boSaveGB_CounterCompare;

                try
                {
                    // Release the counter compare logic

                    RetValue = MSXE371x.TransducerStopAndReleaseAutoRefresh(out syserno);
                    if (RetValue != 0)
                    {
                        MessageBox.Show("TransducerStopAndReleaseAutoRefresh Error : " + RetValue.ToString(), "Function error", MessageBoxButtons.OK, MessageBoxIcon.Exclamation);
                    }
                }

                catch (Exception ex)
                {
                    MessageBox.Show("TransducerStopAndReleaseAutoRefresh Exception occur : " + ex.Message, "Function error", MessageBoxButtons.OK, MessageBoxIcon.Exclamation);
                }
            }

            bo_ThermocoupleRelease();
            bo_CounterRelease();
        }


        private void ReadTimer_Tick(object sender, EventArgs e)
        {
            ulong[] Values = new ulong[8];
            String sOutString;
            int Cpt = 0;
            int iChannelsCpt = 0;
            ulong TestChannel = 0;
            ulong TimeStampLow = 0;
            ulong TimeStampHigh = 0;
            ulong TimeStampLowmus = 0;
            ulong TimeStampLowms = 0;
            ulong IncCounter = 0;
            ulong CounterValue = 0;
            ulong ExternalTemperature = 0;
            MSXE371x_WebReference.DefaultResponse pResponse = new MSXE371x_WebReference.DefaultResponse();

            // Read the auto refresh value
            pResponse = MSXE371x.TransducerGetAllAutoRefreshValues(out CounterValue, out Values, out TimeStampLow, out TimeStampHigh, out IncCounter, out ExternalTemperature);

            // Test if error occur
            if ((pResponse.iReturnValue != 0) || (pResponse.syserrno != 0))
            {
                B_StopSample_Click(sender, null);
                MessageBox.Show("TransducerGetAllAutoRefreshValues Error : " + pResponse.iReturnValue.ToString(), "Function error", MessageBoxButtons.OK, MessageBoxIcon.Exclamation);
            }
            else
            {
                sOutString = "";

                // Test if time stampe
                if (CB_DataFrameTimeStamp.Checked == true)
                {
                    TimeStampLowms = (ulong)(TimeStampLow / 1000);
                    TimeStampLowmus = (ulong)(TimeStampLow % 1000);
                    sOutString = "Time stamp: " + ConvertFromTime_T((long)TimeStampHigh).ToString() + "." + TimeStampLowms.ToString("D3") + "." + TimeStampLowmus.ToString("D3");
                }

                // Test if external temperature used
                if (CB_DataFrameTemperature.Checked == true)
                {
                    if (sOutString != "")
                    {
                        sOutString = sOutString + " / ";
                    }
                    // Test if mOhm
                    if (ulTFormat == 0)
                    {
                        sOutString = sOutString + "External temperature: " + ExternalTemperature.ToString() + " mOhm";
                    }
                    // Test if C
                    else if (ulTFormat == 1)
                    {
                        sOutString = sOutString + "External temperature: " + BitConverter.ToSingle(BitConverter.GetBytes(ExternalTemperature), 0).ToString("F3") + " C";
                    }
                    // F
                    else
                    {
                        sOutString = sOutString + "External temperature: " + BitConverter.ToSingle(BitConverter.GetBytes(ExternalTemperature), 0).ToString("F3") + " F";
                    }
                }

                // Test if incremental counter used
                if (CB_DataFrameIncCounter.Checked == true)
                {
                    if (sOutString != "")
                    {
                        sOutString = sOutString + " / ";
                    }

                    sOutString = sOutString + "Incremental counter : " + IncCounter.ToString() + " Dec";
                }

                if (sOutString != "")
                {
                    sOutString = sOutString + Environment.NewLine;
                }

                // Get the auto refresh counter value
                sOutString = sOutString + "Auto refresh counter: " + CounterValue.ToString();

                // Get the auto refresh channel mask
                for (Cpt = 0, TestChannel = 1, iChannelsCpt = 0; Cpt < 8; Cpt++, TestChannel = TestChannel * 2)
                {
                    // Test if channel used
                    if ((ulChannelMask & TestChannel) != 0)
                    {
                        if ((iChannelsCpt % 4) == 0)
                        {
                            sOutString = sOutString + Environment.NewLine;
                        }

                        // Test if mm
                        if (CB_DataFrameMetric.Checked == true)
                        {
                            if (Values[Cpt] > 0x7FFFFF)
                            {
                            }
                            sOutString = sOutString + alAcquisitionChannels[iChannelsCpt] + BitConverter.ToSingle(BitConverter.GetBytes(Values[Cpt]), 0).ToString("F5") + " mm  ";
                        }
                        else
                        {
                          /*  ulong offset = (((((0x7FFFFF - Values[Cpt]) / 466) * 72) / 18000) * 466);
                            double test = 0;
                            //Values[Cpt] = Values[Cpt] - offset;
                            if (Values[Cpt] < 0x7FFFFF)
                                test = ((double) (0x7FFFFF - Values[Cpt] - offset) * 180.0 / (double)0x7FFFFF) / 1000.0;
                            else
                                test = ((double)(Values[Cpt]- 0x7FFFFF) * 180.0 / (double)0x7FFFFF) / 1000.0;
                           // test = test - offset;*/
                            sOutString = sOutString + alAcquisitionChannels[iChannelsCpt] + Values[Cpt].ToString("F5") + " Hex  ";
                        }

                        iChannelsCpt++;
                    }
                }
                DisplayText = sOutString;
            }
        }

        private void ClientThreadFct()
        {
            String sOutString;
            String sAutoRefresh;
            int SocketDataIndex;
            byte[] SocketData = new byte[Client.SingleFramesSize];

            while ((SocketData = Client.ReadData()) != null)
            {
                sOutString = "";
                SocketDataIndex = 0;

                // Test if time stampe
                if (CB_DataFrameTimeStamp.Checked == true)
                {
                    sOutString = "Time stamp: " + ConvertFromTime_T(SocketData, SocketDataIndex);
                    SocketDataIndex = SocketDataIndex + 8;
                }

                // Get the auto refresh counter value
                sAutoRefresh = "Auto refresh counter: " + BitConverter.ToUInt32(SocketData, SocketDataIndex).ToString();
                SocketDataIndex = SocketDataIndex + 4;

                // Test if external temperature used
                if (CB_DataFrameTemperature.Checked == true)
                {
                    if (sOutString != "")
                    {
                        sOutString = sOutString + " / ";
                    }
                    // Test if mOhm
                    if (ulTFormat == 0)
                    {
                        sOutString = sOutString + "External temperature: " + BitConverter.ToUInt32(SocketData, SocketDataIndex).ToString() + " mOhm";
                    }
                    // Test if C
                    else if (ulTFormat == 1)
                    {
                        sOutString = sOutString + "External temperature: " + BitConverter.ToSingle(SocketData, SocketDataIndex).ToString("F3") + " C";
                    }
                    // F
                    else
                    {
                        sOutString = sOutString + "External temperature: " + BitConverter.ToSingle(SocketData, SocketDataIndex).ToString("F3") + " F";
                    }

                    SocketDataIndex = SocketDataIndex + 4;
                }

                // Test if incremental counter used
                if (CB_DataFrameIncCounter.Checked == true)
                {
                    if (sOutString != "")
                    {
                        sOutString = sOutString + " / ";
                    }

                    sOutString = sOutString + "Incremental counter : " + BitConverter.ToUInt32(SocketData, SocketDataIndex).ToString() + " Dec";
                    SocketDataIndex = SocketDataIndex + 4;
                }

                if (sOutString != "")
                {
                    sOutString = sOutString + Environment.NewLine + sAutoRefresh;
                }
                else
                {
                    sOutString = sAutoRefresh;
                }

                // Get the transducer channels value
                int iChannelsCpt = 0;
                foreach (String Channel in alAcquisitionChannels)
                {
                    if ((iChannelsCpt % 4) == 0)
                    {
                        sOutString = sOutString + Environment.NewLine;
                    }

                    // Test if mm
                    if (CB_DataFrameMetric.Checked == true)
                    {
                        sOutString = sOutString + Channel + BitConverter.ToSingle(SocketData, SocketDataIndex).ToString("F3") + " mm  ";
                    }
                    else
                    {
                        sOutString = sOutString + Channel + BitConverter.ToUInt32(SocketData, SocketDataIndex).ToString("X") + " Hex  ";
                    }
                    SocketDataIndex = SocketDataIndex + 4;
                    iChannelsCpt++;
                }

                DisplayText = sOutString;
            }
        }

        private void TransducerAutoRefresh_FormClosing(object sender, FormClosingEventArgs e)
        {
            B_StopSample_Click(sender, null);
        }

        private void RB_SOAP_CheckedChanged(object sender, EventArgs e)
        {
            L_Info.Visible = false;
        }

        private void RB_DataServer_CheckedChanged(object sender, EventArgs e)
        {
            L_Info.Visible = true;
        }

        private void DisplayTimer_Tick(object sender, EventArgs e)
        {
            L_AutoRefreshInfo.Text = DisplayText;
        }


        /// <summary>
        /// Event is called whenn the selected temperature sensor is changed
        /// if TC is selected then acqusition frequencies change and display format also change
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>        
        private void LB_T_Thermocouple_SelectedIndexChanged(object sender, EventArgs e)
        {
            //Display the temperature
            if (tcVersion)
            {
                // Display wihtout mOm
                ArrayList Formats_withoutMOhm = new ArrayList();
                Formats_withoutMOhm.Add(new ListData("C", 1));
                Formats_withoutMOhm.Add(new ListData("F", 2));
                LB_T_Format.DataSource = Formats_withoutMOhm;
            }
            else
            {
                // Display with mOm
                ArrayList Formats_witMOhm = new ArrayList();
                Formats_witMOhm.Add(new ListData("mOhm", 0));
                Formats_witMOhm.Add(new ListData("C", 1));
                Formats_witMOhm.Add(new ListData("F", 2));
                LB_T_Format.DataSource = Formats_witMOhm;
            }


            //Frequency
            if (tcVersion)
            {
                // Display wihtout 10Hz mOm
                ArrayList Frequencys = new ArrayList();
                Frequencys.Add(new ListData("50 Hz", 50));
                Frequencys.Add(new ListData("240 Hz", 240));
                LB_T_Frequency.DataSource = Frequencys;
            }
            else
            {
                // Display with 10Hz
                ArrayList Frequencys = new ArrayList();
                Frequencys.Add(new ListData("10 Hz", 10));
                Frequencys.Add(new ListData("50 Hz", 50));
                Frequencys.Add(new ListData("240 Hz", 240));
                LB_T_Frequency.DataSource = Frequencys;
            }
        }

    }
}