using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using System.Collections;
using System.Net;
using System.Net.Sockets;

namespace TransducerCalibration
{
    public partial class TransducerCalibration : Form
    {
        // Global Variable
        private MXCommon_WebReference.MXCommon MX_Common;
        private MSXE371x_WebReference.MSXE371x MSXE371x;
        private MXCommon_WebReference.DefaultResponse MXResponse;
        private MSXE371x_WebReference.DefaultResponse Response;
        private int RetValue = 0;
        private int syserno = 0;
        private UInt64 ulLastStatus = 0;
        double  dRange              = 0;
        double  dPosition           = 0;
        private TranducerInfo[] TranducerInformations;


        private UInt64 ulGetNbrOfTransducerChannel(String sModuleType)
        {
            if (sModuleType.IndexOf("-", 0) == -1)
                return 0;
            sModuleType = sModuleType.Substring(sModuleType.IndexOf("-", 0) + 1);

            if (sModuleType.IndexOf("-", 0) == -1)
                return 0;
            sModuleType = sModuleType.Substring(sModuleType.IndexOf("-", 0) + 1);

            if (sModuleType.IndexOf("-", 0) == -1)
                return 0;
            sModuleType = sModuleType.Remove(sModuleType.IndexOf("-", 0));

            return (Convert.ToUInt64(sModuleType));
        }

        public class TranducerInfo
        {
            private ulong ulSelectionIndex;
            private String sName;
            private ulong ulCalibrationStatus;
            private ulong ulCalibratedChannels;
            private ulong ulType;
            private ulong ulFrequency;
            private ulong ulImpedance;
            private double dVeff;
            private double dSensibility;
            private double dRange;

            public TranducerInfo(ulong SelectionIndex,
                                 String Name,
                                 ulong CalibrationStatus,
                                 ulong CalibratedChannels,
                                 ulong Type,
                                 ulong Frequency,
                                 ulong Impedance,
                                 double Veff,
                                 double Sensibility,
                                 double Range)
            {
                ulSelectionIndex = SelectionIndex;
                sName = Name.Remove(Name.IndexOf((char)0));
                ulCalibrationStatus = CalibrationStatus;
                ulCalibratedChannels = CalibratedChannels;
                ulType = Type;
                ulFrequency = Frequency;
                ulImpedance = Impedance;
                dVeff = Veff;
                dSensibility = Sensibility;
                dRange = Range;
            }

            public ulong SelectionIndex { get { return (ulSelectionIndex); } set { ulSelectionIndex = value; } }
            public ulong CalibrationStatus { get { return (ulCalibrationStatus); } set { ulCalibrationStatus = value; } }
            public ulong CalibratedChannels { get { return (ulCalibratedChannels); } set { ulCalibratedChannels = value; } }
            public ulong Type { get { return (ulType); } set { ulType = value; } }
            public ulong Frequency { get { return (ulFrequency); } set { ulFrequency = value; } }
            public ulong Impedance { get { return (ulImpedance); } set { ulImpedance = value; } }
            public double Veff { get { return (dVeff); } set { dVeff = value; } }
            public double Sensibility { get { return (dSensibility); } set { dSensibility = value; } }
            public double Range { get { return (dRange); } set { dRange = value; } }
            public String Name { get { return (sName); } set { sName = value; } }

            public override String ToString()
            {
                String sRet;

                sRet = "Transducer Selection Index : " + ulSelectionIndex.ToString();
                sRet = sRet + Environment.NewLine + "Name                                   : " + sName;
                sRet = sRet + Environment.NewLine + "Calibration status                  : " + ulCalibrationStatus.ToString();
                sRet = sRet + Environment.NewLine + "Type Value                          : " + ulType.ToString();
                sRet = sRet + Environment.NewLine + "Frequency                           : " + ulFrequency.ToString() + " Hz";
                sRet = sRet + Environment.NewLine + "Impedance                          : " + ulImpedance.ToString() + " Ohm";
                sRet = sRet + Environment.NewLine + "Nominal Voltage                  : " + dVeff.ToString() + " V";
                sRet = sRet + Environment.NewLine + "Sensibility                            : " + dSensibility.ToString() + " mV/V/mm";
                sRet = sRet + Environment.NewLine + "Range                                 : +/-" + dRange.ToString() + " mm";

                return (sRet);
            }
        }

        public class ListData
        {
            private string myItem;
            private long myValue;

            public ListData(string strItem, long lValue)
            {

                this.myItem = strItem;
                this.myValue = lValue;
            }

            public string Item
            {
                get { return myItem; }
            }

            public long Value
            {

                get { return myValue; }
            }
        }

        public TransducerCalibration()
        {
            InitializeComponent();
        }

        private void TransducerCalibration_Load(object sender, EventArgs e)
        {
            MX_Common = new MXCommon_WebReference.MXCommon();
            MSXE371x = new MSXE371x_WebReference.MSXE371x();
            MX_Common.Timeout = 5000;
            MSXE371x.Timeout = 5000;
            TB_CalibrationStatus.ReadOnly = true;
            

        }


        private void LB_TransducerList_SelectedIndexChanged(object sender, EventArgs e)
        {
            L_TransducerInfo.Text = TranducerInformations[LB_TransducerList.SelectedIndex].ToString();

        }

        private void B_Connect_Click(object sender, EventArgs e)
        {
            System.Text.ASCIIEncoding enc = new System.Text.ASCIIEncoding();
            byte[] pcName = new byte[255];
            ulong ulNbrOfTransducers;
            ulong ulNbrOfChannels;
            ulong ulSelectionIndex;
            ulong ulCalibrationStatus;
            ulong ulCalibratedChannels;
            ulong ulType;
            ulong ulFrequency;
            ulong ulImpedance;
            double dVeff;
            double dSensibility;
            double dRange;

            if ((TB_IPAddress_0.Text.ToString() == "") || (TB_IPAddress_1.Text.ToString() == "") || (TB_IPAddress_2.Text.ToString() == "") || (TB_IPAddress_3.Text.ToString() == ""))
            {
                MessageBox.Show("The IP address you enter is not valid, please try again.", "IP address error", MessageBoxButtons.OK, MessageBoxIcon.Exclamation);
            }
            else
            {
                if ((Convert.ToUInt64(TB_IPAddress_0.Text.ToString(), 10) >= 0) && (Convert.ToUInt64(TB_IPAddress_0.Text.ToString(), 10) < 256))
                {
                    if ((Convert.ToUInt64(TB_IPAddress_1.Text.ToString(), 10) >= 0) && (Convert.ToUInt64(TB_IPAddress_1.Text.ToString(), 10) < 256))
                    {
                        if ((Convert.ToUInt64(TB_IPAddress_2.Text.ToString(), 10) >= 0) && (Convert.ToUInt64(TB_IPAddress_2.Text.ToString(), 10) < 256))
                        {
                            if ((Convert.ToUInt64(TB_IPAddress_3.Text.ToString(), 10) >= 0) && (Convert.ToUInt64(TB_IPAddress_3.Text.ToString(), 10) < 256))
                            {
                                B_StartCalibration.Enabled = false;
                                MX_Common.Url = "http://" + TB_IPAddress_0.Text + "." + TB_IPAddress_1.Text + "." + TB_IPAddress_2.Text + "." + TB_IPAddress_3.Text + ":5555";
                                MSXE371x.Url = MX_Common.Url;

                                try
                                {
                                    // Get the number of transducers type

                                    Response = MSXE371x.TransducerGetNbrOfType(out ulNbrOfTransducers);
                                    if (Response.iReturnValue != 0)
                                    {
                                        MessageBox.Show("TransducerGetNbrOfType Error : " + Response.iReturnValue.ToString(), "Function error", MessageBoxButtons.OK, MessageBoxIcon.Exclamation);
                                        return;
                                    }
                                }

                                catch (Exception ex)
                                {
                                    MessageBox.Show("TransducerGetNbrOfType Exception occur : " + ex.Message, "Function error", MessageBoxButtons.OK, MessageBoxIcon.Exclamation);
                                    return;
                                }

                                TranducerInformations = new TranducerInfo[ulNbrOfTransducers];
                                ArrayList Transducers = new ArrayList();
                                L_NbrOfTransducers.Text = ulNbrOfTransducers.ToString();

                                for (ulong ulCpt = 0; ulCpt < ulNbrOfTransducers; ulCpt++)
                                {
                                    // Get the transducer informations
                                    MSXE371x.TransducerGetTypeInformation(ulCpt,
                                                                          out ulSelectionIndex,
                                                                          out pcName,
                                                                          out ulCalibrationStatus,
                                                                          out ulCalibratedChannels,
                                                                          out ulType,
                                                                          out ulFrequency,
                                                                          out ulImpedance,
                                                                          out dVeff,
                                                                          out dSensibility,
                                                                          out dRange);

                                    // Save the transducer informations
                                    TranducerInformations[ulCpt] = new TranducerInfo(ulSelectionIndex,
                                                                                     enc.GetString(pcName),
                                                                                     ulCalibrationStatus,
                                                                                     ulCalibratedChannels,
                                                                                     ulType,
                                                                                     ulFrequency,
                                                                                     ulImpedance,
                                                                                     dVeff,
                                                                                     dSensibility,
                                                                                     dRange);

                                    Transducers.Add(new ListData(enc.GetString(pcName), (long)ulSelectionIndex));
                                }

                                LB_TransducerList.DataSource = Transducers;
                                LB_TransducerList.DisplayMember = "Item";
                                LB_TransducerList.ValueMember = "Value";

                                // Get the module type
                                MXResponse = MX_Common.GetModuleType(out pcName);

                                if ((enc.GetString(pcName).Contains("2-HB")) ||
                                    (enc.GetString(pcName).Contains("HB-2")) ||
                                    (enc.GetString(pcName).Contains("LVDT-2")) ||
                                    (enc.GetString(pcName).Contains("M-2")))
                                    resetCalibrationDefaultValuesCheckbox.Visible = true;

                                ulNbrOfChannels = ulGetNbrOfTransducerChannel(enc.GetString(pcName));
                                
                                // Channel list box initialisation
                                ArrayList Channels = new ArrayList();
                                Channels.Add(new ListData("All", 255));

                                for (long i = 0; i < (long)ulNbrOfChannels; i++)
                                {
                                    Channels.Add(new ListData(i.ToString(), i));
                                }

                                LB_Channel.DisplayMember = "Item";
                                LB_Channel.ValueMember = "Value";
                                LB_Channel.DataSource = Channels;

                                GB_Transducers.Enabled = true;
                                B_StartCalibration.Enabled = true;

                            }
                            else
                            {
                                MessageBox.Show("The IP address (part 4) you enter is not valid, please try again.", "IP address error", MessageBoxButtons.OK, MessageBoxIcon.Exclamation);
                            }
                        }
                        else
                        {
                            MessageBox.Show("The IP address (part 3) you enter is not valid, please try again.", "IP address error", MessageBoxButtons.OK, MessageBoxIcon.Exclamation);
                        }
                    }
                    else
                    {
                        MessageBox.Show("The IP address (part 2) you enter is not valid, please try again.", "IP address error", MessageBoxButtons.OK, MessageBoxIcon.Exclamation);
                    }
                }
                else
                {
                    MessageBox.Show("The IP address (part 1) you enter is not valid, please try again.", "IP address error", MessageBoxButtons.OK, MessageBoxIcon.Exclamation);
                }
            }
        }



        private void B_StartCalibration_Click(object sender, EventArgs e)
        {
            ulong ulCalibrationMode = 0;

            try
            {

                ulLastStatus = 0;
                TB_CalibrationStatus.Text = "";
                L_CalibrationValue.Text = "";
                
                dRange = TranducerInformations[LB_TransducerList.SelectedIndex].Range;

                if (userPositionCheckBox.Enabled)
                {
                    dPosition = Convert.ToDouble(userPositionTextBox.Text) / 1000;
                }
                else
                {
                    dPosition = dRange / 2;
                }

                // Get the calibration mmode
                if (Convert.ToInt32(LB_Channel.SelectedValue) != 255)
                {
                    ulCalibrationMode = 1;
                }

                if (resetCalibrationDefaultValuesCheckbox.Checked)
                {
                    ulCalibrationMode |= 2;
                }

                if ((dPosition > dRange) || (dPosition < -dRange))
                {
                    MessageBox.Show("CalibrationStart error, invalid user position, must be between " + -dRange + " mm and " + dRange + " mm", "Parameter error", MessageBoxButtons.OK, MessageBoxIcon.Exclamation);
                    return;
                }

                //Init and start the calibration
                RetValue = MSXE371x.CalibrationStart(Convert.ToUInt32(LB_TransducerList.SelectedValue),
                                                     dPosition,
                                                     ulCalibrationMode,
                                                     Convert.ToUInt32(CB_Invert.Checked),
                                                     Convert.ToUInt32(LB_Channel.SelectedValue),
                                                     out syserno);

                if (RetValue != 0)
                {
                    MessageBox.Show("CalibrationStart Error : " + RetValue.ToString(), "Function error", MessageBoxButtons.OK, MessageBoxIcon.Exclamation);
                }
                else
                {
                    B_Connect.Enabled = false;
                    GB_Transducers.Enabled = false;
                    B_StartCalibration.Enabled = false;
                    B_NextStep.Enabled = false;
                    LB_Channel.Enabled = false;
                    CB_Invert.Enabled = false;
                    B_BreakCalibration.Enabled = true;
                    T_Calibration.Enabled = true;
                    resetCalibrationDefaultValuesCheckbox.Enabled = false;
                    userPositionCheckBox.Enabled = false;
                    userPositionTextBox.Enabled = false;
                }
            }

            catch (Exception ex)
            {
                MessageBox.Show("CalibrationStart Exception occur : " + ex.Message, "Function error", MessageBoxButtons.OK, MessageBoxIcon.Exclamation);
            }
        }

        private void B_NextStep_Click(object sender, EventArgs e)
        {
            B_NextStep.Enabled = false;
            T_Color.Enabled = false;
            B_NextStep.ForeColor = Color.Black;

            try
            {
                //Init and start the calibration

                RetValue = MSXE371x.CalibrationNextStep(out syserno);

                if (RetValue != 0)
                {
                    MessageBox.Show("CalibrationNextStep Error : " + RetValue.ToString(), "Function error", MessageBoxButtons.OK, MessageBoxIcon.Exclamation);
                }
            }

            catch (Exception ex)
            {
                MessageBox.Show("CalibrationNextStep Exception occur : " + ex.Message, "Function error", MessageBoxButtons.OK, MessageBoxIcon.Exclamation);
            }
        }

        private void B_BreakCalibration_Click(object sender, EventArgs e)
        {
            if (B_BreakCalibration.Enabled == true)
            {
                try
                {
                    //Init and start the calibration

                    RetValue = MSXE371x.CalibrationBreak(out syserno);

                    if (RetValue != 0)
                    {
                        MessageBox.Show("CalibrationBreak Error : " + RetValue.ToString(), "Function error", MessageBoxButtons.OK, MessageBoxIcon.Exclamation);
                    }
                }

                catch (Exception ex)
                {
                    MessageBox.Show("CalibrationBreak Exception occur : " + ex.Message, "Function error", MessageBoxButtons.OK, MessageBoxIcon.Exclamation);
                }
            }
        }


        private void T_Calibration_Tick(object sender, EventArgs e)
        {
            UInt64 ulStatus       = 0;
            UInt64 ulDigitalValue = 0;
            UInt64 ulChannel      = 0;
            double dChannelValue  = 0;
            System.Byte[] b_SaveResponse = new System.Byte[200];

            try
            {
                // Get calibration current status
                Response = MSXE371x.CalibrationGetCurrentStatus(out ulStatus, out ulChannel, out ulDigitalValue);

                // Test if error occur

                if (Response.iReturnValue != 0)
                {
                    MessageBox.Show("CalibrationGetCurrentStatus Error : " + Response.iReturnValue.ToString(), "Function error", MessageBoxButtons.OK, MessageBoxIcon.Exclamation);
                    return;
                }
            }

            catch (Exception ex)
            {
                MessageBox.Show("CalibrationGetCurrentStatus Exception occur : " + ex.Message, "Function error", MessageBoxButtons.OK, MessageBoxIcon.Exclamation);
                return;
            }

            //Test if start primary calibration 
            if (((ulStatus == 1) || ((ulStatus == 2))) && (ulLastStatus == 0))
            {
                TB_CalibrationStatus.Text = "Primary calibration started" + Environment.NewLine;

                if (ulStatus == 2)
                {
                    ulLastStatus = 1;
                }
            }

            // Test if primary calibration started 
            if ((ulStatus == 1) && (ulLastStatus == 1))
            {
                // Calculate V 
                dChannelValue = (((2.8 * 2.0) / (double)0xFFFFFF) * (double)ulDigitalValue) - 2.8;

                L_CalibrationValue.Text = "Primary calibration : " + dChannelValue.ToString("F") + "V";
            }

            // Test if end primary calibration 
            if ((ulStatus == 2) && (ulLastStatus != 2))
            {
                TB_CalibrationStatus.Text = TB_CalibrationStatus.Text + "Primary calibration complete" + Environment.NewLine;
                TB_CalibrationStatus.Text = TB_CalibrationStatus.Text + "Set the transducer connected to channel " + ulChannel.ToString() + " to position 0 mm and click next" + Environment.NewLine;
                B_NextStep.Enabled = true;
                T_Color.Enabled = true;
            }

            // Test if wait user set transducer to 0mm 
            if ((ulStatus == 2) && (ulLastStatus == 2))
            {
                // Calculate the position into mm 
                dChannelValue = (((dRange * 2.0) / (double)0xFFFFFF) * (double)ulDigitalValue) - dRange;

                L_CalibrationValue.Text = "Transducer position = " + dChannelValue.ToString("F6") + " mm";
            }

            // Test if start null position calibration 
            if ((ulStatus == 3) && (ulLastStatus != 3))
            {
                TB_CalibrationStatus.Text = TB_CalibrationStatus.Text + "Null position calibration started" + Environment.NewLine;
            }

            // Test if null position calibration 
            if ((ulStatus == 3) && (ulLastStatus == 3))
            {
                // Calculate the position into mm 
                dChannelValue = (((dRange * 2.0) / (double)0xFFFFFF) * (double)ulDigitalValue) - dRange;

                L_CalibrationValue.Text = "Transducer position = " + dChannelValue.ToString("F6") + " mm";
            }

            // Test if end user position calibration 
            if ((ulStatus == 4) && (ulLastStatus != 4))
            {
                if (ulLastStatus != 0)
                {
                    TB_CalibrationStatus.Text = TB_CalibrationStatus.Text + "Null position calibration complete" + Environment.NewLine;
                }
                TB_CalibrationStatus.Text = TB_CalibrationStatus.Text + "Set the transducer connected to channel " + ulChannel.ToString() + " to position " + dPosition.ToString("G4") + "mm and click next" + Environment.NewLine;
                B_NextStep.Enabled = true;
                T_Color.Enabled = true;
            }

            // Test if wait user set transducer to user position 
            if ((ulStatus == 4) && (ulLastStatus == 4))
            {
                // Calculate the position into mm 
                dChannelValue = (((dRange * 2.0) / (double)0xFFFFFF) * (double)ulDigitalValue) - dRange;

                L_CalibrationValue.Text = "Transducer position = " + dChannelValue.ToString("F6") + " mm";
            }

            // Test if start user position calibration 
            if ((ulStatus == 5) && (ulLastStatus != 5))
            {
                TB_CalibrationStatus.Text = TB_CalibrationStatus.Text + "User position calibration started" + Environment.NewLine;
            }

            // Test if user position calibration 
            if ((ulStatus == 5) && (ulLastStatus == 5))
            {
                // Calculate the position into mm 
                dChannelValue = (((dRange * 2.0) / (double)0xFFFFFF) * (double)ulDigitalValue) - dRange;

                L_CalibrationValue.Text = "Transducer position = " + dChannelValue.ToString("F6") + " mm";
            }

            // Test if end user position calibration 
            if ((ulStatus == 6) && (ulLastStatus != 6))
            {
                TB_CalibrationStatus.Text = TB_CalibrationStatus.Text + "User position calibration complete" + Environment.NewLine;
                TB_CalibrationStatus.Text = TB_CalibrationStatus.Text + "Connect only one transducer and click next" + Environment.NewLine;
                B_NextStep.Enabled = true;
                T_Color.Enabled = true;
            }


            // Test if wait that user connect only one transducer 
            if ((ulStatus == 7) && (ulLastStatus != 7))
            {
                TB_CalibrationStatus.Text = TB_CalibrationStatus.Text + "Primary open line calibration started";
            }


            // Test if primary open line test calibration
            if ((ulStatus == 7) && (ulLastStatus == 7))
            {
                // Calculate V 
                dChannelValue = (((2.8 * 2.0) / (double)0xFFFFFF) * (double)ulDigitalValue) - 2.8;

                L_CalibrationValue.Text = "Primary open line calibration : " + dChannelValue.ToString("F") + "V";
            }

            if ((ulStatus == 9) && (ulLastStatus != 9))
            {
                TB_CalibrationStatus.Text = TB_CalibrationStatus.Text + "Set the transducer connected to channel " + ulChannel.ToString() + " to position " + (dPosition * -1).ToString("G4") + "mm and click next" + Environment.NewLine;
                B_NextStep.Enabled = true;
                T_Color.Enabled = true;
            }

            // Test if wait user set transducer to inverted user position 
            if ((ulStatus == 9) && (ulLastStatus == 9))
            {
                // Calculate the position into mm 
                dChannelValue = (((dRange * 2.0) / (double)0xFFFFFF) * (double)ulDigitalValue) - dRange;

                L_CalibrationValue.Text = "Transducer position = " + dChannelValue.ToString("F6") + " mm";
            }

            // Test if start inverted user calibration 
            if ((ulStatus == 10) && (ulLastStatus != 10))
            {
                // Calculate V 
                dChannelValue = (((dRange * 2.0) / (double)0xFFFFFF) * (double)ulDigitalValue) - dRange;

                L_CalibrationValue.Text = "Inverted position calibration started" + dChannelValue.ToString("F6") + "V";
            }

            // Test if user inverted user position calibration
            if ((ulStatus == 10) && (ulLastStatus == 10))
            {
                // Calculate the position into mm 
                dChannelValue = (((dRange * 2.0) / (double)0xFFFFFF) * (double)ulDigitalValue) - dRange;

                L_CalibrationValue.Text = "Transducer position = " + dChannelValue.ToString("F6") + " mm";
            }

            // Test if end calibration 
            if ((ulStatus == 8) && (ulLastStatus != 8))
            {
                B_StartCalibration.Enabled = true;
                B_BreakCalibration.Enabled = false;
                B_NextStep.Enabled = false;
                LB_Channel.Enabled = true;
                CB_Invert.Enabled = true;
                GB_Transducers.Enabled = true;
                T_Calibration.Enabled = false;
                T_Color.Enabled = false;
                B_NextStep.ForeColor = Color.Black;
                B_Connect.Enabled = true;
                resetCalibrationDefaultValuesCheckbox.Enabled = true;
                userPositionCheckBox.Enabled = true;
                if (userPositionCheckBox.Checked)
                    userPositionTextBox.Enabled = true;
                else
                    userPositionTextBox.Enabled = false;

                // Error management 
                switch (ulDigitalValue)
                {
                    case 0:
                        TB_CalibrationStatus.Text = TB_CalibrationStatus.Text + Environment.NewLine + "Calibration complete" + Environment.NewLine;

                        try
                        {
                            // Save transducer board data-base 
                            Response = MSXE371x.DataBaseSaveTransducers(out b_SaveResponse);

                            // Test if error occur
                            if (Response.iReturnValue != 0)
                            {
                                MessageBox.Show("DataBaseSaveTransducers Error : " + Response.iReturnValue.ToString(), "Function error", MessageBoxButtons.OK, MessageBoxIcon.Exclamation);
                                return;
                            }
                        }

                        catch (Exception ex)
                        {
                            MessageBox.Show("DataBaseSaveTransducers Exception occur : " + ex.Message, "Function error", MessageBoxButtons.OK, MessageBoxIcon.Exclamation);
                        }
                        break;

                    case 1:
                        // Stop the calibration 
                        TB_CalibrationStatus.Text = TB_CalibrationStatus.Text + Environment.NewLine + "!!!User brakes the calibration";
                        break;

                    case 2:
                    case 3:
                    case 4:
                    case 5:
                    case 6:
                    case 7:
                        TB_CalibrationStatus.Text = TB_CalibrationStatus.Text + Environment.NewLine + "Can not calibrate the primary feedback";
                        break;

                    case 8:
                    case 9:
                        TB_CalibrationStatus.Text = TB_CalibrationStatus.Text + Environment.NewLine + "The 0mm position can not be found." + Environment.NewLine + "Restart the calibration and select a new position for the 0mm.";
                        break;

                    case 10:
                    case 11:
                    case 12:
                    case 13:
                        TB_CalibrationStatus.Text = TB_CalibrationStatus.Text + Environment.NewLine + "The user position can not be found." + Environment.NewLine + "Restart the calibration and select a new position";
                        break;

                    case 14:
                        TB_CalibrationStatus.Text = TB_CalibrationStatus.Text + Environment.NewLine + "Primary short circuit occurs";
                        break;

                    default:
                        // Format and show the error 
                        TB_CalibrationStatus.Text = TB_CalibrationStatus.Text + Environment.NewLine + "Calibration error -" + ulDigitalValue.ToString();
                        break;
                }
            }

            if (ulLastStatus != ulStatus)
            {
                TB_CalibrationStatus.SelectionStart = TB_CalibrationStatus.Text.Length;
                TB_CalibrationStatus.ScrollToCaret();
            }

            ulLastStatus = ulStatus;

        }

        private void T_Color_Tick(object sender, EventArgs e)
        {
            if (B_NextStep.ForeColor == B_NextStep.BackColor)
                B_NextStep.ForeColor = Color.Black;
            else
                B_NextStep.ForeColor = B_NextStep.BackColor;
        }

        private void TransducerCalibration_FormClosing(object sender, FormClosingEventArgs e)
        {
            B_BreakCalibration_Click(sender, null);
        }

        private void userPositionCheckBox_CheckedChanged(object sender, EventArgs e)
        {
            if (userPositionCheckBox.Checked)
                userPositionTextBox.Enabled = true;
            else
                userPositionTextBox.Enabled = false;
        }


    }
}