using System;
using System.Threading;
using System.Drawing;
using System.Collections;
using System.ComponentModel;
using System.Windows.Forms;
using System.Data;
using System.Net;
using System.Net.Sockets;
using AddiData.ADDIControls;
using AddiData.ADDISocket;

namespace TransducerSequence
{
    public partial class TransducerSequence : Form
    {
        // Global Variable
        private MXCommon_WebReference.MXCommon        MX_Common;
        private MSXE371x_WebReference.MSXE371x        MSXE371x;
        private MXCommon_WebReference.DefaultResponse MXResponse;
        private MSXE371x_WebReference.DefaultResponse Response;
        private ArrayList       CounterOptions0 = new ArrayList();
        private ArrayList       CounterOptions1 = new ArrayList();
        private int             RetValue = 0;
        private int             syserno = 0;
        private TranducerInfo[] TranducerInformations;
        private ComboBox[]       SequenceChannels = new ComboBox[8];
        private CheckBox[]      DataFrame = new CheckBox[8];
        private bool            boExternalTempInit      = false;
        private bool            boCounterInit           = false;
        private bool            boCounterCompareInit    = false;
        private bool            boSaveGB_Thermocouple   = false;
        private bool            boSaveGB_Counter        = false;
        private bool            boSaveGB_CounterCompare = false;
        private bool            tcVersion               = false;
        private int             iDataStreamSize         = 0;
        private ulong           ulTFormat               = 0; 
        ArrayList               alAcquisitionChannels;
        Socket                  SocketClient = null;
        String                  DisplayText = "";
        AsyncClientStream       Client = null;


        public static String ConvertFromTime_T(byte[] bBuffer, int iIndex)
        {
            ulong ulTimeStampHigh = BitConverter.ToUInt32(bBuffer, iIndex);
            ulong ulTimeStampLow = BitConverter.ToUInt32(bBuffer, iIndex + 4);

            return (ConvertFromTime_T(ulTimeStampHigh, ulTimeStampLow));
        }

        public static String ConvertFromTime_T(ulong ulTimeStampHigh, ulong ulTimeStampLow)
        {
            ulong ulTimeStampLowms  = (ulong)(ulTimeStampLow / 1000);
            ulong ulTimeStampLowmus = (ulong)(ulTimeStampLow % 1000);

            DateTime dt70 = new DateTime(1970, 1, 1, 0, 0, 0, 0);
            long tickTotal = dt70.Ticks + (long) (ulTimeStampHigh * 10000000L);
            DateTime netTimeT = new DateTime(tickTotal);
            return (netTimeT.ToString() + "." + ulTimeStampLowms.ToString("D3") + "." + ulTimeStampLowmus.ToString("D3"));
        }

        private UInt64 ulGetNbrOfTransducerChannel(String sModuleType)
        {
            if (sModuleType.IndexOf("-", 0) == -1)
                return 0;
            sModuleType = sModuleType.Substring(sModuleType.IndexOf("-", 0) + 1);

            if (sModuleType.IndexOf("-", 0) == -1)
                return 0;
            sModuleType = sModuleType.Substring(sModuleType.IndexOf("-", 0) + 1);

            if (sModuleType.IndexOf("-", 0) == -1)
                return 0;
            sModuleType = sModuleType.Remove(sModuleType.IndexOf("-", 0));

            return (Convert.ToUInt64(sModuleType));
        }

        public class TranducerInfo
        {
            private ulong  ulSelectionIndex;
            private String sName;
            private ulong  ulCalibrationStatus;
            private ulong  ulCalibratedChannels;
            private ulong  ulType;
            private ulong  ulFrequency;
            private ulong  ulImpedance;
            private double dVeff;
            private double dSensibility;
            private double dRange;

            public TranducerInfo(ulong  SelectionIndex,
                                 String Name,
                                 ulong  CalibrationStatus,
                                 ulong  CalibratedChannels,
                                 ulong  Type,
                                 ulong  Frequency,
                                 ulong  Impedance,
                                 double Veff,
                                 double Sensibility,
                                 double Range)
            {
                ulSelectionIndex = SelectionIndex;
                sName = Name.Remove(Name.IndexOf((char)0));
                ulCalibrationStatus = CalibrationStatus;
                ulCalibratedChannels = CalibratedChannels;
                ulType = Type;
                ulFrequency = Frequency;
                ulImpedance = Impedance;
                dVeff = Veff;
                dSensibility = Sensibility;
                dRange = Range;
            }

            public ulong SelectionIndex { get { return (ulSelectionIndex); } set { ulSelectionIndex = value; } }
            public ulong CalibrationStatus { get { return (ulCalibrationStatus); } set { ulCalibrationStatus = value; } }
            public ulong CalibratedChannels { get { return (ulCalibratedChannels); } set { ulCalibratedChannels = value; } }
            public ulong Type { get { return (ulType); } set { ulType = value; } }
            public ulong Frequency { get { return (ulFrequency); } set { ulFrequency = value; } }
            public ulong Impedance { get { return (ulImpedance); } set { ulImpedance = value; } }
            public double Veff { get { return (dVeff); } set { dVeff = value; } }
            public double Sensibility { get { return (dSensibility); } set { dSensibility = value; } }
            public double Range { get { return (dRange); } set { dRange = value; } }
            public String Name { get { return (sName); } set { sName = value; } }

            public override String ToString()
            {
                String sRet;

                sRet = "Transducer Selection Index : " + ulSelectionIndex.ToString();
                sRet = sRet + Environment.NewLine + "Name                                   : " + sName;
                sRet = sRet + Environment.NewLine + "Calibration status                  : " + ulCalibrationStatus.ToString();
                sRet = sRet + Environment.NewLine + "Type Value                          : " + ulType.ToString();
                sRet = sRet + Environment.NewLine + "Frequency                           : " + ulFrequency.ToString() + " Hz";
                sRet = sRet + Environment.NewLine + "Impedance                          : " + ulImpedance.ToString() + " Ohm";
                sRet = sRet + Environment.NewLine + "Nominal Voltage                  : " + dVeff.ToString() + " V";
                sRet = sRet + Environment.NewLine + "Sensibility                            : " + dSensibility.ToString() + " mV/V/mm";
                sRet = sRet + Environment.NewLine + "Range                                 : +/-" + dRange.ToString() + " mm";

                return (sRet);
            }
        }

        public class ListData
        {
            private string myItem;
            private long myValue;

            public ListData(string strItem, long lValue)
            {

                this.myItem = strItem;
                this.myValue = lValue;
            }

            public string Item
            {
                get { return myItem; }
            }

            public long Value
            {

                get { return myValue; }
            }
        }

        public TransducerSequence()
        {
            InitializeComponent();
        }

        private void TransducerSequence_Load(object sender, EventArgs e)
        {
            MX_Common = new MXCommon_WebReference.MXCommon();
            MSXE371x = new MSXE371x_WebReference.MSXE371x();
            MX_Common.Timeout = 5000;
            MSXE371x.Timeout = 5000;

            // Gain list box initialisation
            ArrayList Gains = new ArrayList();
            Gains.Add(new ListData("Auto mode", 0));

            LB_T_Gain.DisplayMember = "Item";
            LB_T_Gain.ValueMember = "Value";
            LB_T_Gain.DataSource = Gains;

            // Frequency list box initialisation
            ArrayList Frequencys = new ArrayList();
            Frequencys.Add(new ListData("10 Hz", 10));
            Frequencys.Add(new ListData("50 Hz", 50));
            Frequencys.Add(new ListData("240 Hz", 240));

            LB_T_Frequency.DisplayMember = "Item";
            LB_T_Frequency.ValueMember = "Value";
            LB_T_Frequency.DataSource = Frequencys;

            // Display format list box initialisation
            ArrayList Formats = new ArrayList();
            Formats.Add(new ListData("mOhm", 0));
            Formats.Add(new ListData("C", 1));
            Formats.Add(new ListData("F", 2));

            LB_T_Format.DisplayMember = "Item";
            LB_T_Format.ValueMember = "Value";
            LB_T_Format.DataSource = Formats;

            // Counter mode list box initialisation
            ArrayList CounterModes = new ArrayList();
            CounterModes.Add(new ListData("Simple mode", 1));
            CounterModes.Add(new ListData("Double mode", 2));
            CounterModes.Add(new ListData("Quadruple mode", 4));
            CounterModes.Add(new ListData("Direct", 0));

            LB_IC_Mode.DisplayMember = "Item";
            LB_IC_Mode.ValueMember = "Value";
            LB_IC_Mode.DataSource = CounterModes;

            // Counter option list box initialisation
            CounterOptions0.Add(new ListData("Hysteresus on", 1));
            CounterOptions0.Add(new ListData("Hysteresus off", 0));
            CounterOptions1.Add(new ListData("Increment", 0));
            CounterOptions1.Add(new ListData("Decrement", 1));

            LB_IC_Option.DisplayMember = "Item";
            LB_IC_Option.ValueMember = "Value";
            LB_IC_Option.DataSource = CounterOptions0;

            // Counter compare mode list box initialisation
            ArrayList CompareModes = new ArrayList();
            CompareModes.Add(new ListData("Disabled", -1));
            CompareModes.Add(new ListData("Equals compare value", 0));
            CompareModes.Add(new ListData("Equals a multiple of the compare value", 1));

            LB_IC_CompareMode.DisplayMember = "Item";
            LB_IC_CompareMode.ValueMember = "Value";
            LB_IC_CompareMode.DataSource = CompareModes;

            // Counter compare options list box initialisation
            ArrayList CompareOptions = new ArrayList();
            CompareOptions.Add(new ListData("No synchro trigger used", 0));
            CompareOptions.Add(new ListData("Generates a synchro trigger", 1));

            LB_IC_SynchroTrigger.DisplayMember = "Item";
            LB_IC_SynchroTrigger.ValueMember = "Value";
            LB_IC_SynchroTrigger.DataSource = CompareOptions;

            // Trigger source list box initialisation
            ArrayList TriggerSources = new ArrayList();
            TriggerSources.Add(new ListData("Disabled", 0));
            TriggerSources.Add(new ListData("Hardware trigger", 1));
            TriggerSources.Add(new ListData("Synchro trigger", 2));
            TriggerSources.Add(new ListData("Counter compare trigger", 4));

            LB_TriggerSource.DisplayMember = "Item";
            LB_TriggerSource.ValueMember = "Value";
            LB_TriggerSource.DataSource = TriggerSources;


            // Trigger mode list box initialisation
            ArrayList TriggerModes = new ArrayList();
            TriggerModes.Add(new ListData("One shot trigger", 1));
            TriggerModes.Add(new ListData("Sequence trigger", 2));

            LB_TriggerMode.DisplayMember = "Item";
            LB_TriggerMode.ValueMember = "Value";
            LB_TriggerMode.DataSource = TriggerModes;


            // HW Trigger edge list box initialisation
            ArrayList HWTriggerEdges = new ArrayList();
            HWTriggerEdges.Add(new ListData("Rising edge", 1));
            HWTriggerEdges.Add(new ListData("Falling edge", 2));
            HWTriggerEdges.Add(new ListData("Both front", 3));

            LB_HWEdge.DisplayMember = "Item";
            LB_HWEdge.ValueMember = "Value";
            LB_HWEdge.DataSource = HWTriggerEdges;

            // Save the sequence channels check box
            SequenceChannels[0] = LB_Sequnce0;
            SequenceChannels[1] = LB_Sequnce1;
            SequenceChannels[2] = LB_Sequnce2;
            SequenceChannels[3] = LB_Sequnce3;
            SequenceChannels[4] = LB_Sequnce4;
            SequenceChannels[5] = LB_Sequnce5;
            SequenceChannels[6] = LB_Sequnce6;
            SequenceChannels[7] = LB_Sequnce7;

            for (long i = 0; i < 8; i++)
            {
                // Sequence channels list box initialisation
                ArrayList Sequences = new ArrayList();
                Sequences.Add(new ListData(i.ToString (), i));

                SequenceChannels[i].DisplayMember = "Item";
                SequenceChannels[i].ValueMember = "Value";
                SequenceChannels[i].DataSource = Sequences;
            }

            // Save the data frame check box
            DataFrame[0] = CB_DataFrameTimeStamp;
            DataFrame[1] = CB_DataFrameSeqCounter;
            DataFrame[2] = CB_DataFrameMetric;
            DataFrame[3] = CB_DataFrameInvert;
            DataFrame[4] = CB_DataFrameTemperature;
            DataFrame[5] = CB_DataFrameIncCounter;
            DataFrame[6] = CB_DataFrameHWTrigger;
            DataFrame[7] = CB_DataFrameIndex;


            // Delay mode list box initialisation
            ArrayList DelayModes = new ArrayList();
            DelayModes.Add(new ListData("Disabled", 0));
            DelayModes.Add(new ListData("Enabled", 1));

            LB_DelayMode.DisplayMember = "Item";
            LB_DelayMode.ValueMember = "Value";
            LB_DelayMode.DataSource = DelayModes;

            // Delay time list box initialisation
            ArrayList DelayTimes = new ArrayList();
            DelayTimes.Add(new ListData("ms", 0));
            DelayTimes.Add(new ListData("s", 1));

            LB_DelayTime.DisplayMember = "Item";
            LB_DelayTime.ValueMember = "Value";
            LB_DelayTime.DataSource = DelayTimes;
        }

        private void LB_IC_Mode_SelectedIndexChanged(object sender, EventArgs e)
        {
            if (Convert.ToUInt32(LB_IC_Mode.SelectedValue) == 0)
            {
                LB_IC_Option.DataSource = CounterOptions1;
            }
            else
            {
                LB_IC_Option.DataSource = CounterOptions0;
            }
        }

        private bool bo_ThermocoupleInitialisation ()
        {
            bool bo_Ret = true;

            if (GB_Thermocouple.Enabled == true)
            {
                try
                {
                    // External temperature initialisation

                    RetValue = MSXE371x.ExternalTemperatureInit(Convert.ToUInt32 (LB_T_Thermocouple.SelectedValue),
                                                                Convert.ToUInt32 (LB_T_Format.SelectedValue),
                                                                Convert.ToUInt32 (LB_T_Gain.SelectedValue),
                                                                Convert.ToUInt32 (LB_T_Frequency.SelectedValue),
                                                                0, 0, 0, 0, 0, out syserno);
                    if (RetValue != 0)
                    {
                        MessageBox.Show("ExternalTemperatureInit Error : " + RetValue.ToString(), "Function error", MessageBoxButtons.OK, MessageBoxIcon.Exclamation);
                        bo_Ret = false;
                    }
                    else
                    {
                        boExternalTempInit = true;
                    }
                }

                catch (Exception ex)
                {
                    MessageBox.Show("ExternalTemperatureInit Exception occur : " + ex.Message, "Function error", MessageBoxButtons.OK, MessageBoxIcon.Exclamation);
                    bo_Ret = false;
                }
            }

            return (bo_Ret);
        }

        private bool bo_ThermocoupleRelease ()
        {
            bool bo_Ret = true;

            if (boExternalTempInit == true)
            {
                try
                {
                    // Release external temperature

                    RetValue = MSXE371x.ExternalTemperatureRelease(out syserno);

                    if (RetValue != 0)
                    {
                        MessageBox.Show("ExternalTemperatureRelease Error : " + RetValue.ToString(), "Function error", MessageBoxButtons.OK, MessageBoxIcon.Exclamation);
                        bo_Ret = false;
                    }
                }

                catch (Exception ex)
                {
                    MessageBox.Show("ExternalTemperatureRelease Exception occur : " + ex.Message, "Function error", MessageBoxButtons.OK, MessageBoxIcon.Exclamation);
                    bo_Ret = false;
                }

            boExternalTempInit = false;
            }
            return (bo_Ret);
        }

        private bool bo_CounterInitialisation()
        {
            bool bo_Ret = true;

            if (GB_Counter.Enabled == true)
            {
                try
                {
                    // Counter initialisation

                    RetValue = MSXE371x.IncCounterInit(Convert.ToUInt32(LB_IC_Mode.SelectedValue),
                                                       Convert.ToUInt32(LB_IC_Option.SelectedValue),
                                                       0,
                                                       0,
                                                       0,
                                                       0,
                                                       out syserno);
                    if (RetValue != 0)
                    {
                        MessageBox.Show("IncCounterInit Error : " + RetValue.ToString(), "Function error", MessageBoxButtons.OK, MessageBoxIcon.Exclamation);
                        bo_Ret = false;
                    }
                    else
                    {
                        boCounterInit = true;
                    }
                }

                catch (Exception ex)
                {
                    MessageBox.Show("IncCounterInit Exception occur : " + ex.Message, "Function error", MessageBoxButtons.OK, MessageBoxIcon.Exclamation);
                    bo_Ret = false;
                }

                if ((bo_Ret == true) && (GB_CounterCompare.Enabled == true) && (Convert.ToInt32(LB_IC_CompareMode.SelectedValue) != -1))
                {
                    try
                    {
                        // Counter compare initialisation

                        RetValue = MSXE371x.IncCounterInitAndEnableCompareLogic(Convert.ToUInt64(TB_IC_CompareValue.Text),
                                                                                Convert.ToUInt32(LB_IC_CompareMode.SelectedValue),
                                                                                Convert.ToUInt32(LB_IC_SynchroTrigger.SelectedValue),
                                                                                0,
                                                                                0,
                                                                                out syserno);
                        if (RetValue != 0)
                        {
                            MessageBox.Show("IncCounterInitAndEnableCompareLogic Error : " + RetValue.ToString(), "Function error", MessageBoxButtons.OK, MessageBoxIcon.Exclamation);
                            bo_Ret = false;
                        }
                        else
                        {
                            boCounterCompareInit = true;
                        }
                    }

                    catch (Exception ex)
                    {
                        MessageBox.Show("IncCounterInitAndEnableCompareLogic Exception occur : " + ex.Message, "Function error", MessageBoxButtons.OK, MessageBoxIcon.Exclamation);
                        bo_Ret = false;
                    }
                }
            }
            return (bo_Ret);
        }

        private bool bo_CounterRelease()
        {
            bool bo_Ret = true;

            if (boCounterInit == true)
            {
                if (boCounterCompareInit == true)
                {
                    try
                    {
                        // Release the counter compare logic

                        RetValue = MSXE371x.IncCounterDisableAndReleaseCompareLogic(out syserno);
                        if (RetValue != 0)
                        {
                            MessageBox.Show("IncCounterDisableAndReleaseCompareLogic Error : " + RetValue.ToString(), "Function error", MessageBoxButtons.OK, MessageBoxIcon.Exclamation);
                            bo_Ret = false;
                        }
                    }

                    catch (Exception ex)
                    {
                        MessageBox.Show("IncCounterDisableAndReleaseCompareLogic Exception occur : " + ex.Message, "Function error", MessageBoxButtons.OK, MessageBoxIcon.Exclamation);
                        bo_Ret = false;
                    }

                    boCounterCompareInit = false;
                }

                try
                {
                    // Counter compare release

                    RetValue = MSXE371x.IncCounterRelease(out syserno);
                    if (RetValue != 0)
                    {
                        MessageBox.Show("IncCounterRelease Error : " + RetValue.ToString(), "Function error", MessageBoxButtons.OK, MessageBoxIcon.Exclamation);
                        bo_Ret = false;
                    }
                }

                catch (Exception ex)
                {
                    MessageBox.Show("IncCounterRelease Exception occur : " + ex.Message, "Function error", MessageBoxButtons.OK, MessageBoxIcon.Exclamation);
                    bo_Ret = false;
                }

                boCounterInit = false;
            }
            return (bo_Ret);
        }

        private void LB_TransducerList_SelectedIndexChanged(object sender, EventArgs e)
        {
            L_TransducerInfo.Text = TranducerInformations[LB_TransducerList.SelectedIndex].ToString();

        }

        private void LB_Sequnce_SelectedIndexChanged(object sender, EventArgs e)
        {
            for (int i = 0; i < (SequenceChannels.Length - 1); i++)
            {
                if (SequenceChannels[i].SelectedIndex == 0)
                {
                    for (int j = i + 1; j < SequenceChannels.Length; j++)
                    {
                        if (SequenceChannels[j].SelectedIndex != -1)
                        {
                            SequenceChannels[j].SelectedIndex = 0;
                            SequenceChannels[j].Enabled = false;
                        }
                    }
                    break;
                }
                else if (SequenceChannels[i + 1].SelectedIndex != -1)
                {
                    SequenceChannels[i + 1].Enabled = true;
                }
                    

            }
        }

        private void CB_DataFrame3_CheckedChanged(object sender, EventArgs e)
        {
            GB_Thermocouple.Enabled = CB_DataFrameTemperature.Checked;
        }

        private void TriggerManagement()
        {
            LB_TriggerMode.Enabled     = Convert.ToBoolean(LB_TriggerSource.SelectedValue);
            TB_SequenceTrigger.Enabled = Convert.ToBoolean(LB_TriggerSource.SelectedValue) && Convert.ToBoolean(Convert.ToUInt32(LB_TriggerMode.SelectedValue) == 2);
            LB_HWEdge.Enabled          = Convert.ToBoolean(Convert.ToUInt32(LB_TriggerSource.SelectedValue) == 1);
            TB_HTriggerCount.Enabled   = Convert.ToBoolean(Convert.ToUInt32(LB_TriggerSource.SelectedValue) == 1);

            CounterManagement();
        }

        private void LB_TriggerSource_SelectedIndexChanged(object sender, EventArgs e)
        {
            TriggerManagement();
        }

        private void LB_TriggerMode_SelectedIndexChanged(object sender, EventArgs e)
        {
            TriggerManagement();
        }

        private void CounterManagement()
        {
            GB_Counter.Enabled = CB_DataFrameIncCounter.Checked || Convert.ToBoolean(Convert.ToUInt32(LB_TriggerSource.SelectedValue) == 4) || Convert.ToBoolean(Convert.ToUInt32(LB_TriggerSource.SelectedValue) == 2);
            GB_CounterCompare.Enabled = GB_Counter.Enabled;
        }

        private void CB_DataFrame4_CheckedChanged(object sender, EventArgs e)
        {
            CounterManagement();
        }

        private void LB_IC_CompareMode_SelectedIndexChanged(object sender, EventArgs e)
        {
            TB_IC_CompareValue.Enabled = Convert.ToBoolean(Convert.ToInt32(LB_IC_CompareMode.SelectedValue) != -1);
            LB_IC_SynchroTrigger.Enabled = TB_IC_CompareValue.Enabled;
        }

        private void LB_DelayMode_SelectedIndexChanged(object sender, EventArgs e)
        {
            LB_DelayTime.Enabled = Convert.ToBoolean(Convert.ToInt32(LB_DelayMode.SelectedValue) != 0);
            TB_DelayValue.Enabled = LB_DelayTime.Enabled;
        }

        private void B_Connect_Click(object sender, EventArgs e)
        {
            System.Text.ASCIIEncoding enc = new System.Text.ASCIIEncoding();
            byte[] pcName = new byte[255];
            ulong ulNbrOfTransducers;
            ulong ulNbrOfChannels;
            ulong ulSelectionIndex;
            ulong  ulCalibrationStatus;
            ulong  ulCalibratedChannels;
            ulong  ulType;
            ulong  ulFrequency;
            ulong  ulImpedance;
            double dVeff;
            double dSensibility;
            double dRange;

            if ((TB_IPAddress_0.Text.ToString() == "") || (TB_IPAddress_1.Text.ToString() == "") || (TB_IPAddress_2.Text.ToString() == "") || (TB_IPAddress_3.Text.ToString() == ""))
            {
                MessageBox.Show("The IP address you enter is not valid, please try again.", "IP address error", MessageBoxButtons.OK, MessageBoxIcon.Exclamation);
            }
            else
            {
                if ((Convert.ToUInt64(TB_IPAddress_0.Text.ToString(), 10) >= 0) && (Convert.ToUInt64(TB_IPAddress_0.Text.ToString(), 10) < 256))
                {
                    if ((Convert.ToUInt64(TB_IPAddress_1.Text.ToString(), 10) >= 0) && (Convert.ToUInt64(TB_IPAddress_1.Text.ToString(), 10) < 256))
                    {
                        if ((Convert.ToUInt64(TB_IPAddress_2.Text.ToString(), 10) >= 0) && (Convert.ToUInt64(TB_IPAddress_2.Text.ToString(), 10) < 256))
                        {
                            if ((Convert.ToUInt64(TB_IPAddress_3.Text.ToString(), 10) >= 0) && (Convert.ToUInt64(TB_IPAddress_3.Text.ToString(), 10) < 256))
                            {
                                B_StartSample.Enabled = false;
                                MX_Common.Url = "http://" + TB_IPAddress_0.Text + "." + TB_IPAddress_1.Text + "." + TB_IPAddress_2.Text + "." + TB_IPAddress_3.Text + ":5555";
                                MSXE371x.Url = MX_Common.Url;

                                try
                                {
                                    // Get the number of transducers type

                                    Response = MSXE371x.TransducerGetNbrOfType (out ulNbrOfTransducers);
                                    if (Response.iReturnValue != 0)
                                    {
                                        MessageBox.Show("TransducerGetNbrOfType Error : " + Response.iReturnValue.ToString(), "Function error", MessageBoxButtons.OK, MessageBoxIcon.Exclamation);
                                        return;
                                    }
                                }

                                catch (Exception ex)
                                {
                                    MessageBox.Show("TransducerGetNbrOfType Exception occur : " + ex.Message, "Function error", MessageBoxButtons.OK, MessageBoxIcon.Exclamation);
                                    return;
                                }

                                TranducerInformations = new TranducerInfo[ulNbrOfTransducers];
                                ArrayList Transducers = new ArrayList();
                                L_NbrOfTransducers.Text = ulNbrOfTransducers.ToString();

                                for (ulong ulCpt = 0; ulCpt < ulNbrOfTransducers; ulCpt++)
                                {
                                    // Get the transducer informations
                                    MSXE371x.TransducerGetTypeInformation(ulCpt,
                                                                          out ulSelectionIndex,
                                                                          out pcName,
                                                                          out ulCalibrationStatus,
                                                                          out ulCalibratedChannels,
                                                                          out ulType,
                                                                          out ulFrequency,
                                                                          out ulImpedance,
                                                                          out dVeff,
                                                                          out dSensibility,
                                                                          out dRange);

                                    // Save the transducer informations
                                    TranducerInformations[ulCpt] = new TranducerInfo(ulSelectionIndex,
                                                                                     enc.GetString(pcName),
                                                                                     ulCalibrationStatus,
                                                                                     ulCalibratedChannels,
                                                                                     ulType,
                                                                                     ulFrequency,
                                                                                     ulImpedance,
                                                                                     dVeff,
                                                                                     dSensibility,
                                                                                     dRange);

                                    Transducers.Add(new ListData(enc.GetString(pcName), (long)ulSelectionIndex));
                                }

                                LB_TransducerList.DataSource = Transducers;
                                LB_TransducerList.DisplayMember = "Item";
                                LB_TransducerList.ValueMember = "Value";

                                // Get the module type
                                MXResponse = MX_Common.GetModuleType(out pcName);
                                ulNbrOfChannels = ulGetNbrOfTransducerChannel(enc.GetString(pcName));

                                if (enc.GetString(pcName).Contains("{2000}"))
                                    tcVersion = true;

                                // Thermocouple list box initialisation
                                ArrayList SensorsType = new ArrayList();
                                if (tcVersion)
                                {
                                    SensorsType.Add(new ListData("TC type B", 0)); //
                                    SensorsType.Add(new ListData("TC type E", 1)); //
                                    SensorsType.Add(new ListData("TC type J", 2)); //
                                    SensorsType.Add(new ListData("TC type K", 3)); //
                                    SensorsType.Add(new ListData("TC type N", 4)); //
                                    SensorsType.Add(new ListData("TC type R", 5)); //
                                    SensorsType.Add(new ListData("TC type S", 6)); //
                                    SensorsType.Add(new ListData("TC type T", 7)); //
                                }
                                else
                                {
                                    SensorsType.Add(new ListData("PT100", 100));
                                    SensorsType.Add(new ListData("PT200", 200));
                                    SensorsType.Add(new ListData("PT500", 500));
                                    SensorsType.Add(new ListData("PT1000", 1000));
                                }

                                LB_T_Thermocouple.DataSource = SensorsType;
                                LB_T_Thermocouple.SelectedIndex = 0;
                                LB_T_Thermocouple.DisplayMember = "Item";
                                LB_T_Thermocouple.ValueMember = "Value";


                                for (int i = 0; i < 8; i++)
                                {
                                    ArrayList Sequences = new ArrayList();
                                    for (long j = -1; j < (long) ulNbrOfChannels; j++)
                                    {
                                        if (j != -1)
                                            Sequences.Add(new ListData(j.ToString(), j));
                                        else
                                            Sequences.Add(new ListData("", j));
                                    }
                                    SequenceChannels[i].DataSource = Sequences;
                                    SequenceChannels[i].Visible = Convert.ToBoolean(i < (long) ulNbrOfChannels);
                                    SequenceChannels[i].SelectedIndex = i + 1;
                                    SequenceChannels[i].Enabled = true;
                                }

                                GB_Transducers.Enabled          = true;
                                B_StartSample.Enabled           = true;
                                GB_Sequence.Enabled             = true;
                                GB_DataFormat.Enabled           = true;
                                GB_TriggerConfiguration.Enabled = true;
                                GB_DelayConfiguration.Enabled   = true;
                                B_StartSample.Enabled           = true;

                            }
                            else
                            {
                                MessageBox.Show("The IP address (part 4) you enter is not valid, please try again.", "IP address error", MessageBoxButtons.OK, MessageBoxIcon.Exclamation);
                            }
                        }
                        else
                        {
                            MessageBox.Show("The IP address (part 3) you enter is not valid, please try again.", "IP address error", MessageBoxButtons.OK, MessageBoxIcon.Exclamation);
                        }
                    }
                    else
                    {
                        MessageBox.Show("The IP address (part 2) you enter is not valid, please try again.", "IP address error", MessageBoxButtons.OK, MessageBoxIcon.Exclamation);
                    }
                }
                else
                {
                    MessageBox.Show("The IP address (part 1) you enter is not valid, please try again.", "IP address error", MessageBoxButtons.OK, MessageBoxIcon.Exclamation);
                }
            }
        }

        private bool boCheckConfiguration(out MSXE371x_WebReference.unsignedLong8FixedArray ulSequenceArray, out ulong ulSequenceSize, out ulong ulDataFormat, out int iDataStreamSize)
        {

            ulSequenceSize        = 0;
            ulDataFormat          = 0;
            iDataStreamSize       = 0;

            alAcquisitionChannels   = new ArrayList();
            ulSequenceArray         = new MSXE371x_WebReference.unsignedLong8FixedArray();
            ulSequenceArray.ulValue = new UInt64[8];

            // Check the channel sequence
            for (int i = 0; i < 8; i++)
            {
                if (Convert.ToInt32 (SequenceChannels[i].SelectedValue) != -1)
                {
                    alAcquisitionChannels.Add("Channel " + SequenceChannels[i].SelectedValue.ToString() + ":");
                    ulSequenceArray.ulValue[ulSequenceSize] = Convert.ToUInt64(SequenceChannels[i].SelectedValue);
                    ulSequenceSize = ulSequenceSize + 1;
                }
            }

            if (ulSequenceSize == 0)
            {
                MessageBox.Show("No channel selected", "Configuration error", MessageBoxButtons.OK, MessageBoxIcon.Exclamation);
                return (false);
            }

            // Check the number of sequence
            if ((Convert.ToInt64(TB_SequenceNbr.Text) < 0) || (Convert.ToInt64(TB_SequenceNbr.Text) > 0xFFFFFFFF))
            {
                MessageBox.Show("Sequence number value selection error (0 <= value <= 0xFFFFFFFF)", "Configuration error", MessageBoxButtons.OK, MessageBoxIcon.Exclamation);
                return (false);
            }

            // Check the send data size
            if ((Convert.ToInt64(TB_TransferNbr.Text) < 1) || ((Convert.ToInt64(TB_TransferNbr.Text) > 0xFFFFFFFF) && (Convert.ToInt64(TB_SequenceNbr.Text) == 0)) || ((Convert.ToInt64(TB_TransferNbr.Text) > Convert.ToInt64(TB_SequenceNbr.Text)) && (Convert.ToInt64(TB_SequenceNbr.Text) != 0)))
            {
                if (Convert.ToInt64(TB_SequenceNbr.Text) == 0)
                {
                    MessageBox.Show("Send sequence number value selection error (1 <= value <= 0xFFFFFFFF)", "Configuration error", MessageBoxButtons.OK, MessageBoxIcon.Exclamation);
                }
                else
                {
                    MessageBox.Show("Send sequence number value selection error (1 <= value <= " + TB_SequenceNbr.Text + ")", "Configuration error", MessageBoxButtons.OK, MessageBoxIcon.Exclamation);
                }
                return (false);
            }

            // Check the hardware trigger sequence number
            if ((TB_SequenceTrigger.Enabled == true) && ((Convert.ToInt64(TB_SequenceTrigger.Text) < 1) || (Convert.ToInt64(TB_SequenceTrigger.Text) > 0xFFFFFFFF)))
            {
                MessageBox.Show("Selection of sequence number to acquire for each trigger is wrong (0 < value <= 0xFFFFFFFF)", "Configuration error", MessageBoxButtons.OK, MessageBoxIcon.Exclamation);
                return (false);
            }


            // Check the delay time value
            if ((Convert.ToUInt32(LB_DelayMode.SelectedValue) != 0) && ((Convert.ToInt32(TB_DelayValue.Text) < 1) || (Convert.ToInt32(TB_DelayValue.Text) > 65535)))
            {
                MessageBox.Show("Delay time value selection error (0 < value <= 65535)", "Configuration error", MessageBoxButtons.OK, MessageBoxIcon.Exclamation);
                return (false);
            }

            // Check the counter compare configuration
            if ((Convert.ToUInt32(LB_TriggerSource.SelectedValue) == 4) && (Convert.ToInt32(LB_IC_CompareMode.SelectedValue) == -1))
            {
                MessageBox.Show("Counter compare logic used for trigger but counter compare not configurated", "Configuration error", MessageBoxButtons.OK, MessageBoxIcon.Exclamation);
                return (false);
            }

            if ((TB_IC_CompareValue.Enabled == true) && ((Convert.ToInt64(TB_IC_CompareValue.Text) < 0) || (Convert.ToInt64(TB_IC_CompareValue.Text) > 0xFFFFFFFF)))
            {
                MessageBox.Show("Counter compare value selection error (0 <= value <= 0xFFFFFFFF)", "Configuration error", MessageBoxButtons.OK, MessageBoxIcon.Exclamation);
                return (false);
            }


            // Get the data frame format
            for (int i = 0; i < DataFrame.Length; i ++)
            {
                if (i >= 6)
                    ulDataFormat = ulDataFormat | (Convert.ToUInt32(DataFrame[i].Checked) << (i + 1));
                else
                    ulDataFormat = ulDataFormat | (Convert.ToUInt32(DataFrame[i].Checked) << i);

            }

            // Get the data stream size
            iDataStreamSize = (int)((ulSequenceSize * 4) +
                                     (Convert.ToUInt32(CB_DataFrameTimeStamp.Checked) * 8) +
                                     (Convert.ToUInt32(CB_DataFrameSeqCounter.Checked) * 4) +
                                     (Convert.ToUInt32(CB_DataFrameTemperature.Checked) * 4) +
                                     (Convert.ToUInt32(CB_DataFrameIncCounter.Checked) * 4));

            if (CB_DataFrameIndex.Checked || CB_DataFrameHWTrigger.Checked)
                iDataStreamSize = iDataStreamSize + 4;

            return (true);
        }

        private void B_StartSample_Click(object sender, EventArgs e)
        {
            MSXE371x_WebReference.unsignedLong8FixedArray ulSequenceArray;
            ulong ulDataFormat;
            ulong ulSequenceSize;
            Thread ClientThread;

            // Check the user configuration
            if (boCheckConfiguration(out ulSequenceArray, out ulSequenceSize, out ulDataFormat, out iDataStreamSize) == true)
            {
                try
                {
                    // Create the client stream
                    Client = new AddiData.ADDISocket.AsyncClientStream();

                    // Open the socket connection to receive the data
                    SocketClient = null;
                    SocketClient = new Socket(AddressFamily.InterNetwork, SocketType.Stream, ProtocolType.Tcp);
                    SocketClient.ReceiveBufferSize = 1048576;
                    SocketClient.Connect(TB_IPAddress_0.Text + "." + TB_IPAddress_1.Text + "." + TB_IPAddress_2.Text + "." + TB_IPAddress_3.Text, 8989);

                    // Start the read socket and the user thread
                    Client.SingleFramesSize = iDataStreamSize;
                    Client.NbrOfFrames = Convert.ToInt32(TB_TransferNbr.Text.ToString(), 10);
                    Client.Start(SocketClient);
                    ClientThread = new Thread(ClientThreadFct);
                    DisplayText = "";

                    ulTFormat = Convert.ToUInt32(LB_T_Format.SelectedValue);

                    // Extern temperature initialisation
                    if (bo_ThermocoupleInitialisation() == true)
                    {
                        // Counter configuration
                        if (bo_CounterInitialisation() == true)
                        {
                            try
                            {
                                //Init and start the sequence acquisition

                                RetValue = MSXE371x.TransducerInitAndStartSequence(Convert.ToUInt32(LB_TransducerList.SelectedValue),
                                                                                   ulSequenceSize,
                                                                                   ulSequenceArray,
                                                                                   Convert.ToUInt32(TB_SequenceNbr.Text),
                                                                                   Convert.ToUInt32(TB_TransferNbr.Text),
                                                                                   Convert.ToUInt32(LB_DelayMode.SelectedValue),
                                                                                   Convert.ToUInt32(LB_DelayTime.SelectedValue),
                                                                                   Convert.ToUInt32(TB_DelayValue.Text),
                                                                                   Convert.ToUInt32(LB_TriggerSource.SelectedValue),
                                                                                   Convert.ToUInt32(LB_TriggerMode.SelectedValue),
                                                                                   Convert.ToUInt32(LB_HWEdge.SelectedValue),
                                                                                   Convert.ToUInt32(TB_HTriggerCount.Text),
                                                                                   Convert.ToUInt32(TB_SequenceTrigger.Text),
                                                                                   ulDataFormat,
                                                                                   0, 0, 0, 0, out syserno);



                                if (RetValue != 0)
                                {
                                    MessageBox.Show("TransducerInitAndStartSequence Error : " + RetValue.ToString(), "Function error", MessageBoxButtons.OK, MessageBoxIcon.Exclamation);
                                }
                                else
                                {
                                    DisplayTimer.Enabled = true;
                                    ClientThread.Priority = ThreadPriority.Lowest;
                                    ClientThread.Start();
                                    boSaveGB_Thermocouple = GB_Thermocouple.Enabled;
                                    boSaveGB_Counter = GB_Counter.Enabled;
                                    boSaveGB_CounterCompare = GB_CounterCompare.Enabled;
                                    B_Connect.Enabled = false;
                                    GB_Transducers.Enabled = false;
                                    B_StartSample.Enabled = false;
                                    GB_Sequence.Enabled = false;
                                    GB_DataFormat.Enabled = false;
                                    GB_TriggerConfiguration.Enabled = false;
                                    GB_DelayConfiguration.Enabled = false;
                                    GB_Thermocouple.Enabled = false;
                                    GB_Counter.Enabled = false;
                                    GB_CounterCompare.Enabled = false;
                                    B_StopSample.Enabled = true;
                                }
                            }

                            catch (Exception ex)
                            {
                                MessageBox.Show("TransducerInitAndStartSequence Exception occur : " + ex.Message, "Function error", MessageBoxButtons.OK, MessageBoxIcon.Exclamation);
                            }
                        }
                    }
                }
                catch (Exception ex)
                {
                    MessageBox.Show("Socket connection exception occur : " + ex.Message, "Function error", MessageBoxButtons.OK, MessageBoxIcon.Exclamation);
                }

                // Test if error occur
                if (B_StartSample.Enabled == true)
                {
                    B_StopSample_Click(sender, e);
                }
            }
        }

        private void B_StopSample_Click(object sender, EventArgs e)
        {
            DisplayTimer.Enabled = false;

            // Test if socket open
            if (SocketClient != null)
            {
                if (Client != null)
                    Client.Stoop();

                try
                {
                    // Close the socket connection
                    SocketClient.Shutdown(SocketShutdown.Both);
                    SocketClient.Close();
                    SocketClient = null;
                    System.GC.Collect();
                }
                catch (Exception ex)
                {
                    MessageBox.Show("Socket close connection exception occur : " + ex.Message, "Function error", MessageBoxButtons.OK, MessageBoxIcon.Exclamation);
                }
            }

            // Test if sequence started
            if (B_StartSample.Enabled == false)
            {
                B_Connect.Enabled = true;
                GB_Transducers.Enabled = true;
                B_StartSample.Enabled = true;
                GB_Sequence.Enabled = true;
                GB_DataFormat.Enabled = true;
                GB_TriggerConfiguration.Enabled = true;
                GB_DelayConfiguration.Enabled = true;
                GB_Thermocouple.Enabled = boSaveGB_Thermocouple;
                GB_Counter.Enabled = boSaveGB_Counter;
                GB_CounterCompare.Enabled = boSaveGB_CounterCompare;

                try
                {
                    // Stop and release the sequence

                    RetValue = MSXE371x.TransducerStopAndReleaseSequence(out syserno);

                    if (RetValue != 0)
                    {
                        MessageBox.Show("TransducerStopAndReleaseSequence Error : " + RetValue.ToString(), "Function error", MessageBoxButtons.OK, MessageBoxIcon.Exclamation);
                    }
                }

                catch (Exception ex)
                {
                    MessageBox.Show("TransducerStopAndReleaseSequence Exception occur : " + ex.Message, "Function error", MessageBoxButtons.OK, MessageBoxIcon.Exclamation);
                }
            }

            bo_ThermocoupleRelease();
            bo_CounterRelease();

            B_StartSample.Enabled = true;
            B_StopSample.Enabled = false;
        }

        private void ClientThreadFct()
        {
            byte[] SocketData;
            int StreamCounter = 0;
            int Offset = 0;
            String sOutString;
            String sOutAllString;
            String sSequenceCounter = "";

            while ((SocketData = Client.ReadData()) != null)
            {
                sOutAllString = "";

                for (StreamCounter = 0; StreamCounter < Client.NbrOfFrames; StreamCounter++)
                {
                    sOutString = "";

                    Offset = StreamCounter * Client.SingleFramesSize;

                    // Test if time stampe
                    if (CB_DataFrameTimeStamp.Checked == true)
                    {
                        sOutString = "Time stamp: " + ConvertFromTime_T(SocketData, Offset);
                        Offset = Offset + 8;
                    }

                    // Test if sequence counter used
                    if (CB_DataFrameSeqCounter.Checked == true)
                    {
                        sSequenceCounter = "Sequence counter: " + BitConverter.ToUInt32(SocketData, Offset).ToString();
                        Offset = Offset + 4;
                    }

                    // Test if external temperature used
                    if (CB_DataFrameTemperature.Checked == true)
                    {
                        if (sOutString != "")
                        {
                            sOutString = sOutString + " / ";
                        }
                        // Test if mOhm
                        if (ulTFormat == 0)
                        {
                            sOutString = sOutString + "External temperature: " + BitConverter.ToUInt32(SocketData, Offset).ToString() + " mOhm";
                        }
                        // Test if C
                        else if (ulTFormat == 1)
                        {
                            sOutString = sOutString + "External temperature: " + BitConverter.ToSingle(SocketData, Offset).ToString("F3") + " C";
                        }
                        // F
                        else
                        {
                            sOutString = sOutString + "External temperature: " + BitConverter.ToSingle(SocketData, Offset).ToString("F3") + " F";
                        }

                        Offset = Offset + 4;
                    }

                    // Test if incremental counter used
                    if (CB_DataFrameIncCounter.Checked == true)
                    {
                        if (sOutString != "")
                        {
                            sOutString = sOutString + " / ";
                        }

                        sOutString = sOutString + "Incremental counter : " + BitConverter.ToUInt32(SocketData, Offset).ToString() + " Dec";
                        Offset = Offset + 4;
                    }

                    // Test if hardware trigger status or index status
                    if ((CB_DataFrameHWTrigger.Checked == true) || (CB_DataFrameIndex.Checked == true))
                    {
                        if (sOutString != "")
                        {
                            sOutString = sOutString + " / ";
                        }
                        sOutString = sOutString + "Digital status : " + BitConverter.ToUInt32(SocketData, Offset).ToString() + " Dec";
                        Offset = Offset + 4;
                    }

                    if (sOutString != "")
                    {
                        sOutString = sOutString + Environment.NewLine + sSequenceCounter;
                    }
                    else
                    {
                        sOutString = sSequenceCounter;
                    }

                    // Get the transducer channels value
                    int iChannelsCpt = 0;
                    foreach (String Channel in alAcquisitionChannels)
                    {
                        if ((iChannelsCpt % 4) == 0)
                        {
                            sOutString = sOutString + Environment.NewLine;
                        }

                        // Test if mm
                        if (CB_DataFrameMetric.Checked == true)
                        {
                            sOutString = sOutString + Channel + BitConverter.ToSingle(SocketData, Offset).ToString("F3") + " mm  ";
                        }
                        else
                        {
                            sOutString = sOutString + Channel + BitConverter.ToUInt32(SocketData, Offset).ToString("X") + " Hex  ";
                        }
                        Offset = Offset + 4;
                        iChannelsCpt++;
                    }

                    sOutAllString = sOutAllString + sOutString + Environment.NewLine;
                }
                DisplayText = sOutAllString;
            }
        }

        private void TransducerSequence_FormClosing(object sender, FormClosingEventArgs e)
        {
            B_StopSample_Click(sender, null);
        }

        private void DisplayTimer_Tick(object sender, EventArgs e)
        {
            TB_Values.Text = DisplayText;
        }

        private void LB_T_Thermocouple_SelectedIndexChanged(object sender, EventArgs e)
        {
            //Display the temperature
            if (tcVersion)
            {
                // Display wihtout mOm
                ArrayList Formats_withoutMOhm = new ArrayList();
                Formats_withoutMOhm.Add(new ListData("C", 1));
                Formats_withoutMOhm.Add(new ListData("F", 2));
                LB_T_Format.DataSource = Formats_withoutMOhm;
            }
            else
            {
                // Display with mOm
                ArrayList Formats_witMOhm = new ArrayList();
                Formats_witMOhm.Add(new ListData("mOhm", 0));
                Formats_witMOhm.Add(new ListData("C", 1));
                Formats_witMOhm.Add(new ListData("F", 2));
                LB_T_Format.DataSource = Formats_witMOhm;
            }


            //Frequency
            if (tcVersion)
            {
                // Display wihtout 10Hz mOm
                ArrayList Frequencys = new ArrayList();
                Frequencys.Add(new ListData("50 Hz", 50));
                Frequencys.Add(new ListData("240 Hz", 240));
                LB_T_Frequency.DataSource = Frequencys;
            }
            else
            {
                // Display with 10Hz
                ArrayList Frequencys = new ArrayList();
                Frequencys.Add(new ListData("10 Hz", 10));
                Frequencys.Add(new ListData("50 Hz", 50));
                Frequencys.Add(new ListData("240 Hz", 240));
                LB_T_Frequency.DataSource = Frequencys;
            }
        }
    }
}