Public Class ExtTemperatureRead

    Private MX_Common As MXCommon_WebReference.MXCommon
    Private MSXE371x As MSXE371x_WebReference.MSXE371x
    Private Response As MSXE371x_WebReference.DefaultResponse
    Private RetValue As Integer = 0
    Private syserno As Integer = 0
    Private ulTempValue As System.UInt64 = 0
    Private ulTimeStampHigh As System.UInt64 = 0
    Private ulTimeStampLow As System.UInt64 = 0
    Private bSingleData(4) As Byte

    Public Class ListData
        Private myItem As String
        Private myValue As Long

        Public Sub New(ByVal strItem As String, ByVal lValue As Long)

            Me.myItem = strItem
            Me.myValue = lValue
        End Sub

        Public ReadOnly Property Item() As String
            Get
                Return myItem
            End Get
        End Property

        Public ReadOnly Property Value() As Long
            Get
                Return myValue
            End Get
        End Property
    End Class

    Public Shared Function ConvertFromTime_T(ByVal time_t As Long) As DateTime
        Dim dt70 As New DateTime(1970, 1, 1, 0, 0, 0, 0)
        Dim tickTotal As Long = dt70.Ticks + (time_t * 10000000)
        Dim netTimeT = New DateTime(tickTotal)
        Return netTimeT
    End Function

    Private Sub ExtTemperatureRead_Load(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles MyBase.Load
        MX_Common = New MXCommon_WebReference.MXCommon()
        MSXE371x = New MSXE371x_WebReference.MSXE371x()
        MX_Common.Timeout = 2000
        MSXE371x.Timeout = 2000

        ' Thermocouple list box initialisation

        Dim Thermocouples = New ArrayList()
        Thermocouples.Add(New ListData("PT100", 100))
        Thermocouples.Add(New ListData("PT200", 200))
        Thermocouples.Add(New ListData("PT500", 500))
        Thermocouples.Add(New ListData("PT1000", 1000))

        LB_T_Thermocouple.DataSource = Thermocouples
        LB_T_Thermocouple.DisplayMember = "Item"
        LB_T_Thermocouple.ValueMember = "Value"

        ' Gain list box initialisation

        Dim Gains = New ArrayList()
        Gains.Add(New ListData("Auto mode", 0))

        LB_T_Gain.DataSource = Gains
        LB_T_Gain.DisplayMember = "Item"
        LB_T_Gain.ValueMember = "Value"

        ' Frequency list box initialisation

        Dim Frequencys = New ArrayList()
        Frequencys.Add(New ListData("10 Hz", 10))
        Frequencys.Add(New ListData("240 Hz", 240))

        LB_T_Frequency.DataSource = Frequencys
        LB_T_Frequency.DisplayMember = "Item"
        LB_T_Frequency.ValueMember = "Value"

        ' Display format list box initialisation

        Dim Formats = New ArrayList()
        Formats.Add(New ListData("mOhm", 0))
        Formats.Add(New ListData("C", 1))
        Formats.Add(New ListData("F", 2))

        LB_T_Format.DataSource = Formats
        LB_T_Format.DisplayMember = "Item"
        LB_T_Format.ValueMember = "Value"

    End Sub

    Private Sub B_StartSample_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles B_StartSample.Click
        If (TB_IPAddress_0.Text.ToString() = "") Or (TB_IPAddress_1.Text.ToString() = "") Or (TB_IPAddress_2.Text.ToString() = "") Or (TB_IPAddress_3.Text.ToString() = "") Then
            MessageBox.Show("The IP address you enter is not valid, please try again.", "IP address error", MessageBoxButtons.OK, MessageBoxIcon.Exclamation)
        Else
            If (Convert.ToUInt64(TB_IPAddress_0.Text.ToString(), 10) >= 0) And (Convert.ToUInt64(TB_IPAddress_0.Text.ToString(), 10) < 256) Then
                If (Convert.ToUInt64(TB_IPAddress_1.Text.ToString(), 10) >= 0) And (Convert.ToUInt64(TB_IPAddress_1.Text.ToString(), 10) < 256) Then
                    If (Convert.ToUInt64(TB_IPAddress_2.Text.ToString(), 10) >= 0) And (Convert.ToUInt64(TB_IPAddress_2.Text.ToString(), 10) < 256) Then
                        If (Convert.ToUInt64(TB_IPAddress_3.Text.ToString(), 10) >= 0) And (Convert.ToUInt64(TB_IPAddress_3.Text.ToString(), 10) < 256) Then
                            MX_Common.Url = "http://" + TB_IPAddress_0.Text + "." + TB_IPAddress_1.Text + "." + TB_IPAddress_2.Text + "." + TB_IPAddress_3.Text + ":5555"
                            MSXE371x.Url = MX_Common.Url

                            Try
                                ' External temperature initialisation

                                RetValue = MSXE371x.ExternalTemperatureInit(Convert.ToUInt32(LB_T_Thermocouple.SelectedValue), _
                                                                            Convert.ToUInt32(LB_T_Format.SelectedValue), _
                                                                            Convert.ToUInt32(LB_T_Gain.SelectedValue), _
                                                                            Convert.ToUInt32(LB_T_Frequency.SelectedValue), _
                                                                            0, 0, 0, 0, 0, syserno)
                                If RetValue <> 0 Then
                                    MessageBox.Show("ExternalTemperatureInit Error : " + RetValue.ToString(), "Function error", MessageBoxButtons.OK, MessageBoxIcon.Exclamation)
                                Else
                                    If (CType((1 / (Convert.ToDouble(LB_T_Frequency.SelectedValue) / 1000.0)), Integer) = 0) Then
                                        T_ReadValue.Interval = 1
                                    Else
                                        T_ReadValue.Interval = CType((1 / (Convert.ToDouble(LB_T_Frequency.SelectedValue) / 1000.0)), Integer)
                                    End If

                                    T_ReadValue.Enabled = True
                                    B_StartSample.Enabled = False
                                    B_StopSample.Enabled = True
                                    GB_Thermocouple.Enabled = False
                                End If
                            Catch ex As Exception
                                MessageBox.Show("ExternalTemperatureInit Exception occur : " + ex.Message, "Function error", MessageBoxButtons.OK, MessageBoxIcon.Exclamation)
                            End Try
                        Else
                            MessageBox.Show("The IP address (part 4) you enter is not valid, please try again.", "IP address error", MessageBoxButtons.OK, MessageBoxIcon.Exclamation)
                        End If
                    Else
                        MessageBox.Show("The IP address (part 3) you enter is not valid, please try again.", "IP address error", MessageBoxButtons.OK, MessageBoxIcon.Exclamation)
                    End If
                Else
                    MessageBox.Show("The IP address (part 2) you enter is not valid, please try again.", "IP address error", MessageBoxButtons.OK, MessageBoxIcon.Exclamation)
                End If
            Else
                MessageBox.Show("The IP address (part 1) you enter is not valid, please try again.", "IP address error", MessageBoxButtons.OK, MessageBoxIcon.Exclamation)
            End If
        End If
    End Sub

    Private Sub B_StopSample_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles B_StopSample.Click
        T_ReadValue.Enabled = False
        B_StartSample.Enabled = True
        B_StopSample.Enabled = False
        GB_Thermocouple.Enabled = True

        If B_StopSample.Enabled = True Then
            Try
                ' Release external temperature

                RetValue = MSXE371x.ExternalTemperatureRelease(syserno)

                If RetValue <> 0 Then
                    MessageBox.Show("ExternalTemperatureRelease Error : " + RetValue.ToString(), "Function error", MessageBoxButtons.OK, MessageBoxIcon.Exclamation)
                End If
            Catch ex As Exception
                MessageBox.Show("ExternalTemperatureRelease Exception occur : " + ex.Message, "Function error", MessageBoxButtons.OK, MessageBoxIcon.Exclamation)
            End Try
        End If

    End Sub

    Private Sub T_ReadValue_Tick(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles T_ReadValue.Tick
        Dim ulTimeStampLowmus As UInt64 = 0
        Dim ulTimeStampLowms As UInt64 = 0
        Dim sTimeStamp As String

        Try
            ' Read the external temperature

            Response = MSXE371x.ExternalTemperatureRead(ulTempValue, ulTimeStampHigh, ulTimeStampLow)
            If Response.iReturnValue <> 0 Then
                MessageBox.Show("ExternalTemperatureRead Error : " + Response.iReturnValue.ToString(), "Function error", MessageBoxButtons.OK, MessageBoxIcon.Exclamation)
            Else
                ulTimeStampLowms = CType((ulTimeStampLow / 1000), System.UInt64)
                ulTimeStampLowmus = CType((ulTimeStampLow Mod 1000), System.UInt64)
                sTimeStamp = "Time stamp: " + ConvertFromTime_T(CType(ulTimeStampHigh, Long)).ToString() + "." + ulTimeStampLowms.ToString("D3") + "." + ulTimeStampLowmus.ToString("D3")

                If Convert.ToUInt32(LB_T_Format.SelectedValue) = 0 Then
                    L_TemperatureInfo.Text = "External temperature = " + ulTempValue.ToString() + " mOhm. " + sTimeStamp
                ElseIf Convert.ToUInt32(LB_T_Format.SelectedValue) = 1 Then
                    bSingleData(0) = CType(((ulTempValue >> 0) And &HFF), Byte)
                    bSingleData(1) = CType(((ulTempValue >> 8) And &HFF), Byte)
                    bSingleData(2) = CType(((ulTempValue >> 16) And &HFF), Byte)
                    bSingleData(3) = CType(((ulTempValue >> 24) And &HFF), Byte)
                    L_TemperatureInfo.Text = "External temperature = " + BitConverter.ToSingle(bSingleData, 0).ToString("F3") + " C. " + sTimeStamp
                ElseIf Convert.ToUInt32(LB_T_Format.SelectedValue) = 2 Then
                    bSingleData(0) = CType(((ulTempValue >> 0) And &HFF), Byte)
                    bSingleData(1) = CType(((ulTempValue >> 8) And &HFF), Byte)
                    bSingleData(2) = CType(((ulTempValue >> 16) And &HFF), Byte)
                    bSingleData(3) = CType(((ulTempValue >> 24) And &HFF), Byte)
                    L_TemperatureInfo.Text = "External temperature = " + BitConverter.ToSingle(bSingleData, 0).ToString("F3") + " F. " + sTimeStamp
                End If
            End If
        Catch ex As Exception
            MessageBox.Show("ExternalTemperatureRead Exception occur : " + ex.Message, "Function error", MessageBoxButtons.OK, MessageBoxIcon.Exclamation)
        End Try

    End Sub

    Private Sub ExtTemperatureRead_FormClosed(ByVal sender As System.Object, ByVal e As System.Windows.Forms.FormClosedEventArgs) Handles MyBase.FormClosed
        If Not MSXE371x Is Nothing Then
            MSXE371x = Nothing
        End If
        If Not MX_Common Is Nothing Then
            MX_Common = Nothing
        End If

        System.GC.Collect()
    End Sub

    Private Sub ExtTemperatureRead_FormClosing(ByVal sender As System.Object, ByVal e As System.Windows.Forms.FormClosingEventArgs) Handles MyBase.FormClosing
        B_StopSample_Click(sender, Nothing)
    End Sub
End Class
