Imports System
Imports System.Windows.Forms
Imports System.Threading
Imports System.Net.Sockets

Namespace AddiData
    Namespace ADDIControls
        ' <summary>
        ' Add a propriety Index to the ComboBox Element
        ' </summary>
        Public Class AddiComboBox(Of ComboBox)
            Private i_Index As Integer = 0

            Public Sub AddiComboBox()
            End Sub

            Public Property Index() As Integer
                Set(ByVal value As Integer)
                    i_Index = value
                End Set
                Get
                    Return i_Index
                End Get
            End Property
        End Class

        ' <summary>
        ' Add a propriety Index to the CheckBox Element
        ' </summary>
        Public Class AddiCheckBox(Of CheckBox)
            Private i_Index As Integer = 0

            Public Sub AddiCheckBox()
            End Sub

            Public Property Index() As Integer
                Set(ByVal value As Integer)
                    i_Index = value
                End Set
                Get
                    Return i_Index
                End Get
            End Property
        End Class

        ' <summary>
        ' Add a propriety Index to the TextBox Element
        ' </summary>
        Public Class AddiTextBox(Of TextBox)
            Private i_Index As Integer = 0

            Public Sub AddiTextBox()
            End Sub

            Public Property Index() As Integer
                Set(ByVal value As Integer)
                    i_Index = value
                End Set
                Get
                    Return i_Index
                End Get
            End Property
        End Class
    End Namespace

    Namespace ADDISocket
        Public Class AsyncClientStream
            Public DataStreamSize As Integer = 0
            Public NbrOfFrames As Integer = 0
            Public SingleFramesSize As Integer = 0

            Private Const MAX_STREAM As Integer = 1000
            Private acReadCallBack As AsyncCallback = Nothing
            Private sClient As Socket = Nothing
            Private WriteStreamIndex As Integer = 0
            Private ReadStreamIndex As Integer = 0
            Private CurrentReceive As Integer = 0
            Private Initialised As Boolean = False
            Private ClientThread As Thread = Nothing
            Private CurrentSocketBuffer() As Byte = Nothing
            Private bStreamDataBuffer(MAX_STREAM)() As Byte
            Private receiveDone As ManualResetEvent = New ManualResetEvent(False)

            Private Function GetNextStreamBuffer(ByVal SocketClient As Socket) As Byte()
                Dim SaveReceiveBufferSize As Integer = 0

                ' Test if number of frames and frame size initialised

                If ((NbrOfFrames <> 0) And (SingleFramesSize <> 0)) Then
                    ' Test if initialised

                    If (Initialised = False) Then
                        ' Streem buffer initialisation

                        Initialised = True
                        WriteStreamIndex = 0
                        ReadStreamIndex = 0
                        CurrentReceive = 0
                        DataStreamSize = SingleFramesSize * NbrOfFrames

                        Try
                            SaveReceiveBufferSize = SocketClient.ReceiveBufferSize
                            If (SocketClient.ReceiveBufferSize < (DataStreamSize * MAX_STREAM * 2)) Then
                                SocketClient.ReceiveBufferSize = DataStreamSize * MAX_STREAM * 2
                            End If
                        Catch
                            SocketClient.ReceiveBufferSize = SaveReceiveBufferSize
                        End Try

                        For i As Integer = 0 To MAX_STREAM - 1
                            ReDim bStreamDataBuffer(i)(DataStreamSize)
                        Next i

                        Return bStreamDataBuffer(0)
                    Else
                        ' Test if no overflow

                        If (((WriteStreamIndex + 1) Mod MAX_STREAM) <> ReadStreamIndex) Then
                            WriteStreamIndex = (WriteStreamIndex + 1) Mod MAX_STREAM
                            Return bStreamDataBuffer(WriteStreamIndex)
                        Else
                            MessageBox.Show("Sample buffer overflow", "MSX-E sample error", MessageBoxButtons.OK, MessageBoxIcon.Exclamation)
                            Return Nothing
                        End If
                    End If
                Else
                    Return Nothing
                End If
            End Function

            Private Sub ClientThreadFct()
                Dim iFrameSize As Integer = 0

                Try
                    While ((Initialised = True) And (sClient.Connected) And (Not IsNothing(CurrentSocketBuffer)))
                        iFrameSize = sClient.Receive(CurrentSocketBuffer, CurrentReceive, DataStreamSize - CurrentReceive, SocketFlags.None)

                        If (iFrameSize > 0) Then
                            ' Save the number of receive datas
                            CurrentReceive = CurrentReceive + iFrameSize

                            'Test if receive all datas
                            If (CurrentReceive = DataStreamSize) Then
                                CurrentReceive = 0
                                CurrentSocketBuffer = GetNextStreamBuffer(sClient)
                                receiveDone.Set()
                            End If
                        ElseIf (Initialised = True) Then
                            MessageBox.Show("MSX-E module have close the socket connection.\nThis stop the sample", "MSX-E sample error", MessageBoxButtons.OK, MessageBoxIcon.Exclamation)
                            Initialised = False
                            receiveDone.Set()
                            Exit Sub
                        Else
                            Exit Sub
                        End If
                    End While

                Catch ex As Exception
                    If (Initialised = True) Then
                        Initialised = False
                        receiveDone.Set()
                        MessageBox.Show("Socket reception exception occur : " + ex.Message, "MSX-E sample error", MessageBoxButtons.OK, MessageBoxIcon.Exclamation)
                    End If
                End Try
            End Sub

            Public Function Start(ByVal SocketClient As Socket) As Boolean
                CurrentSocketBuffer = Me.GetNextStreamBuffer(SocketClient)

                ' Test if no error occur isnothing

                If (Not IsNothing(CurrentSocketBuffer)) Then
                    sClient = SocketClient
                    sClient.ReceiveTimeout = 0
                    ClientThread = New Thread(AddressOf ClientThreadFct)
                    ClientThread.Priority = ThreadPriority.Highest
                    ClientThread.Start()
                    Return True
                Else
                    Return False
                End If
            End Function

            Public Function StartViaCallback(ByVal SocketClient As Socket) As Boolean
                CurrentSocketBuffer = Me.GetNextStreamBuffer(SocketClient)

                ' Test if no error occur

                If (Not IsNothing(CurrentSocketBuffer)) Then
                    acReadCallBack = New AsyncCallback(AddressOf ReadCallBack)
                    sClient = SocketClient
                    sClient.ReceiveTimeout = 0
                    sClient.BeginReceive(CurrentSocketBuffer, 0, DataStreamSize, SocketFlags.None, acReadCallBack, Nothing)
                    Return True
                Else
                    Return False
                End If
            End Function

            Public Function Stoop() As Boolean
                If (Initialised = True) Then
                    Initialised = False
                    sClient.Shutdown(SocketShutdown.Both)
                    receiveDone.Set()
                    Return True
                Else
                    Return False
                End If
            End Function

            Public Function ReadData() As Byte()
                Dim ReceiveData As Byte() = Nothing

                While ((Initialised = True) And (IsNothing(ReceiveData)))
                    ' Test if any data avaible

                    If (WriteStreamIndex <> ReadStreamIndex) Then
                        ReceiveData = bStreamDataBuffer(ReadStreamIndex)
                        ReadStreamIndex = (ReadStreamIndex + 1) Mod MAX_STREAM
                    Else
                        ' Wait datas 
                        receiveDone.WaitOne()
                    End If
                End While

                Return ReceiveData
            End Function

            Private Sub ReadCallBack(ByVal asyn As IAsyncResult)
                Dim iFrameSize As Integer = 0

                ' Test if data available

                If ((asyn.IsCompleted = True) And (Initialised = True)) Then
                    ' Get the number of datas
                    iFrameSize = sClient.EndReceive(asyn)

                    If (iFrameSize <= 0) Then
                        MessageBox.Show("MSX-E module have close the socket connection.\nThis stop the sample", "MSX-E sample error", MessageBoxButtons.OK, MessageBoxIcon.Exclamation)
                        Initialised = False
                        receiveDone.Set()
                        Exit Sub
                    End If

                    ' Save the number of receive datas
                    CurrentReceive = CurrentReceive + iFrameSize

                    ' Test if receive all datas
                    If (CurrentReceive = DataStreamSize) Then
                        CurrentReceive = 0
                        CurrentSocketBuffer = GetNextStreamBuffer(sClient)
                        receiveDone.Set()
                    End If

                    ' Test if can start a new data read
                    If ((Initialised = True) And (Not IsNothing(CurrentSocketBuffer))) Then
                        If (sClient.Connected = True) Then
                            sClient.BeginReceive(CurrentSocketBuffer, CurrentReceive, DataStreamSize - CurrentReceive, SocketFlags.None, acReadCallBack, Nothing)
                        End If
                    End If
                End If
            End Sub
        End Class
    end Namespace
End Namespace