/** @file ExtTemperatureRead.c
 * 
 * ExtTemperatureRead: Read the temperature 
 *
 * @author C. Guinot
 * @date 26.09.2007
 * 
 * @version  $LastChangedRevision$
 * @version $LastChangedDate$
 * 
 * 
 * @par LICENCE
 * @verbatim
 *  Copyright (C) 2004,2005  ADDI-DATA GmbH for the source code of this module.
 *        
 *        ADDI-DATA GmbH 
 *        Airport Boulevard B210
 *        D-77836 Rheinmnster Germany 
 *        Tel: +49(0)7229/1847-0 
 *        Fax: +49(0)7229/1847-200
 *        http://www.addi-data-com 
 *        info@addi-data.com 
 *        
 *
 * @endverbatim   
 *
 * @par UPDATES                                                                                                          
 * @verbatim                                                                                                             
 *                                                                                                                       
 *  Date      : 
 *  Author        : 
 *  Description   : 
 *
 * @endverbatim     
 */ 

#include <stdio.h>
#include <conio.h>
#include <stdlib.h>
#include <time.h>


#include "..\..\..\..\Interface_Library\MSXE371x.nsmap"
#include "..\UTIL\utility.h"



//-------------------------------------------------------------------------------------------------------
//-------------------------------------------------------------------------------------------------------

/** Set temperature configuration.
 * 
 * @retval 0:	No error.
 * @retval 1:	Error.
 */
int i_SAMPLE_ExtTemperatureInitialization (struct soap *ps_SoapEnv, unsigned char *pc_SOAPIPAddress, unsigned long *pul_TempFormat)
{
	struct MSXE371x__Response Response;
	int i_Error = 0;
    unsigned long ulSensorType [3] = {100,500,1000};
    unsigned long ulSensorSelection = 0;
	unsigned long ulConvertMode = 0;
    unsigned long ulGainSelection = 0;
    unsigned long ulFrequencySelection = 0;


		do
		{
			printf("\n 0. PT100");
			printf("\n 1. PT500");
			printf("\n 2. PT1000");
			printf("\nSelect the temperature sensor : ");
			scanf("%lu", &ulSensorSelection);
		}
		while (	ulSensorSelection > 2);


		do
		{
			printf("\n 0. Disabled. Return the value in mOhm form");
            printf("\n 1. degree Celsius");
            printf("\n 2. degree Fahrenheit");
    		printf("\nSet the convert mode: ");
			scanf("%lu", &ulConvertMode);
		}
		while (	ulConvertMode > 2);
        *pul_TempFormat = ulConvertMode;


		do
		{
    		printf("\nEnter the acquisition frequency to use (10, 240): ");
			scanf("%lu", &ulFrequencySelection);
		}
		while (	(ulFrequencySelection != 10) && 
                (ulFrequencySelection != 240));

	
		printf ("\n");	

		i_Error = soap_call_MSXE371x__ExternalTemperatureInit (ps_SoapEnv, pc_SOAPIPAddress, SOAP_ACTION, 
									 		                   ulSensorType[ulSensorSelection], 
												               ulConvertMode,
                                                               ulGainSelection,
                                                               ulFrequencySelection,
                                                               0,
   												               0,
                                                               0,
                                                               0,
                                                               0,
                                                               &Response); 

		if ((i_Error == 0) && (Response.iReturnValue == 0))
		{			
			printf ("soap_call_MSXE371x__ExternalTemperatureInit OK\n");			
		} 
		else
		{
			printf("soap_call_MSXE371x__ExternalTemperatureInit SOAP error: %d, Response error: %d\n",i_Error,Response.iReturnValue);
		    if (i_Error != 0)
		        printf("SOAP Error : %s\n",*soap_faultstring(ps_SoapEnv));
			return 1;
		}

	
	return 0;
}

//-------------------------------------------------------------------------------------------------------

/** Release temperature.
 * 
 * @retval 0:	No error.
 * @retval 1:	Error.
 */
int i_SAMPLE_ReleaseExtTemperature (struct soap *ps_SoapEnv, unsigned char *pc_SOAPIPAddress)
{
	struct MSXE371x__Response Response;
	int i_Error = 0;

		i_Error = soap_call_MSXE371x__ExternalTemperatureRelease (ps_SoapEnv, pc_SOAPIPAddress, SOAP_ACTION, NULL,&Response); 
		if ((i_Error == 0) && (Response.iReturnValue == 0))
		{			
			printf ("soap_call_MSXE371x__ExternalTemperatureRelease OK\n");			
		} 
		else
		{
			printf("soap_call_MSXE371x__ExternalTemperatureRelease SOAP error: %d, Response error: %d\n",i_Error,Response.iReturnValue);
		    if (i_Error != 0)
		        printf("SOAP Error : %s\n",*soap_faultstring(ps_SoapEnv));
			return 1;
		}
	
	return 0;
}

//-------------------------------------------------------------------------------------------------------
//-------------------------------------------------------------------------------------------------------

void main(void)
{
	unsigned char                                    pc_IPAddressTemp[50];
	unsigned char                                    pc_IPAddress    [50];
	unsigned char                                    pc_SOAPIPAddress[50];
	DWORD                                            dw_Dummy          = 0;
	int                                               i_Error          = 0;
	int                                               i_ReturnValue    = 0;
	int                                               i_key            = 0;
    unsigned long                                    ul_TempFormat     = 0;
	struct soap                                     *ps_SoapEnv        = NULL;
	struct MXCommon__ByteArrayResponse                s_ByteArrayResponse;
	struct MSXE371x__ExternalTemperatureReadResponse  s_ReadTemperatureResponse;
	float                                           *pf_FloatValue = (float *) &s_ReadTemperatureResponse.ulValue;

	/* Print sample information */
	printf("+---------------------------------------------------+\n");
	printf("| MSX-E371x                                         |\n"); 
 	printf("| Read the external temperature                     |\n");  
	printf("+---------------------------------------------------+\n");
	printf("| <ESC> exit sample                                 |\n");
	printf("+---------------------------------------------------+\n");
	printf("\n");

	/* Initialise the SOAP connection */
	if (i_SOAP_Init(&ps_SoapEnv) == 0)
	{
		/* Get the IP address */
		printf("Enter the module IP Address (XXX.XXX.XXX.XXX): ");
		gets(pc_IPAddressTemp);

		/* Format and set the IP address */
		v_UTILS_ClearNullInIPAddress(pc_IPAddressTemp,pc_IPAddress);
		sprintf(pc_SOAPIPAddress,"%s:5555", pc_IPAddress);
		printf("\nSOAP IP Address: %s\n", pc_SOAPIPAddress);

		/* Get the module type */
		i_Error = soap_call_MXCommon__GetModuleType(ps_SoapEnv, pc_SOAPIPAddress, SOAP_ACTION, &dw_Dummy, &s_ByteArrayResponse);
		if ((i_Error == 0) && (s_ByteArrayResponse.sResponse.iReturnValue == 0))
		{
			printf("Module Type: %s\n", s_ByteArrayResponse.sArray.__ptr);
									
			if (i_SAMPLE_ExtTemperatureInitialization (ps_SoapEnv, pc_SOAPIPAddress,&ul_TempFormat) == 0) 
            {


				/* Start the counter */
				printf ("\nPress a key to start reading the temperature.");
				printf ("\nESC to quit.\n");
                getch();

				do {
					
					/* Read the counter value */
					i_Error = soap_call_MSXE371x__ExternalTemperatureRead (ps_SoapEnv, pc_SOAPIPAddress, SOAP_ACTION, 
                                                                           NULL, 
                                                                           &s_ReadTemperatureResponse); 
					if ((i_Error == 0) && (s_ReadTemperatureResponse.sResponse.iReturnValue == 0))
					{			
                        if (ul_TempFormat = 0)
                        {
                        printf("Value : %lumOhms",s_ReadTemperatureResponse.ulValue);
                        }
                        else if (ul_TempFormat = 1)
                        {
                        printf("Value : %f degree Celsius", *pf_FloatValue);
                        }
                        else if (ul_TempFormat = 2)
                        {
                        printf("Value : %f degree Fahrenheit", *pf_FloatValue);
                        }

                        DisplayTimeStamp (s_ReadTemperatureResponse.ulTimeStampHigh, s_ReadTemperatureResponse.ulTimeStampLow);
                        printf ("\r");

					} 
					else
					{
						printf("soap_call_MSXE371x__ExternalTemperatureRead SOAP error: %d, Response error: %d\n",i_Error,s_ReadTemperatureResponse.sResponse.iReturnValue);
						if (i_Error != 0) {
							printf("SOAP Error : %s\n",*soap_faultstring(ps_SoapEnv));
							break;
						}
					}

					/* 
					 * gSOAP doesn't release automatically the memory that 
					 * it allocates to deserialize data, that's why both 
					 * following functions have to be used to release the 
					 * allocated memory.
					 */
					soap_destroy(ps_SoapEnv);
					soap_end(ps_SoapEnv);

					/* Press any key to clear the counter */
					if (kbhit()) 
					{
						i_key = getch();
					}

				/* Exit on ESC */
				} while (i_key != 27);

			}
			
			i_SAMPLE_ReleaseExtTemperature (ps_SoapEnv, pc_SOAPIPAddress);

		}
		else
		{
			printf("soap_call_MXCommon__GetModuleType SOAP error: %d, Response error :%d\n", i_Error,s_ByteArrayResponse.sResponse.iReturnValue);
            if (i_Error != 0)
                 printf("SOAP Error : %s\n",*soap_faultstring(ps_SoapEnv));
		}

		/* Close the SOAP connection */
		i_SOAP_Close (ps_SoapEnv);
	}

	/* Print End programme informations */
	printf("\n\n");
	printf("+---------------------------------------------------+\n");
	printf("| Program stopped                                   |\n");
	printf("+---------------------------------------------------+\n");
	printf("| Press any key to exit from sample                 |\n");
	printf("+---------------------------------------------------+\n");

	while (!(kbhit()));
}
