/** @file sample01.c
 * 
 * IncCounterReadWriteClear: Read / Write / Clear the incremental counter
 *
 * @author C. Guinot
 * @date 26.09.2007
 * 
 * @version  $LastChangedRevision$
 * @version $LastChangedDate$
 * 
 * 
 * @par LICENCE
 * @verbatim
 *  Copyright (C) 2004,2005  ADDI-DATA GmbH for the source code of this module.
 *        
 *        ADDI-DATA GmbH 
 *        Airport Boulevard B210
 *        D-77836 Rheinmnster Germany 
 *        Tel: +49(0)7229/1847-0 
 *        Fax: +49(0)7229/1847-200
 *        http://www.addi-data-com 
 *        info@addi-data.com 
 *        
 *
 * @endverbatim   
 *
 * @par UPDATES                                                                                                          
 * @verbatim                                                                                                             
 *                                                                                                                       
 *  Date      : 
 *  Author        : 
 *  Description   : 
 *
 * @endverbatim     
 */ 

#include <stdio.h>
#include <conio.h>
#include <stdlib.h>
#include <time.h>


#include "..\..\..\..\Interface_Library\MSXE371x.nsmap"
#include "..\UTIL\utility.h"

//-------------------------------------------------------------------------------------------------------
//-------------------------------------------------------------------------------------------------------

/** Set counter configuration.
 * 
 * @retval 0:	No error.
 * @retval 1:	Error.
 */
int i_SAMPLE_CounterInitialization (struct soap *ps_SoapEnv, unsigned char *pc_SOAPIPAddress)
{
	struct MSXE371x__Response Response;
	int i_Error = 0;
	unsigned long  ulCounterMode = 0;
	unsigned long  ulCounterValue = 0;
	unsigned long  ulCounterOption = 0;

		printf ("\n");

		do
		{
			printf("SIMPLE MODE     = %lu\n", MSXE371X_COUNTER_SIMPLE_MODE);
			printf("DOUBLE MODE     = %lu\n", MSXE371X_COUNTER_DOUBLE_MODE);
			printf("QUADRUPLE MODE  = %lu\n", MSXE371X_COUNTER_QUADRUPLE_MODE);
			printf("DIRECT MODE     = %lu\n", MSXE371X_COUNTER_DIRECT_MODE);
			printf("Set the counter mode : ");
			scanf("%lu", &ulCounterMode);
		}
		while (	(ulCounterMode != MSXE371X_COUNTER_QUADRUPLE_MODE) && (ulCounterMode != MSXE371X_COUNTER_DOUBLE_MODE) &&
				(ulCounterMode != MSXE371X_COUNTER_SIMPLE_MODE) && (ulCounterMode != MSXE371X_COUNTER_DIRECT_MODE) );

		printf ("\n");

		do
		{
			if (ulCounterMode != MSXE371X_COUNTER_DIRECT_MODE)
               {
               printf("HYSTERESIS ON  = %lu\n",MSXE371X_COUNTER_HYSTERESIS_ON);
               printf("HYSTERESIS OFF = %lu\n",MSXE371X_COUNTER_HYSTERESIS_OFF);
               }
            else
               {
               printf("INCREMENT = %lu\n",MSXE371X_COUNTER_INCREMENT);
               printf("DECREMENT = %lu\n",MSXE371X_COUNTER_DECREMENT);
               }

			printf("Set the counter option : ");
			scanf("%lu", &ulCounterOption);
		}
		while (	(ulCounterOption != MSXE371X_COUNTER_HYSTERESIS_ON) && (ulCounterOption != MSXE371X_COUNTER_HYSTERESIS_OFF) &&
				(ulCounterOption != MSXE371X_COUNTER_INCREMENT) && (ulCounterOption != MSXE371X_COUNTER_DECREMENT) );

	
		printf ("\n");	

		i_Error = soap_call_MSXE371x__IncCounterInit (ps_SoapEnv, pc_SOAPIPAddress, SOAP_ACTION, 
									 		          ulCounterMode, 
												      ulCounterOption,
   												      0,
                                                      0,
                                                      0,
                                                      0,
                                                      &Response); 

		if ((i_Error == 0) && (Response.iReturnValue == 0))
		{			
			printf ("soap_call_MSXE371x__IncCounterInit OK\n");			
		} 
		else
		{
			printf("soap_call_MSXE371x__IncCounterInit SOAP error: %d, Response error: %d\n",i_Error,Response.iReturnValue);
		    if (i_Error != 0)
		        printf("SOAP Error : %s\n",*soap_faultstring(ps_SoapEnv));
			return 1;
		}

		/* You can specify a value at witch the counter will began to count */ 
		do
		{
			printf("Enter the value to write on the counter : ");
			scanf("%lu", &ulCounterValue);
		}
		while (ulCounterValue > 4294967295UL);
		
		printf ("\n");	
	
		i_Error = soap_call_MSXE371x__IncCounterWrite32BitValue (ps_SoapEnv, pc_SOAPIPAddress, SOAP_ACTION, ulCounterValue, &Response); 
		if ((i_Error == 0) && (Response.iReturnValue == 0))
		{			
			printf ("soap_call_MSXE371x__IncCounterWrite32BitValue OK\n");			
		} 
		else
		{
			printf("soap_call_MSXE371x__IncCounterWrite32BitValue SOAP error: %d, Response error: %d\n",i_Error,Response.iReturnValue);
			if (i_Error != 0)
				printf("SOAP Error : %s\n",*soap_faultstring(ps_SoapEnv));
			return 1;
		}
	
	return 0;
}

//-------------------------------------------------------------------------------------------------------

/** Release counter.
 * 
 * @retval 0:	No error.
 * @retval 1:	Error.
 */
int i_SAMPLE_ReleaseCounter (struct soap *ps_SoapEnv, unsigned char *pc_SOAPIPAddress)
{
	struct MSXE371x__Response Response;
	int i_Error = 0;

		i_Error = soap_call_MSXE371x__IncCounterRelease (ps_SoapEnv, pc_SOAPIPAddress, SOAP_ACTION, NULL,&Response); 
		if ((i_Error == 0) && (Response.iReturnValue == 0))
		{			
			printf ("soap_call_MSXE371x__IncCounterRelease OK\n");			
		} 
		else
		{
			printf("soap_call_MSXE371x__IncCounterRelease SOAP error: %d, Response error: %d\n",i_Error,Response.iReturnValue);
		    if (i_Error != 0)
		        printf("SOAP Error : %s\n",*soap_faultstring(ps_SoapEnv));
			return 1;
		}
	
	return 0;
}

//-------------------------------------------------------------------------------------------------------
//-------------------------------------------------------------------------------------------------------

void main(void)
{
	struct soap *ps_SoapEnv = NULL;
	struct MXCommon__ByteArrayResponse  s_ByteArrayResponse;
	struct MSXE371x__IncCounterRead32BitValueResponse TimeStampResponse;
	struct MSXE371x__Response Response;
	unsigned char pc_IPAddressTemp[50];
	unsigned char pc_IPAddress[50];
	unsigned char pc_SOAPIPAddress[50];
	DWORD dw_Dummy = 0;
	int i_Error = 0;
	int i_ReturnValue = 0;
	int i_key = 0;
	unsigned long  ulCounterValue = 0;

	/* Print sample information */
	printf("+---------------------------------------------------+\n");
	printf("| MSX-E371x                                         |\n"); 
 	printf("| Read / Write / Clear the incremental counter      |\n");  
	printf("+---------------------------------------------------+\n");
	printf("| <ESC> exit sample                                 |\n");
	printf("+---------------------------------------------------+\n");
	printf("\n");

	/* Initialise the SOAP connection */
	if (i_SOAP_Init(&ps_SoapEnv) == 0)
	{
		/* Get the IP address */
		printf("Enter the module IP Address (XXX.XXX.XXX.XXX): ");
		gets(pc_IPAddressTemp);

		/* Format and set the IP address */
		v_UTILS_ClearNullInIPAddress(pc_IPAddressTemp,pc_IPAddress);
		sprintf(pc_SOAPIPAddress,"%s:5555", pc_IPAddress);
		printf("\nSOAP IP Address: %s\n", pc_SOAPIPAddress);

		/* Get the module type */
		i_Error = soap_call_MXCommon__GetModuleType(ps_SoapEnv, pc_SOAPIPAddress, SOAP_ACTION, &dw_Dummy, &s_ByteArrayResponse);
		if ((i_Error == 0) && (s_ByteArrayResponse.sResponse.iReturnValue == 0))
		{
			printf("Module Type: %s\n", s_ByteArrayResponse.sArray.__ptr);
									
			if (i_SAMPLE_CounterInitialization (ps_SoapEnv, pc_SOAPIPAddress) == 0) {


				/* Start the counter */
				printf ("Press a key to clear the counter.\n");
				printf ("ESC to quit.\n");

				do {
					
					/* Read the counter value */
					i_Error = soap_call_MSXE371x__IncCounterRead32BitValue (ps_SoapEnv, pc_SOAPIPAddress, SOAP_ACTION, NULL, &TimeStampResponse); 
					if ((i_Error == 0) && (TimeStampResponse.sResponse.iReturnValue == 0))
					{			
						printf ("Counter = %10lu Timestamp: ", TimeStampResponse.ulValue);
						DisplayTimeStamp (TimeStampResponse.ulTimeStampHigh, TimeStampResponse.ulTimeStampLow);
						printf ("\r");
					} 
					else
					{
						printf("soap_call_MSXE371x__IncCounterRead32BitValue SOAP error: %d, Response error: %d\n",i_Error,TimeStampResponse.sResponse.iReturnValue);
						if (i_Error != 0) {
							printf("SOAP Error : %s\n",*soap_faultstring(ps_SoapEnv));
							break;
						}
					}

					/* Press any key to clear the counter */
					if (kbhit()) 
					{
						i_key = getch();
                        printf("\n");

					
						i_Error = soap_call_MSXE371x__IncCounterClear (ps_SoapEnv, pc_SOAPIPAddress, SOAP_ACTION, NULL, &Response); 
						if ((i_Error == 0) && (Response.iReturnValue == 0))
						{			
							printf ("soap_call_MSXE371x__IncCounterClear OK\n");			
						} 
						else
						{
							printf("soap_call_MSXE371x__IncCounterClear SOAP error: %d, Response error: %d\n",i_Error,Response.iReturnValue);
							if (i_Error != 0)
								printf("SOAP Error : %s\n",*soap_faultstring(ps_SoapEnv));
							break;
						}						

					}

					/* 
					 * gSOAP doesn't release automatically the memory that 
					 * it allocates to deserialize data, that's why both 
					 * following functions have to be used to release the 
					 * allocated memory.
					 */
					soap_destroy(ps_SoapEnv);
					soap_end(ps_SoapEnv);

				/* Exit on ESC */
				} while (i_key != 27);

			}
			
			i_SAMPLE_ReleaseCounter (ps_SoapEnv, pc_SOAPIPAddress);

		}
		else
		{
			printf("soap_call_MXCommon__GetModuleType SOAP error: %d, Response error :%d\n", i_Error,s_ByteArrayResponse.sResponse.iReturnValue);
            if (i_Error != 0)
                 printf("SOAP Error : %s\n",*soap_faultstring(ps_SoapEnv));
		}

		/* Close the SOAP connection */
		i_SOAP_Close (ps_SoapEnv);
	}

	/* Print End programme informations */
	printf("\n\n");
	printf("+---------------------------------------------------+\n");
	printf("| Program stopped                                   |\n");
	printf("+---------------------------------------------------+\n");
	printf("| Press any key to exit from sample                 |\n");
	printf("+---------------------------------------------------+\n");

	while (!(kbhit()));
}
