 /** @file TransducerAutoRefresh.c
 * AutoRefresh : MSX-E371x : Transducer Auto Refresh acquisition
 * 
 *
 * @author C. Guinot
 * @date 26.09.2007
 * 
 * @version  $LastChangedRevision$
 * @version $LastChangedDate$
 * 
 * 
 * @par LICENCE
 * @verbatim
 *  Copyright (C) 2004,2005  ADDI-DATA GmbH for the source code of this module.
 *        
 *        ADDI-DATA GmbH 
 *        Airport Boulevard B210
 *        D-77836 Rheinmnster Germany 
 *        Tel: +49(0)7229/1847-0 
 *        Fax: +49(0)7229/1847-200
 *        http://www.addi-data-com 
 *        info@addi-data.com 
 *        
 *
 * @endverbatim   
 *
 * @par UPDATES                                                                                                          
 * @verbatim                                                                                                             
 *                                                                                                                       
 *  Date      : 
 *  Author        : 
 *  Description   : 
 *
 * @endverbatim     
 */ 

#include <stdio.h>
#include <conio.h>
#include <stdlib.h>
#include <time.h>


#include "..\..\..\..\Interface_Library\MSXE371x.nsmap"
#include "..\UTIL\utility.h"
#include "..\UTIL\socket.h"

#define NBR_OF_CHANNELS 8

/********************************/
/* Console management functions */
/********************************/
int    i_X             = 0;
int    i_Y             = 0;
INT    i_ThreadStartY  = 0;
INT    i_ThreadCurentY = 0;

HANDLE h_ConsoleHandle = NULL;
HANDLE h_ConsoleMutex  = NULL;


/*******************************************************************************************************/
/*                                       CONSOLE Functions                                             */               
/*******************************************************************************************************/

/*gotoxy*/
/*+----------------------------------------------------------------------------+*/
/*| Function   Name   : void gotoxy (int X, int Y)                             |*/
/*+----------------------------------------------------------------------------+*/
/*| Task              : Set the cursor to the selected postion                 |*/
/*+----------------------------------------------------------------------------+*/
/*| Input Parameters  : int X : Coordinate X                                   |*/
/*|                     int Y : Coordinate Y                                   |*/
/*+----------------------------------------------------------------------------+*/
/*| Output Parameters : -                                                      |*/
/*+----------------------------------------------------------------------------+*/
/*| Return Value      : -                                                      |*/
/*+----------------------------------------------------------------------------+*/
void gotoxy(int X, int Y)
{
	/************************/
	/* Variable declaration */
	/************************/
	COORD s_CoordScreen = { 0, 0 };
        
	/******************************/
	/* Test if consol handle open */
	/******************************/
	if (h_ConsoleHandle != NULL)
	{
		/***************************/
		/* Save the new coordinate */
		/***************************/
		s_CoordScreen.X = X;
		s_CoordScreen.Y = Y;

		/****************************/
		/* Put the cursor at (X, Y) */
		/****************************/
		SetConsoleCursorPosition(h_ConsoleHandle, s_CoordScreen);
	}
}

/*wherexy*/
/*+----------------------------------------------------------------------------+*/
/*| Function   Name   : void wherexy (int *X, int *Y)                          |*/
/*+----------------------------------------------------------------------------+*/
/*| Task              : Get the cursor postion                                 |*/
/*+----------------------------------------------------------------------------+*/
/*| Input Parameters  : -                                                      |*/
/*+----------------------------------------------------------------------------+*/
/*| Output Parameters : int * X : Coordinate X                                 |*/
/*|                     int * Y : Coordinate Y                                 |*/
/*+----------------------------------------------------------------------------+*/
/*| Return Value      : -                                                      |*/
/*+----------------------------------------------------------------------------+*/
void wherexy (int *X, int *Y)
{
	/************************/
	/* Variable declaration */
	/************************/
	CONSOLE_SCREEN_BUFFER_INFO  csbi;

	/******************************/
	/* Test if consol handle open */
	/******************************/
	if (h_ConsoleHandle != NULL)
	{
		/***********************************************************/
		/* get the number of character cells in the current buffer */
		/***********************************************************/
		GetConsoleScreenBufferInfo (h_ConsoleHandle, &csbi); 

		/*************************** */
		/* Get the current position  */
		/*************************** */
		*X = csbi.dwCursorPosition.X;
		*Y = csbi.dwCursorPosition.Y;
	}
}

/*clrscr*/
/*+----------------------------------------------------------------------------+*/
/*| Function   Name   : void clrscr (void)                                     |*/
/*+----------------------------------------------------------------------------+*/
/*| Task              : Clear the screen and set the cursor to postion 0, 0    |*/
/*+----------------------------------------------------------------------------+*/
/*| Input Parameters  : -                                                      |*/
/*+----------------------------------------------------------------------------+*/
/*| Output Parameters : -                                                      |*/
/*+----------------------------------------------------------------------------+*/
/*| Return Value      : -                                                      |*/
/*+----------------------------------------------------------------------------+*/
void clrscr(void)
{
	/************************/
	/* Variable declaration */
	/************************/
	DWORD                      dw_ConSize;
	DWORD                      dw_CharsWritten; 
	COORD                       s_CoordScreen = { 0, 0 };
	CONSOLE_SCREEN_BUFFER_INFO  csbi;

	/******************************/
	/* Test if consol handle open */
	/******************************/
	if (h_ConsoleHandle != NULL)
	{
		/***********************************************************/
		/* get the number of character cells in the current buffer */
		/***********************************************************/
		GetConsoleScreenBufferInfo (h_ConsoleHandle, &csbi); 

		/**********************************/
		/* Get the number of blank to set */
		/**********************************/
		dw_ConSize = csbi.dwSize.X * csbi.dwSize.Y; 

		/**************************************/
		/* Fill the entire screen with blanks */
		/**************************************/
		FillConsoleOutputCharacter(h_ConsoleHandle, 
									(TCHAR) ' ',
									dw_ConSize,
									s_CoordScreen,
									&dw_CharsWritten ); 

		/****************************/
		/* Put the cursor at (0, 0) */
		/****************************/
		SetConsoleCursorPosition(h_ConsoleHandle, s_CoordScreen);
		wherexy (&i_X, &i_Y);
	}
}

/*SetNewPosition*/
/*+----------------------------------------------------------------------------+*/
/*| Function   Name   : void SetNewPosition (int i_YToInsert)                  |*/
/*+----------------------------------------------------------------------------+*/
/*| Task              : Test and set the number of line to insert              |*/
/*+----------------------------------------------------------------------------+*/
/*| Input Parameters  : i_YToInsert : Number of line to insert                 |*/
/*+----------------------------------------------------------------------------+*/
/*| Output Parameters : -                                                      |*/
/*+----------------------------------------------------------------------------+*/
/*| Return Value      : -                                                      |*/
/*+----------------------------------------------------------------------------+*/
void SetNewPosition(int i_YToInsert)
{
	/************************/
	/* Variable declaration */
	/************************/
	CONSOLE_SCREEN_BUFFER_INFO csbi;

	/****************************************************/
	/* Get the current cursor position and the max X, Y */
	/****************************************************/
	GetConsoleScreenBufferInfo (h_ConsoleHandle, &csbi); 

	/*************************************/
	/* Test the number of line to insert */
	/*************************************/
	if ((csbi.dwCursorPosition.Y + i_YToInsert) >= csbi.dwSize.Y)
	{
		/********************/
		/* Clear the screan */
		/********************/
		clrscr();
	}
	else
	{
		/**************************/
		/* Save te new Y position */
		/**************************/
		i_Y = csbi.dwCursorPosition.Y + i_YToInsert;
	}
}

/*TestPosition*/
/*+----------------------------------------------------------------------------+*/
/*| Function   Name   : void TestPosition (int i_YToInsert)                    |*/
/*+----------------------------------------------------------------------------+*/
/*| Task              : Test the number of line to insert                      |*/
/*+----------------------------------------------------------------------------+*/
/*| Input Parameters  : i_YToInsert : Number of line to insert                 |*/
/*+----------------------------------------------------------------------------+*/
/*| Output Parameters : -                                                      |*/
/*+----------------------------------------------------------------------------+*/
/*| Return Value      : -                                                      |*/
/*+----------------------------------------------------------------------------+*/
void TestPosition(int i_YToInsert)
{
	/************************/
	/* Variable declaration */
	/************************/
	CONSOLE_SCREEN_BUFFER_INFO csbi;

	/****************************************************/
	/* Get the current cursor position and the max X, Y */
	/****************************************************/
	GetConsoleScreenBufferInfo (h_ConsoleHandle, &csbi); 

	/*************************************/
	/* Test the number of line to insert */
	/*************************************/
	if ((csbi.dwCursorPosition.Y + i_YToInsert) >= csbi.dwSize.Y)
	{
		/********************/
		/* Clear the screan */
		/********************/
		clrscr();
	}
}


int i_DisplayAndSelectTransducerType(struct soap *ps_SoapEnv,PCHAR pc_SOAPIPAddress,DWORD * pdw_TransducerTypeSelection)
{
	struct MSXE371x__unsignedlongResponse s_Response;
	struct MSXE371x__TransducerGetTypeInformationResponse TypeInfoResponse;

	INT i_Error = 0;
	DWORD dw_Cpt = 0;
	DWORD pdw_SelectionValueArray[100];
	DWORD dw_Choice = 0;
	DWORD dw_Dummy = 0;
  
	/* Get the number of transducer type */
	i_Error = soap_call_MSXE371x__TransducerGetNbrOfType(ps_SoapEnv, pc_SOAPIPAddress, SOAP_ACTION, &dw_Dummy, &s_Response);
	if ((i_Error == 0) && (s_Response.sResponse.iReturnValue == 0))
	{
		printf("\nNumber of transducer types: %d\n", s_Response.ulValue);

		for (dw_Cpt = 0; dw_Cpt < s_Response.ulValue; dw_Cpt ++)
		{
			/* Get the transducer information */
			i_Error = soap_call_MSXE371x__TransducerGetTypeInformation (ps_SoapEnv,pc_SOAPIPAddress, SOAP_ACTION, dw_Cpt, &TypeInfoResponse);
			if ((i_Error == 0) && (TypeInfoResponse.sResponse.iReturnValue == 0))
			{
				printf("\n");
				printf("Transducer Index: %d\n", dw_Cpt);
				printf("Selection Index: %d\n", TypeInfoResponse.ulTransducerSelectionIndex);

				pdw_SelectionValueArray[dw_Cpt] = TypeInfoResponse.ulTransducerSelectionIndex;
				printf("Name: %s\n", TypeInfoResponse.pcName);
				printf("Calibration status: %d\n", TypeInfoResponse.ulCalibrationStatus);
				printf("Type value: %d\n", TypeInfoResponse.ulType);
				printf("Frequency: %d Hz\n", TypeInfoResponse.ulFrequency);
				printf("Impedance: %d Ohm\n", TypeInfoResponse.ulImpedance);
				printf("Nominal Voltage: %lf V\n", TypeInfoResponse.dVeff);
				printf("Sensibility: %lf mV/V/mm\n", TypeInfoResponse.dSensibility);
				printf("Range: +/- %lf mm\n", TypeInfoResponse.dRange);
			}
			else
			{
				printf("soap_call_MSXE371x__Transducer_GetTypeInformation Error\n");
				return -2;
			}
		}

		do 
		{
			printf("\nEnter the index of the transducer type you want ot use (0 to %d): ", (s_Response.ulValue - 1));
			scanf("%d",&dw_Choice);
		}
		while (dw_Choice >= s_Response.ulValue);
		*pdw_TransducerTypeSelection = pdw_SelectionValueArray[dw_Choice];
	}
	else
	{
		printf("soap_call_MSXE371x__Transducer_GetNbrOfType Error\n");
		return -1;
	}
	return 0;
}



//-------------------------------------------------------------------------------------------------------
//-------------------------------------------------------------------------------------------------------

/** Set temperature configuration.
 * 
 * @retval 0:	No error.
 * @retval 1:	Error.
 */
int i_SAMPLE_ExtTemperatureInitialization (struct soap *ps_SoapEnv, unsigned char *pc_SOAPIPAddress, unsigned long *pul_TempFormat)
{
	struct MSXE371x__Response Response;
	int i_Error = 0;
    unsigned long ulSensorType [3] = {100,500,1000};
    unsigned long ulSensorSelection = 0;
	unsigned long ulConvertMode = 0;
    unsigned long ulGainSelection = 0;
    unsigned long ulFrequencySelection = 0;
    unsigned long ulPowerSaveMode = 0;


        printf("\n EXTERNAL TEMPERATURE INITIALIZATION");
        printf("\n -----------------------------------\n");

		do
		{
			printf("\n 0. PT100");
			printf("\n 1. PT500");
			printf("\n 2. PT1000");
			printf("\nSelect the temperature sensor : ");
			scanf("%lu", &ulSensorSelection);
		}
		while (	ulSensorSelection > 2);


		do
		{
			printf("\n 0. Return the value in mOhm form");
            printf("\n 1. degree Celsius");
            printf("\n 2. degree Fahrenheit");
    		printf("\nSet the convert mode: ");
			scanf("%lu", &ulConvertMode);
		}
		while (	ulConvertMode > 2);

        *pul_TempFormat = ulConvertMode;


		do
		{
    		printf("\nEnter the acquisition frequency to use (10, 240): ");
			scanf("%lu", &ulFrequencySelection);
		}
		while (	(ulFrequencySelection != 10) && 
                (ulFrequencySelection != 240) );

	
   		do
		{
            printf("\n 0 : Disable power save mode");
            printf("\n 1 : Enable power save mode. In this case the value can only be read if any");
			printf("\n     transducer acquisition is enabled");
    		printf("\nPower save selection :");
			scanf("%lu", &ulPowerSaveMode);
		}
		while (ulPowerSaveMode > 1);

		printf ("\n");	

		i_Error = soap_call_MSXE371x__ExternalTemperatureInit (ps_SoapEnv, pc_SOAPIPAddress, SOAP_ACTION, 
									 		                   ulSensorType[ulSensorSelection], 
												               ulConvertMode,
                                                               ulGainSelection,
                                                               ulFrequencySelection,
                                                               ulPowerSaveMode,
   												               0,
                                                               0,
                                                               0,
                                                               0,
                                                               &Response); 

		if ((i_Error == 0) && (Response.iReturnValue == 0))
		{			
			printf ("soap_call_MSXE371x__ExternalTemperatureInit OK\n");			
		} 
		else
		{
			printf("soap_call_MSXE371x__ExternalTemperatureInit SOAP error: %d, Response error: %d\n",i_Error,Response.iReturnValue);
		    if (i_Error != 0)
		        printf("SOAP Error : %s\n",*soap_faultstring(ps_SoapEnv));
			return 1;
		}

	
	return 0;
}

//-------------------------------------------------------------------------------------------------------

/** Release temperature.
 * 
 * @retval 0:	No error.
 * @retval 1:	Error.
 */
int i_SAMPLE_ReleaseExtTemperature (struct soap *ps_SoapEnv, unsigned char *pc_SOAPIPAddress)
{
	struct MSXE371x__Response Response;
	int i_Error = 0;

		i_Error = soap_call_MSXE371x__ExternalTemperatureRelease (ps_SoapEnv, pc_SOAPIPAddress, SOAP_ACTION, NULL,&Response); 
		if ((i_Error == 0) && (Response.iReturnValue == 0))
		{			
			printf ("soap_call_MSXE371x__ExternalTemperatureRelease OK\n");			
		} 
		else
		{
			printf("soap_call_MSXE371x__ExternalTemperatureRelease SOAP error: %d, Response error: %d\n",i_Error,Response.iReturnValue);
		    if (i_Error != 0)
		        printf("SOAP Error : %s\n",*soap_faultstring(ps_SoapEnv));
			return 1;
		}
	
	return 0;
}



/** Set counter configuration.
 * 
 * @retval 0:	No error.
 * @retval 1:	Error.
 */
int i_SAMPLE_CounterInitialization (struct soap *ps_SoapEnv, unsigned char *pc_SOAPIPAddress)
{
	struct MSXE371x__Response Response;
	int i_Error = 0;
	unsigned long  ulCounterMode = 0;
	unsigned long  ulCounterValue = 0;
	unsigned long  ulCounterOption = 0;

        printf("\n INCREMENTAL COUNTER INITIALIZATION :");
        printf("\n ----------------------------------\n\n");

		do
		{
			printf("SIMPLE MODE     = %lu\n", MSXE371X_COUNTER_SIMPLE_MODE);
			printf("DOUBLE MODE     = %lu\n", MSXE371X_COUNTER_DOUBLE_MODE);
			printf("QUADRUPLE MODE  = %lu\n", MSXE371X_COUNTER_QUADRUPLE_MODE);
			printf("DIRECT MODE     = %lu\n", MSXE371X_COUNTER_DIRECT_MODE);
			printf("Set the counter mode : ");
			scanf("%lu", &ulCounterMode);
		}
		while (	(ulCounterMode != MSXE371X_COUNTER_QUADRUPLE_MODE) && (ulCounterMode != MSXE371X_COUNTER_DOUBLE_MODE) &&
				(ulCounterMode != MSXE371X_COUNTER_SIMPLE_MODE) && (ulCounterMode != MSXE371X_COUNTER_DIRECT_MODE) );

		printf ("\n");

		do
		{
			if (ulCounterMode != MSXE371X_COUNTER_DIRECT_MODE)
               {
               printf("HYSTERESIS ON  = %lu\n",MSXE371X_COUNTER_HYSTERESIS_ON);
               printf("HYSTERESIS OFF = %lu\n",MSXE371X_COUNTER_HYSTERESIS_OFF);
               }
            else
               {
               printf("INCREMENT = %lu\n",MSXE371X_COUNTER_INCREMENT);
               printf("DECREMENT = %lu\n",MSXE371X_COUNTER_DECREMENT);
               }

			printf("Set the counter option : ");
			scanf("%lu", &ulCounterOption);
		}
		while (	(ulCounterOption != MSXE371X_COUNTER_HYSTERESIS_ON) && (ulCounterOption != MSXE371X_COUNTER_HYSTERESIS_OFF) &&
				(ulCounterOption != MSXE371X_COUNTER_INCREMENT) && (ulCounterOption != MSXE371X_COUNTER_DECREMENT) );

	
		i_Error = soap_call_MSXE371x__IncCounterInit (ps_SoapEnv, pc_SOAPIPAddress, SOAP_ACTION, 
									 		          ulCounterMode, 
												      ulCounterOption,
   												      0,
                                                      0,
                                                      0,
                                                      0,
                                                      &Response); 

		if ((i_Error == 0) && (Response.iReturnValue == 0))
		{			
			printf ("soap_call_MSXE371x__IncCounterInit OK\n");			
		} 
		else
		{
			printf("soap_call_MSXE371x__IncCounterInit SOAP error: %d, Response error: %d\n",i_Error,Response.iReturnValue);
		    if (i_Error != 0)
		        printf("SOAP Error : %s\n",*soap_faultstring(ps_SoapEnv));
			return 1;
		}

		printf ("\n");	

		/* You can specify a value at witch the counter will began to count */ 
		do
		{
			printf("Enter the value to write on the counter : ");
			scanf("%lu", &ulCounterValue);
		}
		while (ulCounterValue > 4294967295UL);
		
		printf ("\n");	
	
		i_Error = soap_call_MSXE371x__IncCounterWrite32BitValue (ps_SoapEnv, pc_SOAPIPAddress, SOAP_ACTION, ulCounterValue, &Response); 
		if ((i_Error == 0) && (Response.iReturnValue == 0))
		{			
			printf ("soap_call_MSXE371x__IncCounterWrite32BitValue OK\n");			
		} 
		else
		{
			printf("soap_call_MSXE371x__IncCounterWrite32BitValue SOAP error: %d, Response error: %d\n",i_Error,Response.iReturnValue);
			if (i_Error != 0)
				printf("SOAP Error : %s\n",*soap_faultstring(ps_SoapEnv));
			return 1;
		}



	
	return 0;
}


int i_SAMPLE_CounterCompareInitialization (struct soap *ps_SoapEnv, unsigned char *pc_SOAPIPAddress)
{
	struct MSXE371x__Response Response;
	int i_Error = 0;
	unsigned long  ulCompareValue = 0;
	unsigned long  ulCompareMode = 0;
	unsigned long  ulCompareSynchroTrigger = 0;

        printf("\n INCREMENTAL COUNTER COMPARE LOGIC INITIALIZATION :");
        printf("\n ------------------------------------------------\n\n");

        do
		{
			printf("Enter the compare value of the incremental counter : ");
			scanf("%lu", &ulCompareValue);
		}
		while ((ulCompareValue > 4294967295UL) || (ulCompareValue ==0));

		printf ("\n");

		do
		{
			printf("Compare mode selection\n\n");
			printf("0: condition true when counter equals compare value\n");
			printf("1: condition true when counter equals a multiple of the compare value\n");
			printf("Enter the compare mode of the incremental counter : ");
			scanf("%lu", &ulCompareMode);
		}
		while (ulCompareMode > 1);

		printf ("\n");

		do
		{
			printf("Compare with synchro trigger\n");
			printf("0 : no synchro trigger\n");
			printf("1 : generates a synchro trigger when condition is true\n");
			printf("Use the synchro trigger of the incremental counter : ");
			scanf("%lu", &ulCompareSynchroTrigger);
		}
		while (ulCompareSynchroTrigger > 1);

		printf ("\n");

		i_Error = soap_call_MSXE371x__IncCounterInitAndEnableCompareLogic(ps_SoapEnv, pc_SOAPIPAddress, SOAP_ACTION,  ulCompareValue, ulCompareMode, ulCompareSynchroTrigger, 0, 0, &Response);
		if ((i_Error == 0) && (Response.iReturnValue == 0))
		{			
			printf ("soap_call_MSXE371x__IncCounterInitAndEnableCompareLogic OK\n");			
		} 
		else
		{
			printf("soap_call_MSXE371x__IncCounterInitAndEnableCompareLogic SOAP error: %d, Response error: %d\n",i_Error,Response.iReturnValue);
			if (i_Error != 0)
				printf("SOAP Error : %s\n",*soap_faultstring(ps_SoapEnv));
		}	
	
	return 0;
}


int i_SAMPLE_ReleaseCounterCompare (struct soap *ps_SoapEnv, unsigned char *pc_SOAPIPAddress)
{
	struct MSXE371x__Response Response;
	int i_Error = 0;

		i_Error = soap_call_MSXE371x__IncCounterDisableAndReleaseCompareLogic(ps_SoapEnv, pc_SOAPIPAddress, SOAP_ACTION, NULL, &Response);
		if ((i_Error == 0) && (Response.iReturnValue == 0))
		{			
			printf ("soap_call_MSXE371x__IncCounterDisableAndReleaseCompareLogic OK\n");			
		} 
		else
		{
			printf("soap_call_MSXE371x__IncCounterDisableAndReleaseCompareLogic SOAP error: %d, Response error: %d\n", i_Error, Response.iReturnValue);
			if (i_Error != 0)
				printf("SOAP Error : %s\n",*soap_faultstring(ps_SoapEnv));
		}	

	
	return 0;
}

//-------------------------------------------------------------------------------------------------------

/** Release counter.
 * 
 * @retval 0:	No error.
 * @retval 1:	Error.
 */
int i_SAMPLE_ReleaseCounter (struct soap *ps_SoapEnv, unsigned char *pc_SOAPIPAddress)
{
	struct MSXE371x__Response Response;
	int i_Error = 0;

		i_Error = soap_call_MSXE371x__IncCounterRelease (ps_SoapEnv, pc_SOAPIPAddress, SOAP_ACTION, NULL,&Response); 
		if ((i_Error == 0) && (Response.iReturnValue == 0))
		{			
			printf ("soap_call_MSXE371x__IncCounterRelease OK\n");			
		} 
		else
		{
			printf("soap_call_MSXE371x__IncCounterRelease SOAP error: %d, Response error: %d\n",i_Error,Response.iReturnValue);
		    if (i_Error != 0)
		        printf("SOAP Error : %s\n",*soap_faultstring(ps_SoapEnv));
			return 1;
		}
	
	return 0;
}


/*******************************************************************************************************/
/*                                       Get auto refresh parameter function                           */               
/*******************************************************************************************************/
void GetAutoRefreshParameters  (DWORD *pdw_ChannelMask, 
			                    DWORD *pdw_AverageMode, 
			                    DWORD *pdw_AverageValue, 
			                    DWORD *pdw_TriggerMask, 
			                    DWORD *pdw_TriggerMode, 
			                    DWORD *pdw_HardwareTriggerEdge, 
			                    DWORD *pdw_HardwareTriggerCount, 
			                    DWORD *pdw_ByTriggerNbrOfSeqToAcquire,
			                    DWORD *pdw_DataFormat)
{
	DWORD dw_Choice = 0;
    DWORD dw_Cpt = 0;
    DWORD dw_Temp = 0;

	/* CHANNEL MASK SELECTION */
    do
	{
		printf("\nEnter the channel mask (in Hex): ");
		scanf("%X", pdw_ChannelMask);
	}
	while ((*pdw_ChannelMask == 0) || (*pdw_ChannelMask > (pow(2,NBR_OF_CHANNELS) - 1)));



	/* AVERAGE MODE SELECTION */
    printf("\nDo You want to activate the average mode (0 or 1)? ");
	scanf("%d",&dw_Choice);

	if (dw_Choice == 1)
	{
		*pdw_AverageMode = 1;

		do
		{
			printf("\nEnter the average value (2 to 255): ");
			scanf("%d", pdw_AverageValue);
		}
		while ((*pdw_AverageValue < 2) || (*pdw_AverageValue > 255));
	}
	else
	{
		*pdw_AverageValue = 0;
		*pdw_AverageMode = 0;
	}

	
	/* TRIGGER SELECTION */
	*pdw_TriggerMask = 0;
	*pdw_TriggerMode = 0;
	*pdw_HardwareTriggerEdge = 0;
	*pdw_HardwareTriggerCount = 0;
	*pdw_ByTriggerNbrOfSeqToAcquire = 0;

    printf("\nDo You want to use the hardware trigger (0 or 1)? ");
	scanf("%d", &dw_Choice);

	if (dw_Choice == 1)
	{
        *pdw_TriggerMask = *pdw_TriggerMask | 1;

		do 
		{
			printf("\n1. rising edge\n");
			printf("2. falling edge\n");
			printf("3. both edges\n");
			printf("Select the active edge: ");
			scanf ("%d", &dw_Choice);
		}
		while ((dw_Choice < 1) || (dw_Choice > 3));

		*pdw_HardwareTriggerEdge = dw_Choice;

		do
		{
			printf("\nEnter the number of edges before the trigger action occure (1 to 65535): ");
			scanf("%d",pdw_HardwareTriggerCount);
		}
		while ((*pdw_HardwareTriggerCount == 0) || (*pdw_HardwareTriggerCount > 65535));


	}

	if (*pdw_TriggerMask == 0)
	{
	    printf("\nDo You want to use the Synchro trigger (0 or 1)? ");
		scanf("%d", &dw_Choice);

		if (dw_Choice == 1)
		{
			*pdw_TriggerMask = *pdw_TriggerMask | 2;

		}
	}


	if (*pdw_TriggerMask == 0)
	{
	    printf("\nDo You want to use the incremental counter compare event trigger (0 or 1)? ");
		scanf("%d", &dw_Choice);

		if (dw_Choice == 1)
		{
			*pdw_TriggerMask = *pdw_TriggerMask | 4;

		}
	}

    if (*pdw_TriggerMask != 0)
    {
		do
		{
			printf("\n1. One shot trigger\n");
			printf("2. Sequence trigger\n");
			printf("Select the trigger mode do You want to use: ");
			scanf("%d", pdw_TriggerMode);
		}
		while ((*pdw_TriggerMode != 1) && (*pdw_TriggerMode != 2));


    	if (*pdw_TriggerMode == 2)
        {
            do
	    	{
		    	printf("\nEnter the number of sequence to acquire by each trigger: ");
			    scanf("%d", pdw_ByTriggerNbrOfSeqToAcquire);
		    }
		    while (*pdw_ByTriggerNbrOfSeqToAcquire == 0);
        }


    }

	
    /* DATA FORMAT SELECTION */
    *pdw_DataFormat = 0;
	do 
	{
	    printf("\nDo You want to receive a time stamp (0: No, 1: Yes) ?");
	    scanf ("%d",&dw_Choice);
	}
	while (dw_Choice > 1);

	*pdw_DataFormat = dw_Choice;

	do 
	{
	    printf("\nDo You want to receive the value in digital form (tape 0) or in analog form mm (type 1) ?");
	    scanf ("%d",&dw_Choice);
	}
	while (dw_Choice > 1);

	*pdw_DataFormat = *pdw_DataFormat | (dw_Choice << 1);
	
	do 
	{
	    printf("\nDo You want to invert the value sign (0: No, 1: Yes) ?");
	    scanf ("%d",&dw_Choice);
	}
	while (dw_Choice > 1);

	*pdw_DataFormat = *pdw_DataFormat | (dw_Choice << 2);


	do 
	{
	    printf("\nDo You want to receive the external temperature value (0: No, 1: Yes) ?");
	    scanf ("%d",&dw_Choice);
	}
	while (dw_Choice > 1);

	*pdw_DataFormat = *pdw_DataFormat | (dw_Choice << 3);
	

	do 
	{
	    printf("\nDo You want to receive the incremental counter value (0: No, 1: Yes) ?");
	    scanf ("%d",&dw_Choice);
	}
	while (dw_Choice > 1);

	*pdw_DataFormat = *pdw_DataFormat | (dw_Choice << 4);


}



typedef struct
      {
      DWORD   dw_DataFormat;
      DWORD   dw_ExtTemperatureFormat;
      DWORD   dw_NumberOfChannelsToAcquire;
      BYTE    b_Exit;
      SOCKET  s_ClientSocket;
      }str_ThreadInfo;



DWORD WINAPI dw_ThreadProc (LPVOID lpParameter)
        {
        str_ThreadInfo * ps_ThreadInfo;
		INT i_ReturnValue             = 0;
    	DWORD dw_Cpt                  = 0;
    	DWORD dw_RequestValue         = 0;
    	DWORD dw_NbrOfReceiveValues   = 0;
    	DWORD dw_NbrOfValuesToReceive = 0;
        DWORD *pdw_Data = NULL;
        float *pf_Data  = NULL;
		BYTE    b_SocketData [13 * 4];

        
        /**************************
        * Variable initialisation 
        ***************************/
        ps_ThreadInfo = (str_ThreadInfo *) lpParameter;
        
        do
        {
            dw_NbrOfValuesToReceive = (ps_ThreadInfo->dw_NumberOfChannelsToAcquire + 1) * sizeof (DWORD);

		    /* Test if the data format time stamp is activated */
			if (ps_ThreadInfo->dw_DataFormat & 1)
			{
                dw_NbrOfValuesToReceive = dw_NbrOfValuesToReceive + (2 * sizeof (DWORD));
			}

		    /* Test if external temperature used */
			if (ps_ThreadInfo->dw_DataFormat & 8)
			{
                dw_NbrOfValuesToReceive = dw_NbrOfValuesToReceive + (1 * sizeof (DWORD));
			}

		    /* Test incremental counter used */
			if (ps_ThreadInfo->dw_DataFormat & 16)
			{
                dw_NbrOfValuesToReceive = dw_NbrOfValuesToReceive + (1 * sizeof (DWORD));
			}
				
			dw_RequestValue       = dw_NbrOfValuesToReceive;
			dw_NbrOfReceiveValues = 0;

			do
			{
				i_ReturnValue = recv (ps_ThreadInfo->s_ClientSocket, &b_SocketData[dw_NbrOfReceiveValues], dw_RequestValue, 0); 

				if (i_ReturnValue != SOCKET_ERROR)
				{
					dw_NbrOfReceiveValues = dw_NbrOfReceiveValues + i_ReturnValue;
					dw_RequestValue       = dw_RequestValue - i_ReturnValue;
				}
			}
			while ((dw_NbrOfReceiveValues < dw_NbrOfValuesToReceive) && (i_ReturnValue != SOCKET_ERROR));

			/* Set the data pointer */
			pdw_Data = (DWORD *) b_SocketData;
            pf_Data  = (float *) b_SocketData;

			if (i_ReturnValue != SOCKET_ERROR)
			{

                    printf("\n\n");

                    /* Test if the data format time stamp is activated */
			        if (ps_ThreadInfo->dw_DataFormat & 1)
				    {
                        DisplayTimeStamp (pdw_Data[0], pdw_Data[1]);
						printf (" ");
                        pdw_Data = pdw_Data + 2;
                        pf_Data  = pf_Data  + 2;
                    }

					/* Show the auto refresh counter value */

                    printf("Counter: %lu\n", *pdw_Data);
                    pdw_Data = pdw_Data + 1;
                    pf_Data  = pf_Data  + 1;


			        /* Test if the data format external temperature is activated */
                    if ((ps_ThreadInfo->dw_DataFormat & 0x8) == 8)
				    {
                        if(ps_ThreadInfo->dw_ExtTemperatureFormat == 0)
                        {
                            printf("Ext Temp    : %lumOhms\n",*pdw_Data);
                        }

                        if (ps_ThreadInfo->dw_ExtTemperatureFormat == 1)
                        {
                            printf("Ext Temp    : %f degree celsius\n",*pf_Data);
                        }

                        if (ps_ThreadInfo->dw_ExtTemperatureFormat == 2)
                        {
                            printf("Ext Temp    : %f degree fahrenheit\n", *pf_Data);
                        }

                        pdw_Data = pdw_Data + 1;
                        pf_Data  = pf_Data  + 1;
                    }


			        /* Test if the data format Incremental counter is activated */
                    if ((ps_ThreadInfo->dw_DataFormat & 16) == 16)
				    {
                        printf("Inc Counter : %lu\n", *pdw_Data);
                        pdw_Data = pdw_Data + 1;
                        pf_Data  = pf_Data  + 1;
                    }

                    /* Display the transducer data */

                    printf("Transducers : ");
				    /* Test if receive analog value */
				    if (ps_ThreadInfo->dw_DataFormat & 2)
				    {
					    for (dw_Cpt = 0; dw_Cpt < ps_ThreadInfo->dw_NumberOfChannelsToAcquire; dw_Cpt ++)
					    { 
						    printf("%lu:%fmm ",dw_Cpt,*pf_Data);
							pf_Data  = pf_Data  + 1;
					    }
				    }
				    else
				    {
					    for (dw_Cpt = 0; dw_Cpt < ps_ThreadInfo->dw_NumberOfChannelsToAcquire; dw_Cpt ++)
					    { 
						    printf("%lu:%lX ",dw_Cpt,*pdw_Data);
							pdw_Data = pdw_Data + 1;
					    }
				    }



			}
			else
			{
				printf("Socket Error\n");
        		ps_ThreadInfo->b_Exit = 1;
            }

        } 
        while (ps_ThreadInfo->b_Exit == 0);

        /*********************
        * Return from thread 
        **********************/
        return (0);

        }




void main(void)
{
	INT i_Error = 0;
	struct soap *ps_SoapEnv = NULL;
	struct MXCommon__ByteArrayResponse  s_ByteArrayResponse;
	CHAR pc_IPAddressTemp[50];
	CHAR pc_IPAddress[50];
	CHAR pc_SOAPIPAddress[50];
	SOCKET  s_ClientSocket;
	DWORD dw_Dummy = 0;
    unsigned long dw_TransducerSelection = 0;
	unsigned long dw_ChannelMask = 0;
	unsigned long dw_AverageMode = 0;
	unsigned long dw_AverageValue = 0;
	unsigned long dw_TriggerMask = 0; 
	unsigned long dw_TriggerMode = 0; 
	unsigned long dw_HardwareTriggerEdge = 0;
	unsigned long dw_HardwareTriggerCount = 0;
	unsigned long dw_ByTriggerNbrOfSeqToAcquire = 0;
	unsigned long dw_DataFormat = 0;
	unsigned long dw_OptionParameter1 = 0;
	unsigned long dw_OptionParameter2 = 0;
	unsigned long dw_OptionParameter3 = 0;
	unsigned long dw_OptionParameter4 = 0;
    unsigned long dw_TemperatureFormat = 0;
	struct MSXE371x__Response Response;
	DWORD dw_NumberOfChannelsToAcquire = 0;
    DWORD dw_Cpt = 0;
    INT i_key = 0;
    DWORD    dw_ThreadId               = 0;
    HANDLE   h_ThreadHandle            = 0;
    str_ThreadInfo   s_ThreadInfo;


	/**************************/
	/* Get the console handle */
	/**************************/
	h_ConsoleHandle = GetStdHandle(STD_OUTPUT_HANDLE);
	h_ConsoleMutex  = CreateMutex (NULL, FALSE, "MSX-E371x AutoRefresh"); 
	ShowWindow(GetForegroundWindow (), SW_SHOWMAXIMIZED);

	/****************************/
	/* Print sample information */
	/****************************/
	printf("+---------------------------------------------------+\n");
	printf("| MSX-E371x                                         |\n");
	printf("| Transducer Auto Refresh Test Program              |\n");
	printf("+---------------------------------------------------+\n");
	printf("| <ESC> exit sample                                 |\n");
	printf("+---------------------------------------------------+\n");
	printf("\n");

	/* Initialise the SOAP connection */
	if (i_SOAP_Init(&ps_SoapEnv) == 0)
	{
		printf("Enter the module IP Address (XXX.XXX.XXX.XXX): ");
		gets(pc_IPAddressTemp);

		v_UTILS_ClearNullInIPAddress(pc_IPAddressTemp,pc_IPAddress);
		sprintf(pc_SOAPIPAddress,"%s:5555", pc_IPAddress);
		printf("\nSOAP IP Address: %s\n", pc_SOAPIPAddress);

		i_Error = soap_call_MXCommon__GetModuleType(ps_SoapEnv,pc_SOAPIPAddress , SOAP_ACTION, &dw_Dummy,&s_ByteArrayResponse);
		if ((i_Error == 0) && (s_ByteArrayResponse.sResponse.iReturnValue == 0))
		{
			printf("Module Type: %s\n", s_ByteArrayResponse.sArray.__ptr);
			
			/* Get Transducer type selection */
			if(i_DisplayAndSelectTransducerType(ps_SoapEnv, pc_SOAPIPAddress, &dw_TransducerSelection) == 0)
			{
                /* Get the auto refresh initialisation parameter */
			    GetAutoRefreshParameters(&dw_ChannelMask, 
						                 &dw_AverageMode, 
						                 &dw_AverageValue, 
                                         &dw_TriggerMask, 
                                         &dw_TriggerMode, 
                                         &dw_HardwareTriggerEdge, 
						                 &dw_HardwareTriggerCount, 
						                 &dw_ByTriggerNbrOfSeqToAcquire,
						                 &dw_DataFormat);
                
                // Test if want to receive the external temperature value
                if (dw_DataFormat & 8)
                {
                    i_Error = i_SAMPLE_ExtTemperatureInitialization(ps_SoapEnv,pc_SOAPIPAddress,&dw_TemperatureFormat);
                }

                // Test if want to receive the incremental counter value or use the compare event as trigger
                if ((i_Error == 0) && ((dw_DataFormat & 16) || (dw_TriggerMask & 4) ))
                {
                    i_Error = i_SAMPLE_CounterInitialization (ps_SoapEnv, pc_SOAPIPAddress);
                }

                if ((i_Error == 0) && (dw_TriggerMask & 4) )
                {
                    i_Error = i_SAMPLE_CounterCompareInitialization (ps_SoapEnv, pc_SOAPIPAddress);
                }
                
                if (i_Error == 0)
                {

			        /* Initialise the socket to receive the data */
			        if (i_Socket_ClientOpen(pc_IPAddress,8989,&s_ClientSocket) == 0)
			        {
                       /* Define the number of data to receive */
				        dw_NumberOfChannelsToAcquire = 0;

				        for (dw_Cpt = 0; dw_Cpt < NBR_OF_CHANNELS; dw_Cpt ++)
				        {
					        if (dw_ChannelMask & (1<<dw_Cpt))
						        dw_NumberOfChannelsToAcquire ++;
				        }

				        s_ThreadInfo.s_ClientSocket = s_ClientSocket;
				        s_ThreadInfo.dw_NumberOfChannelsToAcquire = dw_NumberOfChannelsToAcquire;
				        s_ThreadInfo.dw_DataFormat = dw_DataFormat;
                        s_ThreadInfo.dw_ExtTemperatureFormat = dw_TemperatureFormat;
				        s_ThreadInfo.b_Exit = 0;

				        printf("\n Press ESC to exit the sample...");
				        /*************************
				         * Create the data thread
				         *************************/
				        h_ThreadHandle = CreateThread (NULL,
					 	        		               0,
								   	                   dw_ThreadProc,
									                   &s_ThreadInfo,
									                   0,
									                   &dw_ThreadId);
       
				        Sleep (100);

				        /* Start the auto refresh */
				        i_Error = soap_call_MSXE371x__TransducerInitAndStartAutoRefresh (ps_SoapEnv,
                                                                                         pc_SOAPIPAddress , 
                                                                                         SOAP_ACTION,
                                                                                         dw_TransducerSelection,
                                                                                         dw_ChannelMask, 
                                                                                         dw_AverageMode,
                                                                                         dw_AverageValue, 
                                                                                         dw_TriggerMask, 
                                                                                         dw_TriggerMode, 
                                                                                         dw_HardwareTriggerEdge, 
                                                                                         dw_HardwareTriggerCount, 
                                                                                         dw_ByTriggerNbrOfSeqToAcquire,
                                                                                         dw_DataFormat, 
                                                                                         dw_OptionParameter1, 
                                                                                         dw_OptionParameter2, 
                                                                                         dw_OptionParameter3, 
                                                                                         dw_OptionParameter4, 
                                                                                         &Response);

				        if ((i_Error == 0) && (Response.iReturnValue == 0))
				        {
					        
							/* 
							 * gSOAP doesn't release automatically the memory that 
							 * it allocates to deserialize data, that's why both 
							 * following functions have to be used to release the 
							 * allocated memory.
							 */
							soap_destroy(ps_SoapEnv);
							soap_end(ps_SoapEnv);

                            do
                            {
					            if (kbhit()) 
					            {
						            i_key = getch();
                                }
                            }
                            while ((i_key != 27) && (!s_ThreadInfo.b_Exit));

                    
				            s_ThreadInfo.b_Exit = 1;


				            Sleep (100);
				        
		                    /******************
		                    * Free the thread 
		                    *******************/
		                    /***************************
		                    * Test if thread installed 
		                    ****************************/
		                    if (h_ThreadHandle != 0)
			                {
			                    /*************************
			                    * Close the thread handle 
			                    **************************/
			                    CloseHandle (h_ThreadHandle);
           
				            }
                    
                    
					        i_Error = soap_call_MSXE371x__TransducerStopAndReleaseAutoRefresh(ps_SoapEnv, pc_SOAPIPAddress, SOAP_ACTION, &dw_Dummy, &Response);
					        if ((i_Error != 0) || (Response.iReturnValue != 0))
                            {
					            printf ("soap_call_MSXE371x__TransducerStopAndReleaseAutoRefresh SOAP error: %d, Response error: %d\n",i_Error,Response.iReturnValue);
                                if (i_Error != 0)
                                    printf("SOAP Error : %s\n",*soap_faultstring(ps_SoapEnv));

                            }
				        }
				        else
				        {
					        printf("soap_call_MSXE371x__TransducerInitAndStartAutoRefresh SOAP error: %d, Response error: %d\n",i_Error,Response.iReturnValue);
                            if (i_Error != 0)
                                printf("SOAP Error : %s\n",*soap_faultstring(ps_SoapEnv));
				        }

				        /* Close the socket connection */
				        i_Socket_ClientClose (s_ClientSocket);
			        }
			        else
			        {
			            printf("i_Socket_ClientOpen Error\n");
			        }

                    if (dw_DataFormat & 8)
                    {
                        i_Error = i_SAMPLE_ReleaseExtTemperature(ps_SoapEnv,pc_SOAPIPAddress);
                    }

                    if (dw_TriggerMask & 4) 
                    {
                        i_Error = i_SAMPLE_ReleaseCounterCompare (ps_SoapEnv, pc_SOAPIPAddress);
                    }

                    if ((dw_DataFormat & 16) || (dw_TriggerMask & 4) )
                    {
                        i_Error = i_SAMPLE_ReleaseCounter (ps_SoapEnv, pc_SOAPIPAddress);
                    }



                }



            }

		}
		else
		{
			printf("soap_call_MXCommon__GetModuleType SOAP error: %d, Response error :%d\n", i_Error,s_ByteArrayResponse.sResponse.iReturnValue);
            if (i_Error != 0)
                 printf("SOAP Error : %s\n",*soap_faultstring(ps_SoapEnv));
		}

		/* Close the SOAP connection */
		i_SOAP_Close (ps_SoapEnv);
	}

   
	/**********************************/
	/*Print End programme informations*/
	/**********************************/
	printf("\n\n");
	printf("+---------------------------------------------------+\n");
	printf("| Program stopped                                   |\n");
	printf("+---------------------------------------------------+\n");
	printf("| Press any key to exit from sample                 |\n");
	printf("+---------------------------------------------------+\n");

	while (!(kbhit()));

	/****************************/
	/* Close the console handle */
	/****************************/
	if (h_ConsoleHandle != NULL)
	{
		/****************************/
		/* Close the console handle */
		/****************************/
		CloseHandle(h_ConsoleHandle);
		CloseHandle(h_ConsoleMutex);
	}
}
