 /** @file TransducerCalibration.c
 * Calibration : MSX-E371x : Calibration sample
 * 
 *
 * @author C. Guinot
 * @date 26.09.2007
 * 
 * @version  $LastChangedRevision$
 * @version $LastChangedDate$
 * 
 * 
 * @par LICENCE
 * @verbatim
 *  Copyright (C) 2004,2005  ADDI-DATA GmbH for the source code of this module.
 *        
 *        ADDI-DATA GmbH 
 *        Airport Boulevard B210
 *        D-77836 Rheinmnster Germany 
 *        Tel: +49(0)7229/1847-0 
 *        Fax: +49(0)7229/1847-200
 *        http://www.addi-data-com 
 *        info@addi-data.com 
 *        
 *
 * @endverbatim   
 *
 * @par UPDATES                                                                                                          
 * @verbatim                                                                                                             
 *                                                                                                                       
 *  Date      : 
 *  Author        : 
 *  Description   : 
 *
 * @endverbatim     
 */ 

#include <stdio.h>
#include <conio.h>
#include <stdlib.h>
#include <time.h>


#include "..\..\..\..\Interface_Library\MSXE371x.nsmap"
#include "..\UTIL\utility.h"
#include "..\UTIL\socket.h"


/********************************/
/* Console management functions */
/********************************/
int    i_X             = 0;
int    i_Y             = 0;
INT    i_ThreadStartY  = 0;
INT    i_ThreadCurentY = 0;

HANDLE h_ConsoleMutex  = NULL;



/*******************************************************************************************************/
/*                                       UTILS Functions                                               */               
/*******************************************************************************************************/


INT i_UTILS_GetModuleInformations (PCHAR pc_ModuleType, 
                                   DWORD * pdw_NbrOfChannels,
								   DWORD * pdw_Version)
{
	DWORD dw_Cpt = 0;
	CHAR  pc_CopyString [20];
	DWORD dw_CopyCounter = 0;

	/* check presence of the substring "UNKNOWN" indicating a not yet configured module */
	if (strstr(pc_ModuleType,"UNKNOWN"))
	{
		fprintf(stderr,"! Remote module not configured\n");
		return -1;
	}

	for (dw_Cpt = 0; dw_Cpt < strlen(pc_ModuleType);dw_Cpt++)
	{
		if (pc_ModuleType [dw_Cpt] == '-') 
		{
			dw_Cpt++;
			break;
		}
	}

	if (dw_Cpt < strlen(pc_ModuleType))
	{
		for (dw_CopyCounter = 0; dw_CopyCounter < (strlen(pc_ModuleType) - dw_Cpt ); dw_CopyCounter ++)
		{
			if (pc_ModuleType [dw_Cpt + dw_CopyCounter] == '-')
				break;
			else
				pc_CopyString [dw_CopyCounter] = pc_ModuleType [dw_Cpt + dw_CopyCounter];

		}
		pc_CopyString[dw_CopyCounter] = '\0';

		if (dw_CopyCounter < (strlen(pc_ModuleType) - dw_Cpt ))
		{
			*pdw_NbrOfChannels = atol (pc_CopyString);


			for (dw_Cpt = 0; dw_Cpt < strlen(pc_ModuleType);dw_Cpt++)
			{
				if (pc_ModuleType [dw_Cpt] == '_') 
				{
					dw_Cpt++;
					break;
				}
			}

			if (dw_Cpt < strlen(pc_ModuleType))
			{
				for (dw_CopyCounter = 0; dw_CopyCounter < (strlen(pc_ModuleType) - dw_Cpt ); dw_CopyCounter ++)
				{
					if (pc_ModuleType [dw_Cpt + dw_CopyCounter] == ' ')
						break;
					else
						pc_CopyString [dw_CopyCounter] = pc_ModuleType [dw_Cpt + dw_CopyCounter];

				}
				
				pc_CopyString[dw_CopyCounter] = '\0';

				if (dw_CopyCounter < (strlen(pc_ModuleType) - dw_Cpt ))
				{
					*pdw_Version = atol (pc_CopyString);
				}
				else
				{
					return -1;
				}
			}
			else
				return -1;

		}
		else
		{
			return -1;
		}
	}
	else
		return -1;
	
	return 0;
}


int i_DisplayAndSelectTransducerType(struct soap *ps_SoapEnv,
                                     PCHAR     pc_SOAPIPAddress,
									 unsigned long  * pul_TransducerTypeSelection, 
									 double *  pd_Range)
{
	struct MSXE371x__unsignedlongResponse Response;
	struct MSXE371x__TransducerGetTypeInformationResponse TypeInfoResponse;

	INT i_Error = 0;
	DWORD dw_Cpt = 0;
	DWORD dw_SelectionValueArray[100];
	DWORD dw_Choice = 0;
	DWORD dw_Dummy = 0;
	double d_RangeValueArray [100];
  
	/* Get the number of transducer type */
	i_Error = soap_call_MSXE371x__TransducerGetNbrOfType(ps_SoapEnv, pc_SOAPIPAddress, SOAP_ACTION, &dw_Dummy, &Response);
	if ((i_Error == 0) && (Response.sResponse.iReturnValue == 0))
	{
		printf("\nNumber of transducer types: %d\n", Response.ulValue);

		for (dw_Cpt = 0; dw_Cpt < Response.ulValue; dw_Cpt ++)
		{
			/* Get the transducer information */
			i_Error = soap_call_MSXE371x__TransducerGetTypeInformation (ps_SoapEnv,pc_SOAPIPAddress, SOAP_ACTION, dw_Cpt, &TypeInfoResponse);
			if ((i_Error == 0) && (TypeInfoResponse.sResponse.iReturnValue == 0))
			{
				printf("\n");
				printf("Transducer Index   : %d\n", dw_Cpt);
				printf("Selection Index    : %d\n", TypeInfoResponse.ulTransducerSelectionIndex);

				dw_SelectionValueArray[dw_Cpt] = TypeInfoResponse.ulTransducerSelectionIndex;
				d_RangeValueArray[dw_Cpt] = TypeInfoResponse.dRange;
				printf("Name               : %s\n", TypeInfoResponse.pcName);
				printf("Calibration status : %d\n", TypeInfoResponse.ulCalibrationStatus);
				printf("Type value         : %d\n", TypeInfoResponse.ulType);
				printf("Frequency          : %d Hz\n", TypeInfoResponse.ulFrequency);
				printf("Impedance          : %d Ohm\n", TypeInfoResponse.ulImpedance);
				printf("Nominal Voltage    : %lf V\n", TypeInfoResponse.dVeff);
				printf("Sensibility        : %lf mV/V/mm\n", TypeInfoResponse.dSensibility);
				printf("Range              : +/- %lf mm\n", TypeInfoResponse.dRange);
			}
			else
			{
				printf("soap_call_MSXE371x__Transducer_GetTypeInformation SOAP error: %d, Response error: %d\n",i_Error,TypeInfoResponse.sResponse.iReturnValue);
                if (i_Error != 0)
                    printf("SOAP Error : %s\n",*soap_faultstring(ps_SoapEnv));
				return -2;

			}
		}

		do 
		{
			printf("\nEnter the index of the transducer type you want ot use (0 to %d): ", (Response.ulValue - 1));
			scanf("%d",&dw_Choice);
		}
		while (dw_Choice >= Response.ulValue);
		*pul_TransducerTypeSelection = dw_SelectionValueArray[dw_Choice];
		*pd_Range = d_RangeValueArray[dw_Choice];
	}
	else
	{
		printf("soap_call_MSXE371x__Transducer_GetNbrOfType SOAP error: %d, Response error: %d\n",i_Error,Response.sResponse.iReturnValue);
        if (i_Error != 0)
            printf("SOAP Error : %s\n",*soap_faultstring(ps_SoapEnv));
		return -1;
	}
	return 0;
}


void	v_TransducerCalibration (struct soap *ps_SoapEnv,
                                 PCHAR     pc_SOAPIPAddress, 
								 unsigned long  ul_Channel, 
								 unsigned long  ul_TransducerTypeSelection, 
								 double     d_Range, 
								 double     d_CalibrationPosition)
{
	INT                                                i_Error         = 0;
	ULONG                                             ul_LastStatus    = 0;
	double                                             d_ChannelValue  = 0;
	struct MSXE371x__Response                            s_SingleResponse;
	struct MSXE371x__ByteArrayResponse                   s_SaveResponse;
	struct MSXE371x__CalibrationGetCurrentStatusResponse s_Response;
    unsigned long ul_CalibrationMode = 0;
    unsigned long ul_InvertValue = 0;


	do
	{
		printf("\n 0. Calibre the primary and all channels");
        printf("\n 1. Calibre only the selected channel");
    	printf("\nSet the calibration mode: ");
		scanf("%lu", &ul_CalibrationMode);
	}
	while (	ul_CalibrationMode > 2);

	do
	{
		printf("\n 0 : don't invert the channel value");
        printf("\n 1 : invert the channel value (-2 mm -> + 2mm)");

    	printf("\nInvert the value selection: ");
		scanf("%lu", &ul_InvertValue);
	}
	while (	ul_InvertValue > 2);

	/*************************/
	/* Start the calibration */
	/*************************/

	i_Error = soap_call_MSXE371x__CalibrationStart (ps_SoapEnv,
			                                      pc_SOAPIPAddress, 
				                                  SOAP_ACTION,
					                              ul_TransducerTypeSelection, // Transducer type selection
							                      d_CalibrationPosition,       // Calibration position
                                                  ul_CalibrationMode,
                                                  ul_InvertValue,
						                          ul_Channel,                  // Channel selection
									              &s_SingleResponse);

	/***********************/
	/* Test if error occur */
	/***********************/

	if ((i_Error != 0) || (s_SingleResponse.iReturnValue != 0))
	{
		if (i_Error != 0)
		{
			printf("SOAP connection error\nsoap_call_MSXE371x__CalibrationStart SOAP Error: %d\n", i_Error);
			if (i_Error != 0)
				printf("SOAP Error : %s\n",*soap_faultstring(ps_SoapEnv));
		}
		else
		{
			printf("soap_call_MSXE371x__CalibrationStart Error: %d\n", s_SingleResponse.iReturnValue);
		}
		return;
	}



	while (ul_LastStatus != 8)
	{
		Sleep (10);
           
		/********************************
		 * Get calibration current status
		 *********************************/
		i_Error = soap_call_MSXE371x__CalibrationGetCurrentStatus (ps_SoapEnv,
			                                                     pc_SOAPIPAddress,
				                                                 SOAP_ACTION,
					                                             NULL,
						                                         &s_Response);
           
		/***********************/
		/* Test if error occur */
		/***********************/

		if ((i_Error != 0) || (s_Response.sResponse.iReturnValue != 0))
		{
			if (i_Error != 0)
			{
				printf("\nSOAP connection error\nsoap_call_MSXE371x__CalibrationGetCurrentStatus SOAP Error: %d\n", i_Error);
				if (i_Error != 0)
					printf("SOAP Error : %s\n",*soap_faultstring(ps_SoapEnv));
			}
		else
			{
			printf("\nsoap_call_MSXE371x__CalibrationGetCurrentStatus Error: %d\n", s_Response.sResponse.iReturnValue);
			}

		return;
		}


		/************************************
		 * Test if start primary calibration 
		 *************************************/
		if (((s_Response.ulStatus == 1) || ((s_Response.ulStatus == 2))) && (ul_LastStatus == 0))
		{
			printf ("\nPrimary calibration started\n");

			if (s_Response.ulStatus == 2)
			{
				ul_LastStatus = 1;
			}
		}

		/**************************************
		 * Test if primary calibration started 
		 ***************************************/
		if ((s_Response.ulStatus == 1) && (ul_LastStatus == 1))
		{
			/**************
			 * Calculate V 
			 ***************/

			d_ChannelValue = (((2.8 * 2.0) / (double) 0xFFFFFF) * (double) s_Response.ulDigitalValue) - 2.8;
              
			printf ("\rPrimary calibration %7.3fV", d_ChannelValue);
		}

		/**********************************
		 * Test if end primary calibration 
		 ***********************************/
		if ((s_Response.ulStatus == 2) && (ul_LastStatus != 2))
		{
			printf ("\nPrimary calibration conclude");
			printf ("\n\nSet the transducer connected to channel %d to position 0 mm and press any key\n", s_Response.ulChannel);
		}

		/******************************************
		 * Test if wait user set transducer to 0mm 
		 *******************************************/
		if ((s_Response.ulStatus == 2) && (ul_LastStatus == 2))
		{
			/*********************************
			 * Calculate the position into mm 
			 **********************************/
			d_ChannelValue = (((d_Range * 2.0) / (double) 0xFFFFFF) * (double) s_Response.ulDigitalValue) - d_Range;
              
			printf ("\rTransducer position = %8.4fmm", d_ChannelValue);
		}

		/******************************************
		 * Test if start null position calibration 
		 *******************************************/
		if ((s_Response.ulStatus == 3) && (ul_LastStatus != 3))
		{
			printf ("\n\nNull position calibration started\n");
		}

		/************************************
		 * Test if null position calibration 
		 *************************************/
		if ((s_Response.ulStatus == 3) && (ul_LastStatus == 3))
		{
			/*********************************
			 * Calculate the position into mm 
			 **********************************/
			d_ChannelValue = (((d_Range * 2.0) / (double) 0xFFFFFF) * (double) s_Response.ulDigitalValue) - d_Range;
              
			printf ("\rTransducer position = %8.4fmm", d_ChannelValue);
		}

		/****************************************
		 * Test if end user position calibration 
		 *****************************************/
		if ((s_Response.ulStatus == 4) && (ul_LastStatus != 4))
		{
			printf ("\nNull position calibration conclude");
			printf ("\n\nSet the transducer connected to channel %d to position +%7.4f mm and press any key\n", s_Response.ulChannel, d_CalibrationPosition);
		}

		/****************************************************
		 * Test if wait user set transducer to user position 
		 *****************************************************/
		if ((s_Response.ulStatus == 4) && (ul_LastStatus == 4))
		{
			/*********************************
			 * Calculate the position into mm 
			 **********************************/
			d_ChannelValue = (((d_Range * 2.0) / (double) 0xFFFFFF) * (double) s_Response.ulDigitalValue) - d_Range;
              
			printf ("\rTransducer position = %8.4fmm", d_ChannelValue);
		}

		/******************************************
		 * Test if start user position calibration 
		 *******************************************/
		if ((s_Response.ulStatus == 5) && (ul_LastStatus != 5))
		{
			printf ("\n\nUser position calibration started\n");
		}

		/************************************
		 * Test if user position calibration 
		 *************************************/
		if ((s_Response.ulStatus == 5) && (ul_LastStatus == 5))
		{
			/*********************************
			 * Calculate the position into mm 
			 **********************************/
			d_ChannelValue = (((d_Range * 2.0) / (double) 0xFFFFFF) * (double) s_Response.ulDigitalValue) - d_Range;
              
			printf ("\rTransducer position = %8.4fmm", d_ChannelValue);
		}

		/****************************************
		 * Test if end user position calibration 
		 ****************************************/
		if ((s_Response.ulStatus == 6) && (ul_LastStatus != 6))
		{
			printf ("\nUser position calibration conclude");
			printf ("\n\nConnect only one transducer and press any key\n", s_Response.ulChannel);
		}


		/******************************************************
		 * Test if wait that user connect only one transducer 
		 ******************************************************/
		if ((s_Response.ulStatus == 7) && (ul_LastStatus != 7))
		{
			printf ("\n\nPrimary open line calibration started\n");
		}


		/*********************************************
		 * Test if primary open line test calibration
		 *********************************************/
		if ((s_Response.ulStatus == 7) && (ul_LastStatus == 7))
		{
			/**************
			 * Calculate V 
			 ***************/

			d_ChannelValue = (((2.8 * 2.0) / (double) 0xFFFFFF) * (double) s_Response.ulDigitalValue) - 2.8;
              
			printf ("\rPrimary open line calibration %7.3fV", d_ChannelValue);
		}

		/**************************
		 * Test if end calibration 
		 ***************************/
		if ((s_Response.ulStatus == 8) && (ul_LastStatus != 8))
		{
			/*******************
			 * Error management 
			 ********************/
			switch (s_Response.ulDigitalValue)
			{
				case 0:
						printf ("\n\nCalibration conclude");

						/**********************************
						 * Save transducer board data-base 
						 ***********************************/

						/***********************
						 * Save all transducers 
						 ************************/
                      
						i_Error = soap_call_MSXE371x__DataBaseSaveTransducers(ps_SoapEnv,
									                                        pc_SOAPIPAddress, 
										                                    SOAP_ACTION, 
											                                NULL,
                                                                            &s_SaveResponse);
                    
						if ((i_Error != 0) || (s_SaveResponse.sResponse.iReturnValue != 0))
						{
							if (i_Error != 0)
							{
								printf("SOAP connection error\nsoap_call_MSXE371x__DataBaseSaveTransducers SOAP Error: %d\n", i_Error);
								printf("SOAP Error : %s\n",*soap_faultstring(ps_SoapEnv));
							}
							else
							{
								printf("soap_call_MSXE371x__DataBaseSaveTransducers Error: %d\n", s_SaveResponse.sResponse.iReturnValue);
							}
						}
						else
						{
							printf ("\nTransducer data base saved");
						}
						break;

				case 1:
						/***********************
						 * Stop the calibration 
						 ************************/
						printf ("\n\n!!!User have break the calibration");
						break;

				case 2:
				case 3:
				case 4:
				case 5:
				case 6:
				case 7:
						printf ("\nCan not calibrate the primary feedback");
						break;

				case 8:
				case 9:
						printf ("\nThe 0mm position can not by found.\nRestart the calibration and select a new position for the 0mm."); 
						break;

				case 10:
				case 11:
				case 12:
				case 13:
						printf ("\nThe +%7.4fmm position can not by found.\nRestart the calibration and select a new position for the +%7.4fmm.", 
								d_CalibrationPosition,
								d_CalibrationPosition);
						break;

				case 14:
						printf ("\nPrimary short circuit occur\n");
						break;

				default :
						/****************************
						 * Format and show the error 
						 *****************************/
						printf  ("\nCalibration error %ld", s_Response.ulDigitalValue);
						break;
			}
		}

		/**************************
		 * Save the current status 
		 ***************************/
		ul_LastStatus = s_Response.ulStatus;

		/*********************/
		/* Test if key press */
		/*********************/
	   
		if (kbhit ())
		{
			/******************/
			/* Test if no ESC */
			/******************/

			if (getch () != 27)
			{
				/**********************************
				 * Test if calibration in progress 
				 ***********************************/

				if (ul_LastStatus != 8)
				{
					/*****************************
					 * Test if next step selected 
					 ******************************/

					if ((ul_LastStatus == 2) || (ul_LastStatus == 4) || (ul_LastStatus == 6))
					{
						/*****************************************
						 * Send the next calibration step command 
						 ******************************************/
                 
						soap_call_MSXE371x__CalibrationNextStep (ps_SoapEnv,
															   pc_SOAPIPAddress, 
															   SOAP_ACTION,
                                                               NULL,
                                                               &s_SingleResponse);
                 
					}
				}
			}
			else
			{
				/**********************************
				 * Test if calibration in progress 
				 ***********************************/

				if (ul_LastStatus != 8)
				{
					/***********************
					 * Stop the calibration 
					 ************************/
					soap_call_MSXE371x__CalibrationBreak (ps_SoapEnv,
														pc_SOAPIPAddress, 
														SOAP_ACTION,
                                                        NULL,
                                                        &s_SingleResponse);
                 
				}
			}
		}
		
		/* 
		 * gSOAP doesn't release automatically the memory that 
		 * it allocates to deserialize data, that's why both 
		 * following functions have to be used to release the 
		 * allocated memory.
		 */
		soap_destroy(ps_SoapEnv);
		soap_end(ps_SoapEnv);		
	}
	return;
}


void	main(void)
	{
	CHAR                                pc_IPAddressTemp[50];
	CHAR                                pc_IPAddress    [50];
	CHAR                                pc_SOAPIPAddress[50];
	BYTE                                 b_Exit                      = 0;
	INT                                  i_Error                     = 0;
	INT                                  i_ReturnValue               = 0;
	DWORD                               dw_Dummy                     = 0;
	DWORD                               dw_NbrOfChannels             = 8;
	DWORD                               dw_Cpt                       = 0;
	xsd__double                          d_Range                     = 0;
	xsd__unsignedLong                   dw_TransducerTypeSelection   = 0;
	struct soap                        *ps_SoapEnv                   = NULL;
	struct MXCommon__ByteArrayResponse   s_ByteArrayResponse;


	/**************************/
	/* Get the console handle */
	/**************************/
	ShowWindow(GetForegroundWindow (), SW_SHOWMAXIMIZED);

	/****************************/
	/* Print sample information */
	/****************************/
	printf("+---------------------------------------------------+\n");
	printf("| MSX-E371x                                         |\n");
	printf("| Transducer calibration Program                    |\n");
	printf("+---------------------------------------------------+\n");
	printf("| <ESC> exit sample                                 |\n");
	printf("+---------------------------------------------------+\n");
	printf("\n");

	/* Initialise the SOAP connection */
	if (i_SOAP_Init(&ps_SoapEnv) == 0)
	{
		printf("Enter the module IP Address (XXX.XXX.XXX.XXX): ");
		gets(pc_IPAddressTemp);

		v_UTILS_ClearNullInIPAddress(pc_IPAddressTemp,pc_IPAddress);
		sprintf(pc_SOAPIPAddress,"%s:5555", pc_IPAddress);
		printf("\nSOAP IP Address: %s\n", pc_SOAPIPAddress);

		i_Error = soap_call_MXCommon__GetModuleType(ps_SoapEnv,pc_SOAPIPAddress , SOAP_ACTION, &dw_Dummy,&s_ByteArrayResponse);
	   
		if ((i_Error == 0) && (s_ByteArrayResponse.sResponse.iReturnValue == 0))
		{
			printf("Module Type: %s\n", s_ByteArrayResponse.sArray.__ptr);
			
			printf("\t=> This is a %d transducers module\n",dw_NbrOfChannels);

			/* Get Transducer type selection */
			if(i_DisplayAndSelectTransducerType(ps_SoapEnv, pc_SOAPIPAddress, (DWORD *)&dw_TransducerTypeSelection, &d_Range) == 0)
			{
				v_TransducerCalibration (ps_SoapEnv, pc_SOAPIPAddress, 0, dw_TransducerTypeSelection, d_Range, d_Range / 2);
			}
		}
		else
		{
			printf("soap_call_MXCommon__GetModuleType SOAP error: %d, Response error: %d\n", i_Error,s_ByteArrayResponse.sResponse.iReturnValue);
			if (i_Error != 0)
				printf("SOAP Error : %s\n",*soap_faultstring(ps_SoapEnv));
		}

		/* Close the SOAP connection */
		i_SOAP_Close (ps_SoapEnv);
	}

   
	/**********************************/
	/*Print End programme informations*/
	/**********************************/
	printf("\n\n");
	printf("+---------------------------------------------------+\n");
	printf("| Program stopped                                   |\n");
	printf("+---------------------------------------------------+\n");
	printf("| Press any key to exit from sample                 |\n");
	printf("+---------------------------------------------------+\n");

	while (!(kbhit()));
	}
