 /** @file TransducerSequence.c
 * Sequence : MSXE-371x : Transducer sequence acquisition
 * 
 *
 * @author C. Guinot
 * @date 27.09.2007
 * 
 * @version  $LastChangedRevision$
 * @version $LastChangedDate$
 * 
 * 
 * @par LICENCE
 * @verbatim
 *  Copyright (C) 2004,2005  ADDI-DATA GmbH for the source code of this module.
 *        
 *        ADDI-DATA GmbH 
 *        Airport Boulevard B210
 *        D-77836 Rheinmnster Germany 
 *        Tel: +49(0)7229/1847-0 
 *        Fax: +49(0)7229/1847-200
 *        http://www.addi-data-com 
 *        info@addi-data.com 
 *        
 *
 * @endverbatim   
 *
 * @par UPDATES                                                                                                          
 * @verbatim                                                                                                             
 *                                                                                                                       
 *  Date      : 
 *  Author        : 
 *  Description   : 
 *
 * @endverbatim     
 */ 

/* Color management */
#include <stdio.h>
#include <conio.h>
#include <stdlib.h>
#include <io.h>
#include <math.h>
#include <fcntl.h>
#include <sys/types.h>
#include <sys/stat.h>

#include "..\..\..\..\Interface_Library\MSXE371x.nsmap"
#include "..\UTIL\utility.h"
#include "..\UTIL\socket.h"


int i_DisplayAndSelectTransducerType(struct soap *ps_SoapEnv,PCHAR pc_SOAPIPAddress,DWORD * pdw_TransducerTypeSelection)
{
	struct MSXE371x__unsignedlongResponse s_Response;
	struct MSXE371x__TransducerGetTypeInformationResponse TypeInfoResponse;

	INT i_Error = 0;
	DWORD dw_Cpt = 0;
	DWORD pdw_SelectionValueArray[100];
	DWORD dw_Choice = 0;
	DWORD dw_Dummy = 0;
  
	/* Get the number of transducer type */
	i_Error = soap_call_MSXE371x__TransducerGetNbrOfType(ps_SoapEnv, pc_SOAPIPAddress, SOAP_ACTION, &dw_Dummy, &s_Response);
	if ((i_Error == 0) && (s_Response.sResponse.iReturnValue == 0))
	{
		printf("\nNumber of transducer types: %d\n", s_Response.ulValue);

		for (dw_Cpt = 0; dw_Cpt < s_Response.ulValue; dw_Cpt ++)
		{
			/* Get the transducer information */
			i_Error = soap_call_MSXE371x__TransducerGetTypeInformation (ps_SoapEnv,pc_SOAPIPAddress, SOAP_ACTION, dw_Cpt, &TypeInfoResponse);
			if ((i_Error == 0) && (TypeInfoResponse.sResponse.iReturnValue == 0))
			{
				printf("\n");
				printf("Transducer Index: %d\n", dw_Cpt);
				printf("Selection Index: %d\n", TypeInfoResponse.ulTransducerSelectionIndex);

				pdw_SelectionValueArray[dw_Cpt] = TypeInfoResponse.ulTransducerSelectionIndex;
				printf("Name: %s\n", TypeInfoResponse.pcName);
				printf("Calibration status: %d\n", TypeInfoResponse.ulCalibrationStatus);
				printf("Type value: %d\n", TypeInfoResponse.ulType);
				printf("Frequency: %d Hz\n", TypeInfoResponse.ulFrequency);
				printf("Impedance: %d Ohm\n", TypeInfoResponse.ulImpedance);
				printf("Nominal Voltage: %lf V\n", TypeInfoResponse.dVeff);
				printf("Sensibility: %lf mV/V/mm\n", TypeInfoResponse.dSensibility);
				printf("Range: +/- %lf mm\n", TypeInfoResponse.dRange);
			}
			else
			{
				printf("soap_call_MSXE371x__Transducer_GetTypeInformation Error\n");
				return -2;
			}
		}

		do 
		{
			printf("\nEnter the index of the transducer type you want ot use (0 to %d): ", (s_Response.ulValue - 1));
			scanf("%d",&dw_Choice);
		}
		while (dw_Choice >= s_Response.ulValue);
		*pdw_TransducerTypeSelection = pdw_SelectionValueArray[dw_Choice];
	}
	else
	{
		printf("soap_call_MSXE371x__Transducer_GetNbrOfType Error\n");
		return -1;
	}
	return 0;
}



//-------------------------------------------------------------------------------------------------------
//-------------------------------------------------------------------------------------------------------

/** Set temperature configuration.
 * 
 * @retval 0:	No error.
 * @retval 1:	Error.
 */
int i_SAMPLE_ExtTemperatureInitialization (struct soap *ps_SoapEnv, unsigned char *pc_SOAPIPAddress, unsigned long *pul_TempFormat)
{
	struct MSXE371x__Response Response;
	int i_Error = 0;
    unsigned long ulSensorType [3] = {100,500,1000};
    unsigned long ulSensorSelection = 0;
	unsigned long ulConvertMode = 0;
    unsigned long ulGainSelection = 0;
    unsigned long ulFrequencySelection = 0;
    unsigned long ulPowerSaveMode = 0;


        printf("\n EXTERNAL TEMPERATURE INITIALIZATION");
        printf("\n -----------------------------------\n");

		do
		{
			printf("\n 0. PT100");
			printf("\n 1. PT500");
			printf("\n 2. PT1000");
			printf("\nSelect the temperature sensor : ");
			scanf("%lu", &ulSensorSelection);
		}
		while (	ulSensorSelection > 2);


		do
		{
			printf("\n 0. Return the value in mOhm form");
            printf("\n 1. degree Celsius");
            printf("\n 2. degree Fahrenheit");
    		printf("\nSet the convert mode: ");
			scanf("%lu", &ulConvertMode);
		}
		while (	ulConvertMode > 2);

        *pul_TempFormat = ulConvertMode;


		do
		{
    		printf("\nEnter the acquisition frequency to use (10, 240): ");
			scanf("%lu", &ulFrequencySelection);
		}
		while (	(ulFrequencySelection != 10) && 
                (ulFrequencySelection != 240) );

	
   		do
		{
            printf("\n 0 : Disable power save mode");
            printf("\n 1 : Enable power save mode. In this case the value can only be read if any");
			printf("\n     transducer acquisition is enabled");
    		printf("\nPower save selection :");
			scanf("%lu", &ulPowerSaveMode);
		}
		while (ulPowerSaveMode > 1);

		printf ("\n");	

		i_Error = soap_call_MSXE371x__ExternalTemperatureInit (ps_SoapEnv, pc_SOAPIPAddress, SOAP_ACTION, 
									 		                   ulSensorType[ulSensorSelection], 
												               ulConvertMode,
                                                               ulGainSelection,
                                                               ulFrequencySelection,
                                                               ulPowerSaveMode,
   												               0,
                                                               0,
                                                               0,
                                                               0,
                                                               &Response); 

		if ((i_Error == 0) && (Response.iReturnValue == 0))
		{			
			printf ("soap_call_MSXE371x__ExternalTemperatureInit OK\n");			
		} 
		else
		{
			printf("soap_call_MSXE371x__ExternalTemperatureInit SOAP error: %d, Response error: %d\n",i_Error,Response.iReturnValue);
		    if (i_Error != 0)
		        printf("SOAP Error : %s\n",*soap_faultstring(ps_SoapEnv));
			return 1;
		}

	
	return 0;
}

//-------------------------------------------------------------------------------------------------------

/** Release temperature.
 * 
 * @retval 0:	No error.
 * @retval 1:	Error.
 */
int i_SAMPLE_ReleaseExtTemperature (struct soap *ps_SoapEnv, unsigned char *pc_SOAPIPAddress)
{
	struct MSXE371x__Response Response;
	int i_Error = 0;

		i_Error = soap_call_MSXE371x__ExternalTemperatureRelease (ps_SoapEnv, pc_SOAPIPAddress, SOAP_ACTION, NULL,&Response); 
		if ((i_Error == 0) && (Response.iReturnValue == 0))
		{			
			printf ("soap_call_MSXE371x__ExternalTemperatureRelease OK\n");			
		} 
		else
		{
			printf("soap_call_MSXE371x__ExternalTemperatureRelease SOAP error: %d, Response error: %d\n",i_Error,Response.iReturnValue);
		    if (i_Error != 0)
		        printf("SOAP Error : %s\n",*soap_faultstring(ps_SoapEnv));
			return 1;
		}
	
	return 0;
}



/** Set counter configuration.
 * 
 * @retval 0:	No error.
 * @retval 1:	Error.
 */
int i_SAMPLE_CounterInitialization (struct soap *ps_SoapEnv, unsigned char *pc_SOAPIPAddress)
{
	struct MSXE371x__Response Response;
	int i_Error = 0;
	unsigned long  ulCounterMode = 0;
	unsigned long  ulCounterValue = 0;
	unsigned long  ulCounterOption = 0;

        printf("\n INCREMENTAL COUNTER INITIALIZATION :");
        printf("\n ----------------------------------\n\n");

		do
		{
			printf("SIMPLE MODE     = %lu\n", MSXE371X_COUNTER_SIMPLE_MODE);
			printf("DOUBLE MODE     = %lu\n", MSXE371X_COUNTER_DOUBLE_MODE);
			printf("QUADRUPLE MODE  = %lu\n", MSXE371X_COUNTER_QUADRUPLE_MODE);
			printf("DIRECT MODE     = %lu\n", MSXE371X_COUNTER_DIRECT_MODE);
			printf("Set the counter mode : ");
			scanf("%lu", &ulCounterMode);
		}
		while (	(ulCounterMode != MSXE371X_COUNTER_QUADRUPLE_MODE) && (ulCounterMode != MSXE371X_COUNTER_DOUBLE_MODE) &&
				(ulCounterMode != MSXE371X_COUNTER_SIMPLE_MODE) && (ulCounterMode != MSXE371X_COUNTER_DIRECT_MODE) );

		printf ("\n");

		do
		{
			if (ulCounterMode != MSXE371X_COUNTER_DIRECT_MODE)
               {
               printf("HYSTERESIS ON  = %lu\n",MSXE371X_COUNTER_HYSTERESIS_ON);
               printf("HYSTERESIS OFF = %lu\n",MSXE371X_COUNTER_HYSTERESIS_OFF);
               }
            else
               {
               printf("INCREMENT = %lu\n",MSXE371X_COUNTER_INCREMENT);
               printf("DECREMENT = %lu\n",MSXE371X_COUNTER_DECREMENT);
               }

			printf("Set the counter option : ");
			scanf("%lu", &ulCounterOption);
		}
		while (	(ulCounterOption != MSXE371X_COUNTER_HYSTERESIS_ON) && (ulCounterOption != MSXE371X_COUNTER_HYSTERESIS_OFF) &&
				(ulCounterOption != MSXE371X_COUNTER_INCREMENT) && (ulCounterOption != MSXE371X_COUNTER_DECREMENT) );

	
		i_Error = soap_call_MSXE371x__IncCounterInit (ps_SoapEnv, pc_SOAPIPAddress, SOAP_ACTION, 
									 		          ulCounterMode, 
												      ulCounterOption,
   												      0,
                                                      0,
                                                      0,
                                                      0,
                                                      &Response); 

		if ((i_Error == 0) && (Response.iReturnValue == 0))
		{			
			printf ("soap_call_MSXE371x__IncCounterInit OK\n");			
		} 
		else
		{
			printf("soap_call_MSXE371x__IncCounterInit SOAP error: %d, Response error: %d\n",i_Error,Response.iReturnValue);
		    if (i_Error != 0)
		        printf("SOAP Error : %s\n",*soap_faultstring(ps_SoapEnv));
			return 1;
		}

		printf ("\n");	

		/* You can specify a value at witch the counter will began to count */ 
		do
		{
			printf("Enter the value to write on the counter : ");
			scanf("%lu", &ulCounterValue);
		}
		while (ulCounterValue > 4294967295UL);
		
		printf ("\n");	
	
		i_Error = soap_call_MSXE371x__IncCounterWrite32BitValue (ps_SoapEnv, pc_SOAPIPAddress, SOAP_ACTION, ulCounterValue, &Response); 
		if ((i_Error == 0) && (Response.iReturnValue == 0))
		{			
			printf ("soap_call_MSXE371x__IncCounterWrite32BitValue OK\n");			
		} 
		else
		{
			printf("soap_call_MSXE371x__IncCounterWrite32BitValue SOAP error: %d, Response error: %d\n",i_Error,Response.iReturnValue);
			if (i_Error != 0)
				printf("SOAP Error : %s\n",*soap_faultstring(ps_SoapEnv));
			return 1;
		}



	
	return 0;
}


int i_SAMPLE_CounterCompareInitialization (struct soap *ps_SoapEnv, unsigned char *pc_SOAPIPAddress)
{
	struct MSXE371x__Response Response;
	int i_Error = 0;
	unsigned long  ulCompareValue = 0;
	unsigned long  ulCompareMode = 0;
	unsigned long  ulCompareSynchroTrigger = 0;

        printf("\n INCREMENTAL COUNTER COMPARE LOGIC INITIALIZATION :");
        printf("\n ------------------------------------------------\n\n");

        do
		{
			printf("Enter the compare value of the incremental counter : ");
			scanf("%lu", &ulCompareValue);
		}
		while ((ulCompareValue > 4294967295UL) || (ulCompareValue ==0));

		printf ("\n");

		do
		{
			printf("Compare mode selection\n\n");
			printf("0: condition true when counter equals compare value\n");
			printf("1: condition true when counter equals a multiple of the compare value\n");
			printf("Enter the compare mode of the incremental counter : ");
			scanf("%lu", &ulCompareMode);
		}
		while (ulCompareMode > 1);

		printf ("\n");

		do
		{
			printf("Compare with synchro trigger\n");
			printf("0 : no synchro trigger\n");
			printf("1 : generates a synchro trigger when condition is true\n");
			printf("Use the synchro trigger of the incremental counter : ");
			scanf("%lu", &ulCompareSynchroTrigger);
		}
		while (ulCompareSynchroTrigger > 1);

		printf ("\n");

		i_Error = soap_call_MSXE371x__IncCounterInitAndEnableCompareLogic(ps_SoapEnv, pc_SOAPIPAddress, SOAP_ACTION,  ulCompareValue, ulCompareMode, ulCompareSynchroTrigger, 0, 0, &Response);
		if ((i_Error == 0) && (Response.iReturnValue == 0))
		{			
			printf ("soap_call_MSXE371x__IncCounterInitAndEnableCompareLogic OK\n");			
		} 
		else
		{
			printf("soap_call_MSXE371x__IncCounterInitAndEnableCompareLogic SOAP error: %d, Response error: %d\n",i_Error,Response.iReturnValue);
			if (i_Error != 0)
				printf("SOAP Error : %s\n",*soap_faultstring(ps_SoapEnv));
		}	
	
	return 0;
}


int i_SAMPLE_ReleaseCounterCompare (struct soap *ps_SoapEnv, unsigned char *pc_SOAPIPAddress)
{
	struct MSXE371x__Response Response;
	int i_Error = 0;

		i_Error = soap_call_MSXE371x__IncCounterDisableAndReleaseCompareLogic(ps_SoapEnv, pc_SOAPIPAddress, SOAP_ACTION, NULL, &Response);
		if ((i_Error == 0) && (Response.iReturnValue == 0))
		{			
			printf ("soap_call_MSXE371x__IncCounterDisableAndReleaseCompareLogic OK\n");			
		} 
		else
		{
			printf("soap_call_MSXE371x__IncCounterDisableAndReleaseCompareLogic SOAP error: %d, Response error: %d\n", i_Error, Response.iReturnValue);
			if (i_Error != 0)
				printf("SOAP Error : %s\n",*soap_faultstring(ps_SoapEnv));
		}	

	
	return 0;
}

//-------------------------------------------------------------------------------------------------------

/** Release counter.
 * 
 * @retval 0:	No error.
 * @retval 1:	Error.
 */
int i_SAMPLE_ReleaseCounter (struct soap *ps_SoapEnv, unsigned char *pc_SOAPIPAddress)
{
	struct MSXE371x__Response Response;
	int i_Error = 0;

		i_Error = soap_call_MSXE371x__IncCounterRelease (ps_SoapEnv, pc_SOAPIPAddress, SOAP_ACTION, NULL,&Response); 
		if ((i_Error == 0) && (Response.iReturnValue == 0))
		{			
			printf ("soap_call_MSXE371x__IncCounterRelease OK\n");			
		} 
		else
		{
			printf("soap_call_MSXE371x__IncCounterRelease SOAP error: %d, Response error: %d\n",i_Error,Response.iReturnValue);
		    if (i_Error != 0)
		        printf("SOAP Error : %s\n",*soap_faultstring(ps_SoapEnv));
			return 1;
		}
	
	return 0;
}



void GetSequenceParameters(DWORD dw_NbrOfChannelsMax, 
                           DWORD *pdw_NbrOfChannels, 
                           struct MSXE371x__unsignedLong8FixedArray *pdw_ChannelList, 
                           DWORD *pdw_NbrOfSequence, 
                           DWORD *pdw_NbrMaxSequenceToTransfer, 
                           DWORD *pdw_DelayMode, 
                           DWORD *pdw_DelayTimeUnit, 
                           DWORD *pdw_DelayTime,
	                       DWORD *pdw_TriggerMask, 
			               DWORD *pdw_TriggerMode, 
			               DWORD *pdw_HardwareTriggerEdge, 
                           DWORD *pdw_HardwareTriggerCount, 
                           DWORD *pdw_ByTriggerNbrOfSeqToAcquire, 
                           DWORD *pdw_DataFormat)
{
	unsigned int i;
	DWORD dw_Choice = 0;

	do
	{
		printf("\nEnter the number of channels in sequence: ");
		scanf("%d", pdw_NbrOfChannels);
	}
	while ((*pdw_NbrOfChannels == 0) || (*pdw_NbrOfChannels > dw_NbrOfChannelsMax));

	for (i = 0; i < *pdw_NbrOfChannels; i++)
	{
		do
		{
			printf("\nEnter channel of sequence index %d: ", i);
			scanf("%lu", &pdw_ChannelList->ulValue[i]);
		}
		while (pdw_ChannelList->ulValue[i] > dw_NbrOfChannelsMax);
	}

	do
	{
		printf("\nEnter the number of sequences to acquire (0 = continuous): ");
		scanf("%d", pdw_NbrOfSequence);
	}
	while ((*pdw_NbrOfSequence < 0) || (*pdw_NbrOfSequence > 65535));

	do
	{
		printf("\nEnter the number maximum of sequences to transfer: ");
		scanf("%d", pdw_NbrMaxSequenceToTransfer);
	}
	while ((*pdw_NbrMaxSequenceToTransfer < 1) || (*pdw_NbrMaxSequenceToTransfer > 65535));

//	Delay
	do
	{
		printf("\nDo You want to use the delay mode (0 or 1)? ");
		scanf("%d", pdw_DelayMode);
	}
	while ((*pdw_DelayMode != 0) && (*pdw_DelayMode != 1));

	if (*pdw_DelayMode != 0)
	{
		do
		{
			printf("Select the delay timebase You want to use (0 = Miliseconds, 1 = Seconds): ");
			scanf("%d", pdw_DelayTimeUnit);
		}
		while ((*pdw_DelayTimeUnit != 0) && (*pdw_DelayTimeUnit != 1));

		do
		{
			printf("\nEnter the delay value: ");
			scanf("%d", pdw_DelayTime );
		}
		while ((*pdw_DelayTime  < 0) || (*pdw_DelayTime  > 65535));
	}
	else
	{
		*pdw_DelayTimeUnit = 0;
		*pdw_DelayTime = 0;
	}
	/* TRIGGER SELECTION */
	*pdw_TriggerMask = 0;
	*pdw_TriggerMode = 0;
	*pdw_HardwareTriggerEdge = 0;
	*pdw_HardwareTriggerCount = 0;
	*pdw_ByTriggerNbrOfSeqToAcquire = 0;

    printf("\nDo You want to use the hardware trigger (0 or 1)? ");
	scanf("%d", &dw_Choice);

	if (dw_Choice == 1)
	{
        *pdw_TriggerMask = *pdw_TriggerMask | 1;

		do 
		{
			printf("\n1. rising edge\n");
			printf("2. falling edge\n");
			printf("3. both edges\n");
			printf("Select the active edge: ");
			scanf ("%d", &dw_Choice);
		}
		while ((dw_Choice < 1) || (dw_Choice > 3));

		*pdw_HardwareTriggerEdge = dw_Choice;

		do
		{
			printf("\nEnter the number of edges before the trigger action occure (1 to 65535): ");
			scanf("%d",pdw_HardwareTriggerCount);
		}
		while ((*pdw_HardwareTriggerCount == 0) || (*pdw_HardwareTriggerCount > 65535));


	}

	if (*pdw_TriggerMask == 0)
	{
	    printf("\nDo You want to use the Synchro trigger (0 or 1)? ");
		scanf("%d", &dw_Choice);

		if (dw_Choice == 1)
		{
			*pdw_TriggerMask = *pdw_TriggerMask | 2;

		}
	}

	if (*pdw_TriggerMask == 0)
	{
	    printf("\nDo You want to use the incremental counter compare event trigger (0 or 1)? ");
		scanf("%d", &dw_Choice);

		if (dw_Choice == 1)
		{
			*pdw_TriggerMask = *pdw_TriggerMask | 4;

		}
	}


    if (*pdw_TriggerMask != 0)
    {
		do
		{
			printf("\n1. One shot trigger\n");
			printf("2. Sequence trigger\n");
			printf("Select the trigger mode do You want to use: ");
			scanf("%d", pdw_TriggerMode);
		}
		while ((*pdw_TriggerMode != 1) && (*pdw_TriggerMode != 2));


    	if (*pdw_TriggerMode == 2)
        {
            do
	    	{
		    	printf("\nEnter the number of sequence to acquire by each trigger: ");
			    scanf("%d", pdw_ByTriggerNbrOfSeqToAcquire);
		    }
		    while (*pdw_ByTriggerNbrOfSeqToAcquire == 0);
        }


    }


// Data format
	*pdw_DataFormat = 0;

	do 
	{
	    printf("\nDo You want to receive a time stamp (0: No, 1: Yes) ?");
	    scanf ("%d",&dw_Choice);
	}
	while (dw_Choice > 1);

	*pdw_DataFormat = dw_Choice;

	do 
	{
	    printf("\nDo You want to receive a Sequence counter (0: No, 1: Yes) ?");
	    scanf ("%d",&dw_Choice);
	}
	while (dw_Choice > 1);

	*pdw_DataFormat = *pdw_DataFormat | (dw_Choice << 1);

	do 
	{
	    printf("\nDo You want to receive the value in digital form (tape 0) or in analog form (type 1) ?");
	    scanf ("%d",&dw_Choice);
	}
	while (dw_Choice > 1);

	*pdw_DataFormat = *pdw_DataFormat | (dw_Choice << 2);


	do 
	{
	    printf("\nDo You want to invert the value sign (0: No, 1: Yes) ?");
	    scanf ("%d",&dw_Choice);
	}
	while (dw_Choice > 1);

	*pdw_DataFormat = *pdw_DataFormat | (dw_Choice << 3);


	do 
	{
	    printf("\nDo You want to receive the external temperature value (0: No, 1: Yes) ?");
	    scanf ("%d",&dw_Choice);
	}
	while (dw_Choice > 1);

	*pdw_DataFormat = *pdw_DataFormat | (dw_Choice << 4);
	

	do 
	{
	    printf("\nDo You want to receive the incremental counter value (0: No, 1: Yes) ?");
	    scanf ("%d",&dw_Choice);
	}
	while (dw_Choice > 1);

	*pdw_DataFormat = *pdw_DataFormat | (dw_Choice << 5);
	
	do 
	{
	    printf("\nDo You want to receive the digital trigger information (0: No, 1: Yes) ?");
	    scanf ("%d",&dw_Choice);
	}
	while (dw_Choice > 1);

	*pdw_DataFormat = *pdw_DataFormat | (dw_Choice << 7);

	do 
	{
	    printf("\nDo You want to receive the index signal information (0: No, 1: Yes) ?");
	    scanf ("%d",&dw_Choice);
	}
	while (dw_Choice > 1);

	*pdw_DataFormat = *pdw_DataFormat | (dw_Choice << 8);

}


#define MAX_SOKET_DATA_FIFO 100

typedef struct
      {
      BYTE      b_Exit;
      INT       i_Handle;
      DWORD    dw_DataFormat;
      DWORD    dw_ExtTemperatureFormat;
      DWORD    dw_NumberOfChannelsToAcquire;
	  DWORD    dw_NbrMaxSequenceToTransfer;
	  DWORD    dw_SingelSequenceSize;
	  DWORD    dw_SocketDataSize;
      SOCKET    s_ClientSocket;
	  HANDLE    h_EventHandle;
	  struct
	     {
	     BYTE  *  pb_SocketData [MAX_SOKET_DATA_FIFO];
		 DWORD    dw_ReadPointer;
		 DWORD    dw_WritePointer;
		 BYTE      b_Overflow;
		 }s_SocketDataFIFO;
      }str_ThreadInfo;



DWORD WINAPI dw_ReadSocketThreadProc (LPVOID lpParameter)
        {
        str_ThreadInfo *  ps_ThreadInfo;
		INT                i_ReturnValue        = 0;
    	DWORD             dw_RequestValue       = 0;
    	DWORD             dw_NbrOfReceiveValues = 0;
        
        /**************************
        * Variable initialisation 
        ***************************/
        ps_ThreadInfo = (str_ThreadInfo *) lpParameter;
        
        do
        {
	        /************************************
		     * Test if no internal FIFO overflow 
			 ************************************/

		    if (((ps_ThreadInfo->s_SocketDataFIFO.dw_WritePointer + 1) % MAX_SOKET_DATA_FIFO) != ps_ThreadInfo->s_SocketDataFIFO.dw_ReadPointer)
			{
				/* Calculate the read data size */
				dw_RequestValue         = ps_ThreadInfo->dw_SocketDataSize;
				dw_NbrOfReceiveValues   = 0;

				do
				{
					i_ReturnValue = recv (ps_ThreadInfo->s_ClientSocket, &ps_ThreadInfo->s_SocketDataFIFO.pb_SocketData[ps_ThreadInfo->s_SocketDataFIFO.dw_WritePointer][dw_NbrOfReceiveValues], dw_RequestValue, 0); 

					if (i_ReturnValue != SOCKET_ERROR)
					{
						dw_NbrOfReceiveValues = dw_NbrOfReceiveValues + i_ReturnValue;
						dw_RequestValue = dw_RequestValue - i_ReturnValue;
					}
				}
				while ((dw_NbrOfReceiveValues < ps_ThreadInfo->dw_SocketDataSize) && (i_ReturnValue != SOCKET_ERROR));


				if (i_ReturnValue != SOCKET_ERROR)
				{  
					/************************************
					 * Increment the write FIFO pointer *
					 ************************************/

					ps_ThreadInfo->s_SocketDataFIFO.dw_WritePointer = (ps_ThreadInfo->s_SocketDataFIFO.dw_WritePointer + 1) % MAX_SOKET_DATA_FIFO;

					/**************************
					 * Set the reeceive event *
					 **************************/
					SetEvent (ps_ThreadInfo->h_EventHandle);
				}
				else
				{
					printf("Socket Error %d\n", WSAGetLastError());
        			ps_ThreadInfo->b_Exit = 1;
					SetEvent (ps_ThreadInfo->h_EventHandle);
				}
			}
			else
			{
				printf ("\nInternal sample FIFO overflow. This stop the sample\n");
				ps_ThreadInfo->b_Exit = 1;
				ps_ThreadInfo->s_SocketDataFIFO.b_Overflow = 1;
				SetEvent (ps_ThreadInfo->h_EventHandle);
			}

        } 
        while (ps_ThreadInfo->b_Exit == 0);


        /*********************
        * Return from thread 
        **********************/
        return (0);

        }


DWORD WINAPI dw_SocketDataManagementThreadProc (LPVOID lpParameter)
        {
        str_ThreadInfo *  ps_ThreadInfo;
		INT                i_ReturnValue        = 0;
    	DWORD             dw_Cpt                = 0;
		DWORD             dw_SequenceCounter    = 0;
        CHAR               c_Write   [300];
        DWORD             *pdw_Data = NULL;
        float             *pf_Data  = NULL;
        
        /**************************
        * Variable initialisation 
        ***************************/
        ps_ThreadInfo = (str_ThreadInfo *) lpParameter;
        
        do
        {

			WaitForSingleObject (ps_ThreadInfo->h_EventHandle, INFINITE);

			while ((ps_ThreadInfo->s_SocketDataFIFO.dw_ReadPointer != ps_ThreadInfo->s_SocketDataFIFO.dw_WritePointer) && (ps_ThreadInfo->b_Exit != 2))
			{
				(BYTE *) pdw_Data = ps_ThreadInfo->s_SocketDataFIFO.pb_SocketData[ps_ThreadInfo->s_SocketDataFIFO.dw_ReadPointer];
				(BYTE *) pf_Data  = ps_ThreadInfo->s_SocketDataFIFO.pb_SocketData[ps_ThreadInfo->s_SocketDataFIFO.dw_ReadPointer];

				for (dw_SequenceCounter = 0;
				     (dw_SequenceCounter < ps_ThreadInfo->dw_NbrMaxSequenceToTransfer) && (ps_ThreadInfo->b_Exit != 2);
					 dw_SequenceCounter ++)
				{

					sprintf(c_Write,"");

					/* Test if the data format time stamp is activated */
					if (ps_ThreadInfo->dw_DataFormat & 1)
					{
						sprintf(c_Write,"%s %s",c_Write, ConvertTimeStamp (pdw_Data [0],pdw_Data [1]));
						pdw_Data = pdw_Data + 2;
						pf_Data  = pf_Data  + 2;
					}

					/* Test if the data format Sequence counter is activated */
					if (ps_ThreadInfo->dw_DataFormat & 2)
					{
						sprintf(c_Write,"%s Sequence counter = %lu ",c_Write, *pdw_Data);
						pdw_Data = pdw_Data + 1;
						pf_Data  = pf_Data  + 1;
					}


					/* Test if the data format external temperature is activated */
					if (ps_ThreadInfo->dw_DataFormat & 0x10)
					{
				        if (ps_ThreadInfo->dw_ExtTemperatureFormat == 0)
					       sprintf(c_Write,"%sExt Temp         : %lu mOhms ",c_Write, *pdw_Data);
						else if (ps_ThreadInfo->dw_ExtTemperatureFormat == 1)
	                       sprintf(c_Write,"%sExt Temp         : %f degree celsius ", c_Write, *pf_Data);
		                else 
			               sprintf(c_Write,"%sExt Temp         : %f degree fahrenheit ", c_Write, *pf_Data);

						pdw_Data = pdw_Data + 1;
						pf_Data  = pf_Data  + 1;
					}

					/* Test if the data format incremental counter is activated */
					if (ps_ThreadInfo->dw_DataFormat & 0x20)
					{
						sprintf(c_Write,"%sInc Counter      : %lu ",c_Write, *pdw_Data);

						pdw_Data = pdw_Data + 1;
						pf_Data  = pf_Data  + 1;
					}

					/* Test if the data format digital status is activated */
					if (ps_ThreadInfo->dw_DataFormat & 0x180)
					{
						sprintf(c_Write,"%sHardware trigger status      : 0x%lx ",c_Write, *pdw_Data & 0x7);
						sprintf(c_Write,"%sIndex(C) status      : 0x%lx ",c_Write, (*pdw_Data >> 4)& 0x7);

						pdw_Data = pdw_Data + 1;
						pf_Data  = pf_Data  + 1;
					}



					if (ps_ThreadInfo->dw_DataFormat & 4)
					{
						sprintf(c_Write,"%sTransducers (mm) : ", c_Write);

						for (dw_Cpt = 0; dw_Cpt < ps_ThreadInfo->dw_NumberOfChannelsToAcquire; dw_Cpt ++)
						{ 
							sprintf(c_Write,"%s%lu:%f ",c_Write,dw_Cpt,*pf_Data);
							pdw_Data = pdw_Data + 1;
							pf_Data  = pf_Data  + 1;
						}
					}
					else
					{
						sprintf(c_Write,"%sTransducers (Hex): ", c_Write);
						for (dw_Cpt = 0; dw_Cpt < ps_ThreadInfo->dw_NumberOfChannelsToAcquire; dw_Cpt ++)
						{ 
							sprintf(c_Write,"%s%lu:%lX ",c_Write,dw_Cpt,*pdw_Data);
							pdw_Data = pdw_Data + 1;
							pf_Data  = pf_Data  + 1;
						}
					}


					sprintf(c_Write,"%s\n", c_Write);
					printf (c_Write);

					/*****************
					 * Test if store *
					 *****************/

					if (ps_ThreadInfo->i_Handle != -1)
					{
						write (ps_ThreadInfo->i_Handle, c_Write, strlen (c_Write));
					}
				}
			
				/***********************************
				 * Increment the read FIFO pointer *
				 ***********************************/

				ps_ThreadInfo->s_SocketDataFIFO.dw_ReadPointer = (ps_ThreadInfo->s_SocketDataFIFO.dw_ReadPointer + 1) % MAX_SOKET_DATA_FIFO;
			}

			if (ps_ThreadInfo->b_Exit)
			{
				ps_ThreadInfo->b_Exit = 2;
			}
        } 
        while (ps_ThreadInfo->b_Exit == 0);

        /*********************
        * Return from thread 
        **********************/
        return (0);

        }




void main(void)
{
	CHAR                                            c_IPAddress    [50];
	CHAR                                            c_IPAddressTemp[50];
	CHAR                                            c_SOAPIPAddress[50];
	INT												i_Error                      = 0;
    INT												i_Handle                     = -1;
	INT												i_CurrentPriority            = 0;
	DWORD                                          dw_Dummy                      = 0;
	DWORD                                          dw_Choice                     = 0;
	DWORD                                          dw_NbrOfChannelsMax           = 8;
	DWORD                                          dw_SockedBufferCounter        = 0;
	xsd__unsignedLong                              dw_TransducerSelection        = 0;
	xsd__unsignedLong                              dw_NbrOfChannels              = 0;
	xsd__unsignedLong                              dw_ConversionTimeUnit         = 0;
	xsd__unsignedLong                              dw_ConversionTimeValue        = 0;
	xsd__unsignedLong                              dw_NbrOfSequence              = 0;
	xsd__unsignedLong                              dw_NbrMaxSequenceToTransfer   = 0;
	xsd__unsignedLong                              dw_DelayMode                  = 0;
	xsd__unsignedLong                              dw_DelayTimeUnit              = 0;
	xsd__unsignedLong                              dw_DelayValue                 = 0;
	xsd__unsignedLong                              dw_TriggerMask                = 0;
	xsd__unsignedLong                              dw_TriggerMode                = 0;
	xsd__unsignedLong                              dw_HardwareTriggerEdge        = 0;
	xsd__unsignedLong                              dw_HardwareTriggerCount       = 0;
	xsd__unsignedLong                              dw_ByTriggerNbrOfSeqToAcquire = 0;
	xsd__unsignedLong                              dw_DataFormat                 = 0;
    xsd__unsignedLong                              dw_TemperatureFormat          = 0;
	DWORD										   dw_WantDigitalStatus          = 0;
	struct MXCommon__ByteArrayResponse              s_ByteArrayResponse;
	struct MSXE371x__Response                      Response;
	struct MSXE371x__unsignedLong8FixedArray       dw_ChannelList;
	struct soap                                   *ps_SoapEnv = NULL;
	SOCKET                                          s_ClientSocket;

	
	DWORD	 dw_Cpt					      = 0;
    INT		  i_key		  			      = 0;
	INT		  i_ReturnValue               = 0;
    DWORD    dw_ThreadId                  = 0;
    HANDLE   h_ReadSocketThreadHandle     = 0;
    HANDLE   h_DataManagementThreadHandle = 0;
    str_ThreadInfo   s_ThreadInfo;

	/***************************/
    /* Variable initialisation */
	/***************************/

	memset (&s_ThreadInfo.s_SocketDataFIFO, 0, sizeof (s_ThreadInfo.s_SocketDataFIFO));


	/**************************/
	/* Get the console handle */
	/**************************/
	ShowWindow(GetForegroundWindow (), SW_SHOWMAXIMIZED);

	/****************************/
	/* Print sample information */
	/****************************/
	printf("+---------------------------------------------------+\n");
	printf("| MSX-E371x                                         |\n");
	printf("| Transducer sequence Test Program                  |\n");
	printf("+---------------------------------------------------+\n");
	printf("| <ESC> exit sample                                 |\n");
	printf("+---------------------------------------------------+\n");
	printf("\n");

	/* Initialise the SOAP connection */
	if (i_SOAP_Init(&ps_SoapEnv) == 0)
	{
		ps_SoapEnv->send_timeout    = 5;
		ps_SoapEnv->recv_timeout    = 5;
		ps_SoapEnv->accept_timeout  = 5;
		ps_SoapEnv->connect_timeout = 1;

		printf("Enter the module IP Address (XXX.XXX.XXX.XXX): ");
		gets(c_IPAddressTemp);

		v_UTILS_ClearNullInIPAddress(c_IPAddressTemp,c_IPAddress);
		sprintf(c_SOAPIPAddress,"%s:5555", c_IPAddress);
		printf("SOAP IP Address: %s\n", c_SOAPIPAddress);

		i_Error = soap_call_MXCommon__GetModuleType(ps_SoapEnv,c_SOAPIPAddress , SOAP_ACTION, &dw_Dummy, &s_ByteArrayResponse);
		if ((i_Error == 0) && (s_ByteArrayResponse.sResponse.iReturnValue == 0))
		{
			printf("Module Type: %s\n", s_ByteArrayResponse.sArray.__ptr);
   			/* Get Transducer type selection */
			if(i_DisplayAndSelectTransducerType(ps_SoapEnv, c_SOAPIPAddress, &dw_TransducerSelection) == 0)
			{


			    /* Get the auto sequence initialisation parameter */
			    GetSequenceParameters(	dw_NbrOfChannelsMax,
										    &dw_NbrOfChannels,
										    &dw_ChannelList,
										    &dw_NbrOfSequence,
										    &dw_NbrMaxSequenceToTransfer,
										    &dw_DelayMode,
										    &dw_DelayTimeUnit,
										    &dw_DelayValue,
										    &dw_TriggerMask,
										    &dw_TriggerMode,
										    &dw_HardwareTriggerEdge,
										    &dw_HardwareTriggerCount,
										    &dw_ByTriggerNbrOfSeqToAcquire,
                                            &dw_DataFormat);

			    s_ThreadInfo.b_Exit                       = 0;
                s_ThreadInfo.dw_DataFormat                = dw_DataFormat;
                s_ThreadInfo.dw_NumberOfChannelsToAcquire = dw_NbrOfChannels;
			    s_ThreadInfo.dw_NbrMaxSequenceToTransfer  = dw_NbrMaxSequenceToTransfer;

			    /**********************************************/
			    /* Calculate the reading the socket data size */
			    /**********************************************/
				if (dw_DataFormat & 0x180)
					dw_WantDigitalStatus = 1;

			    s_ThreadInfo.dw_SingelSequenceSize = (dw_NbrOfChannels + ((dw_DataFormat & 1) * 2) + ((dw_DataFormat & 2) >> 1) + ((dw_DataFormat & 16) >> 4) + ((dw_DataFormat & 32) >> 5) + dw_WantDigitalStatus) * sizeof (DWORD);
			    s_ThreadInfo.dw_SocketDataSize     = s_ThreadInfo.dw_SingelSequenceSize * dw_NbrMaxSequenceToTransfer;

                // Test if want to receive the external temperature value
                if (dw_DataFormat & 16)
                {
                    i_Error = i_SAMPLE_ExtTemperatureInitialization(ps_SoapEnv,c_SOAPIPAddress,&dw_TemperatureFormat);
                    s_ThreadInfo.dw_ExtTemperatureFormat = dw_TemperatureFormat;
                }

                // Test if want to receive the incremental counter value or use the compare event as trigger
                if ((i_Error == 0) && ((dw_DataFormat & 32) || (dw_TriggerMask & 4) ))
                {
                    i_Error = i_SAMPLE_CounterInitialization (ps_SoapEnv, c_SOAPIPAddress);
                }

                if ((i_Error == 0) && (dw_TriggerMask & 4) )
                {
                    i_Error = i_SAMPLE_CounterCompareInitialization (ps_SoapEnv, c_SOAPIPAddress);
                }
                
                if (i_Error == 0)
                {

											    
			        /* Initialise the socket to receive the data */
			        if (i_Socket_ClientOpenEx (c_IPAddress, 8989, s_ThreadInfo.dw_SocketDataSize * MAX_SOKET_DATA_FIFO, &s_ClientSocket) == 0)
			        {
				        do 
				        {
					        printf("\nDo You want to store the sequence data (0:No, 1:Yes) ?");
					        scanf ("%d",&dw_Choice);
				        }
				        while (dw_Choice > 1);

				        /* Test if store */

				        if (dw_Choice)
				        {
					        i_Handle = open ("Sequence.txt", O_CREAT | O_TRUNC | O_RDWR,S_IREAD | S_IWRITE);
				        }

                        if (((i_Handle != -1) && (dw_Choice == 1)) || (dw_Choice == 0))
                        {
					        /*************************************************/
					        /* Memory allocation for reading the socket data */
					        /*************************************************/

					        for (dw_SockedBufferCounter = 0; dw_SockedBufferCounter < MAX_SOKET_DATA_FIFO; dw_SockedBufferCounter ++)
					        {
						        if ((s_ThreadInfo.s_SocketDataFIFO.pb_SocketData[dw_SockedBufferCounter] = malloc (s_ThreadInfo.dw_SocketDataSize)) == NULL)
						        {
							        break;
						        }
					        }


					        if (dw_SockedBufferCounter == MAX_SOKET_DATA_FIFO)
					        {
						        /**************************/
						        /* Save the client socket */
						        /**************************/

			                    s_ThreadInfo.s_ClientSocket = s_ClientSocket;
                                s_ThreadInfo.i_Handle       = i_Handle;

						        /********************************
						         * Thread synchronisation event *
						         ********************************/

						        s_ThreadInfo.h_EventHandle = CreateEvent (NULL, FALSE, FALSE, "MSX-E371x sequence");

						        /**************************************
						         * Create the read socked data thread *
						         **************************************/

						        h_ReadSocketThreadHandle = CreateThread (NULL,
						 	        		                             0,
									   	                                 dw_ReadSocketThreadProc,
										                                 &s_ThreadInfo,
										                                 0,
										                                 &dw_ThreadId);

						        /*************************************************************************************************/
						        /* Change the process priority to the maximum for the childs to have maximum OS allowed priority */
						        /*************************************************************************************************/

						        i_CurrentPriority = GetThreadPriority (GetCurrentThread());
						        SetThreadPriority (GetCurrentThread(), THREAD_PRIORITY_TIME_CRITICAL);

						        SetThreadPriority (h_ReadSocketThreadHandle, THREAD_PRIORITY_TIME_CRITICAL);

						        /********************************/
						        /* Restore the process priority */
						        /********************************/
						        
						        SetThreadPriority (GetCurrentThread(), i_CurrentPriority);

					            Sleep (100);


						        /********************************************
						         * Create the socked data management thread *
						         ********************************************/

						        h_DataManagementThreadHandle = CreateThread (NULL,
						 	        		                                 0,
									   	                                     dw_SocketDataManagementThreadProc,
										                                     &s_ThreadInfo,
										                                     0,
										                                     &dw_ThreadId);

						        SetThreadPriority (h_DataManagementThreadHandle, THREAD_PRIORITY_TIME_CRITICAL);


					            /* Start the auto refresh */
					            i_Error = soap_call_MSXE371x__TransducerInitAndStartSequence(					
								            ps_SoapEnv,
								            c_SOAPIPAddress,
								            SOAP_ACTION,
                                            dw_TransducerSelection, 
								            dw_NbrOfChannels,
								            &dw_ChannelList,
								            dw_NbrOfSequence,
								            dw_NbrMaxSequenceToTransfer,
								            dw_DelayMode,
								            dw_DelayTimeUnit,
								            dw_DelayValue,
								            dw_TriggerMask,
								            dw_TriggerMode,
								            dw_HardwareTriggerEdge,
								            dw_HardwareTriggerCount,
								            dw_ByTriggerNbrOfSeqToAcquire,
								            dw_DataFormat,
								            0,
								            0,
								            0,
								            0,
								            &Response);

					            if ((i_Error == 0) && (Response.iReturnValue == 0))
					            {
									/* 
									 * gSOAP doesn't release automatically the memory that 
									 * it allocates to deserialize data, that's why both 
									 * following functions have to be used to release the 
									 * allocated memory.
									 */
									soap_destroy(ps_SoapEnv);
									soap_end(ps_SoapEnv);
									
	                                printf("\n Press ESC to exit the sample...\n");

	                                do
		                            {
						                if (kbhit()) 
						                {
							                if (getch() == 27)
									        {
										        s_ThreadInfo.b_Exit = 2;
									        }
						                }
								        else
								        {
									        Sleep (100);
								        }
	                                }
		                            while (s_ThreadInfo.b_Exit != 2);

							        Sleep (100);

							        /*********************************
							         * Test if internal FIFO overflow
							         *********************************/

							        if (s_ThreadInfo.s_SocketDataFIFO.b_Overflow)
							        {
								        printf ("\nInternal sample FIFO overflow. This have stop the sample\n");
							        }
				            
			                        /******************
			                        * Free the thread 
			                        *******************/

						            i_Error = soap_call_MSXE371x__TransducerStopAndReleaseSequence (ps_SoapEnv, c_SOAPIPAddress, SOAP_ACTION, &dw_Dummy, &Response);
						            if ((i_Error != 0) || (Response.iReturnValue != 0))
	                                {
							            printf ("soap_call_MSXE371x__TransducerStopAndReleaseSequence  SOAP error: %d, Response error: %d\n",i_Error,Response.iReturnValue);
			                            if (i_Error != 0)
				                            printf("SOAP Error : %s\n",*soap_faultstring(ps_SoapEnv));
					                }
					            }
					            else
					            {
						            printf("soap_call_MSXE371x__TransducerInitAndStartSequence SOAP error: %d, Response error: %d\n",i_Error,Response.iReturnValue);
					                if (i_Error != 0)
						                printf("SOAP Error : %s\n",*soap_faultstring(ps_SoapEnv));


						            s_ThreadInfo.b_Exit = 1;
						            Sleep (100);
					            }
                    
					        }
					        else
					        {
						        printf ("Socket memory allocation error\n");
					        }

                        }
                        else
                        {
                            printf ("Sequence.txt file creation error \n");

                        }

				        /* Close the socket connection */
				        i_Socket_ClientClose(s_ClientSocket);
			        }
			        else
			        {
				        printf("i_Socket_ClientOpen Error\n");
			        }
            
                    if (dw_DataFormat & 16)
                    {
                        i_Error = i_SAMPLE_ReleaseExtTemperature(ps_SoapEnv,c_SOAPIPAddress);
                    }

                    if (dw_TriggerMask & 4) 
                    {
                        i_Error = i_SAMPLE_ReleaseCounterCompare (ps_SoapEnv, c_SOAPIPAddress);
                    }

                    if ((dw_DataFormat & 32) || (dw_TriggerMask & 4) )
                    {
                        i_Error = i_SAMPLE_ReleaseCounter (ps_SoapEnv, c_SOAPIPAddress);
                    }
                }


            }

		}
		else
		{
			printf("soap_call_MXCommon__GetModuleType SOAP error: %d, Response error: %d\n", i_Error,s_ByteArrayResponse.sResponse.iReturnValue);
            if (i_Error != 0)
                 printf("SOAP Error : %s\n",*soap_faultstring(ps_SoapEnv));
		}

	}


	/******************************
	 * Test if SOAP connection open
	 ******************************/

	if (ps_SoapEnv != NULL)
	{
	    /****************************
		 * Close the SOAP connection 
		 ****************************/
		i_SOAP_Close(ps_SoapEnv);
	}

    /********************
     * Test if file open 
     ********************/
	if (i_Handle != -1)
	{
		/*****************
		 * Close the file 
	     *****************/
		close (i_Handle);
	}


    /***************************
     * Test if thread installed 
     ****************************/
    if (h_ReadSocketThreadHandle != 0)
	{
		/*************************
		 * Close the thread handle 
	     **************************/
		CloseHandle (h_ReadSocketThreadHandle);
	}

    /***************************
     * Test if thread installed 
     ****************************/
    if (h_DataManagementThreadHandle != 0)
	{
		/*************************
		 * Close the thread handle 
	     **************************/
		CloseHandle (h_DataManagementThreadHandle);
	}

	/***************************
     * Test if thread installed 
     ****************************/
	if (s_ThreadInfo.h_EventHandle != 0)
	{
		/*************************
		 * Close the event handle 
	     **************************/
		CloseHandle (s_ThreadInfo.h_EventHandle);
	}

	/****************************
	 * Free the allocated memory
	 ****************************/

	for (dw_SockedBufferCounter = 0; dw_SockedBufferCounter < MAX_SOKET_DATA_FIFO; dw_SockedBufferCounter ++)
	{
		/**********************************
		 * Test if socket memory allocated 
		 **********************************/
		
		if (s_ThreadInfo.s_SocketDataFIFO.pb_SocketData[dw_SockedBufferCounter] != NULL)
		{
			free (s_ThreadInfo.s_SocketDataFIFO.pb_SocketData[dw_SockedBufferCounter]);
		}
	}

   
	/**********************************/
	/*Print End programme informations*/
	/**********************************/
	printf("\n");
	printf("+---------------------------------------------------+\n");
	printf("| Program stopped                                   |\n");
	printf("+---------------------------------------------------+\n");
	printf("| Press any key to exit from sample                 |\n");
	printf("+---------------------------------------------------+\n");

	while (!(kbhit()));

}
